@@ -22,8 +22,11 @@
 */
 
 /*
-    SUPPORTED DEVICES	PCI ID
-    nForce2 MCP		0064       
+    SUPPORTED DEVICES		PCI ID
+    nForce2 MCP			0064
+    nForce2 Ultra 400 MCP	0084
+    nForce3 Pro150 MCP		00D4
+    nForce3 250Gb MCP		00E4
 
     This driver supports the 2 SMBuses that are included in the MCP2 of the
     nForce2 chipset.
@@ -43,6 +46,7 @@
 #include <linux/slab.h>
 #include <asm/io.h>
 #include "version.h"
+#include "sensors_compat.h"
 
 /* kernel 2.4.9 needs this */
 #ifndef min_t
@@ -54,7 +58,19 @@ MODULE_AUTHOR ("Hans-Frieder Vogt <hfvogt@arcor.de>");
 MODULE_DESCRIPTION("nForce2 SMBus driver");
 
 #ifndef PCI_DEVICE_ID_NVIDIA_NFORCE2_SMBUS
-#define PCI_DEVICE_ID_NVIDIA_NFORCE2_SMBUS   0x0064
+#define PCI_DEVICE_ID_NVIDIA_NFORCE2_SMBUS	0x0064
+#endif
+
+#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE2S_SMBUS
+#define PCI_DEVICE_ID_NVIDIA_NFORCE2S_SMBUS	0x0084
+#endif
+
+#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE3_SMBUS
+#define PCI_DEVICE_ID_NVIDIA_NFORCE3_SMBUS	0x00D4
+#endif
+
+#ifndef PCI_DEVICE_ID_NVIDIA_NFORCE3S_SMBUS
+#define PCI_DEVICE_ID_NVIDIA_NFORCE3S_SMBUS	0x00E4
 #endif
 
 
@@ -289,10 +305,29 @@ u32 nforce2_func(struct i2c_adapter *adapter)
 	    I2C_FUNC_SMBUS_BLOCK_DATA */;
 }
 
+static void nforce2_inc(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_INC_USE_COUNT;
+#endif
+}
+
+static void nforce2_dec(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_DEC_USE_COUNT;
+#endif
+}
 
 static struct pci_device_id nforce2_ids[] = {
 	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_NFORCE2_SMBUS,
 	       	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_NFORCE2S_SMBUS,
+	       	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_NFORCE3_SMBUS,
+	       	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NVIDIA, PCI_DEVICE_ID_NVIDIA_NFORCE3S_SMBUS,
+	       	PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
 
@@ -315,16 +350,12 @@ static int __devinit nforce2_probe_smb (struct pci_dev *dev, int reg, struct nfo
 		return -1;
 	}
 
-	/* TODO: find a better way to find out whether this file is compiled
-	 * with i2c 2.7.0 of earlier
-	 */
-#ifdef I2C_HW_SMBUS_AMD8111
-	smbus->adapter.owner = THIS_MODULE;
-#endif
 	sprintf(smbus->adapter.name, "SMBus nForce2 adapter at %04x", smbus->base);
 	smbus->adapter.id = I2C_ALGO_SMBUS | I2C_HW_SMBUS_NFORCE2;
 	smbus->adapter.algo = &smbus_algorithm;
 	smbus->adapter.algo_data = smbus;
+	smbus->adapter.inc_use = nforce2_inc;
+	smbus->adapter.dec_use = nforce2_dec;
 
 	error = i2c_add_adapter(&smbus->adapter);
 	if (error) {
