@@ -33,17 +33,13 @@
                  Most (all?) drivers assume two pulses per rev fans
                  and the old scaling was producing double the RPM's
                  Thanks to Jerome Hsiao @ Arima for pointing this out.
+	2004-01-27   Remove use of temporary ID.
+                 Define addresses as a range.
 */
 
 #include <linux/version.h>
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <asm/errno.h>
-#include <asm/io.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
@@ -51,16 +47,12 @@
 #include "sensors_vid.h"
 
 #ifndef I2C_DRIVERID_ADM1026
-/* i2c-id.h hasn't been updated to assign us an ID.
- * So...  Use a local ID from 0xf000 to 0xffff as
- *    documented in i2c-id.h
- */
-#define I2C_DRIVERID_ADM1026  (0xf126)
+#define I2C_DRIVERID_ADM1026	1048
 #endif
 
 /* Addresses to scan */
-static unsigned short normal_i2c[] = { 0x2c, 0x2d, 0x2e, SENSORS_I2C_END };
-static unsigned short normal_i2c_range[] = { SENSORS_I2C_END };
+static unsigned short normal_i2c[] = { SENSORS_I2C_END };
+static unsigned short normal_i2c_range[] = { 0x2c, 0x2e, SENSORS_I2C_END };
 static unsigned int normal_isa[] = { SENSORS_ISA_END };
 static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
 
@@ -306,6 +298,7 @@ static int adm1026_scaling[] = {  /* .001 Volts */
  * allocated, when a new client structure is allocated. */
 
 struct adm1026_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 	enum chips type;
@@ -392,7 +385,6 @@ static void adm1026_afc(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
 static struct i2c_driver adm1026_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "ADM1026 compatible sensor driver",
 	.id		= I2C_DRIVERID_ADM1026,
 	.flags		= I2C_DF_NOTIFY,
@@ -645,14 +637,12 @@ int adm1026_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access adm1026_{read,write}_value. */
 
-	if (!(new_client = kmalloc((sizeof(struct i2c_client)) +
-				   sizeof(struct adm1026_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct adm1026_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct adm1026_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -763,7 +753,8 @@ int adm1026_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					template)) < 0) {
+					template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR2;
 	}
@@ -775,7 +766,7 @@ int adm1026_detect(struct i2c_adapter *adapter, int address,
     ERROR2:
 	i2c_detach_client(new_client);
     ERROR1:
-	kfree(new_client);
+	kfree(data);
     ERROR0:
 	return err;
 }
@@ -794,7 +785,7 @@ int adm1026_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
