@@ -1,7 +1,7 @@
 /*
     adm9240.c - Part of lm_sensors, Linux kernel modules for hardware
              monitoring
-    Copyright (c) 1999  Frodo Looijaard <frodol@dds.nl>
+    Copyright (C) 1999  Frodo Looijaard <frodol@dds.nl>
     and Philip Edelbrock <phil@netroedge.com>
 
     This program is free software; you can redistribute it and/or modify
@@ -49,19 +49,11 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
-#include <asm/io.h>
 #include "version.h"
 
-MODULE_LICENSE("GPL");
-
 /* Addresses to scan */
 static unsigned short normal_i2c[] = { SENSORS_I2C_END };
 static unsigned short normal_i2c_range[] = { 0x2c, 0x2f, SENSORS_I2C_END };
@@ -132,7 +124,7 @@ SENSORS_INSMOD_3(adm9240, ds1780, lm81);
    variants. Note that you should be a bit careful with which arguments
    these macros are called: arguments may be evaluated more than once.
    Fixing this is just not worth it. */
-#define IN_TO_REG(val,nr) (SENSORS_LIMIT(((val) & 0xff),0,255))
+#define IN_TO_REG(val,nr) (SENSORS_LIMIT((val), 0, 255))
 #define IN_FROM_REG(val,nr) (val)
 
 static inline u8 FAN_TO_REG(long rpm, int div)
@@ -147,13 +139,12 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define FAN_FROM_REG(val,div) ((val)==0?-1:\
                                (val)==255?0:1350000/((div)*(val)))
 
-#define TEMP_FROM_REG(temp) \
-   ((temp)<256?((((temp)&0x1fe) >> 1) * 10)      + ((temp) & 1) * 5:  \
-               ((((temp)&0x1fe) >> 1) -255) * 10 - ((temp) & 1) * 5)  \
+#define TEMP_FROM_REG(temp) ((temp)<256 ? (temp) * 5 : \
+                             ((temp) - 512) * 5)
 
-#define TEMP_LIMIT_FROM_REG(val) (((val)>0x80?(val)-0x100:(val))*10)
+#define TEMP_LIMIT_FROM_REG(val) (((val)>=0x80?(val)-0x100:(val))*10)
 
-#define TEMP_LIMIT_TO_REG(val) SENSORS_LIMIT(((val)<0?(((val)-5)/10):\
+#define TEMP_LIMIT_TO_REG(val) SENSORS_LIMIT(((val)<0?(((val)-5)/10)+256:\
                                                       ((val)+5)/10), \
                                              0,255)
 
@@ -165,54 +156,9 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define VID_FROM_REG(val) ((val)==0x1f?0:(val)>=0x10?510-(val)*10:\
                            205-(val)*5)
 
-/* Initial limits */
-#define ADM9240_INIT_IN_0 190
-#define ADM9240_INIT_IN_1 190
-#define ADM9240_INIT_IN_2 190
-#define ADM9240_INIT_IN_3 190
-#define ADM9240_INIT_IN_4 190
-#define ADM9240_INIT_IN_5 190
-
-#define ADM9240_INIT_IN_PERCENTAGE 10
-
-#define ADM9240_INIT_IN_MIN_0 \
-        (ADM9240_INIT_IN_0 - ADM9240_INIT_IN_0 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_0 \
-        (ADM9240_INIT_IN_0 + ADM9240_INIT_IN_0 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MIN_1 \
-        (ADM9240_INIT_IN_1 - ADM9240_INIT_IN_1 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_1 \
-        (ADM9240_INIT_IN_1 + ADM9240_INIT_IN_1 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MIN_2 \
-        (ADM9240_INIT_IN_2 - ADM9240_INIT_IN_2 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_2 \
-        (ADM9240_INIT_IN_2 + ADM9240_INIT_IN_2 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MIN_3 \
-        (ADM9240_INIT_IN_3 - ADM9240_INIT_IN_3 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_3 \
-        (ADM9240_INIT_IN_3 + ADM9240_INIT_IN_3 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MIN_4 \
-        (ADM9240_INIT_IN_4 - ADM9240_INIT_IN_4 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_4 \
-        (ADM9240_INIT_IN_4 + ADM9240_INIT_IN_4 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MIN_5 \
-        (ADM9240_INIT_IN_5 - ADM9240_INIT_IN_5 * ADM9240_INIT_IN_PERCENTAGE / 100)
-#define ADM9240_INIT_IN_MAX_5 \
-        (ADM9240_INIT_IN_5 + ADM9240_INIT_IN_5 * ADM9240_INIT_IN_PERCENTAGE / 100)
-
-#define ADM9240_INIT_FAN_MIN_1 3000
-#define ADM9240_INIT_FAN_MIN_2 3000
-
-#define ADM9240_INIT_TEMP_OS_MAX 600
-#define ADM9240_INIT_TEMP_OS_HYST 500
-#define ADM9240_INIT_TEMP_HOT_MAX 700
-#define ADM9240_INIT_TEMP_HOT_HYST 600
-
-/* For each registered ADM9240, we need to keep some data in memory. That
-   data is pointed to by adm9240_list[NR]->data. The structure itself is
-   dynamically allocated, at the same time when a new adm9240 client is
-   allocated. */
+/* For each registered ADM9240, we need to keep some data in memory. */
 struct adm9240_data {
+	struct i2c_client client;
 	int sysctl_id;
 	enum chips type;
 
@@ -240,9 +186,8 @@ static int adm9240_detect(struct i2c_adapter *adapter, int address,
 			  unsigned short flags, int kind);
 static int adm9240_detach_client(struct i2c_client *client);
 
-static int adm9240_read_value(struct i2c_client *client, u8 register);
-static int adm9240_write_value(struct i2c_client *client, u8 register,
-			       u8 value);
+static int adm9240_read_value(struct i2c_client *client, u8 reg);
+static int adm9240_write_value(struct i2c_client *client, u8 reg, u8 value);
 static void adm9240_update_client(struct i2c_client *client);
 static void adm9240_init_client(struct i2c_client *client);
 
@@ -263,13 +208,9 @@ static void adm9240_analog_out(struct i2c_client *client, int operation,
 static void adm9240_vid(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
 
-/* I choose here for semi-static ADM9240 allocation. Complete dynamic
-   allocation could also be used; the code needed for this would probably
-   take more memory than the datastructure takes now. */
 static int adm9240_id = 0;
 
 static struct i2c_driver adm9240_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "ADM9240 sensor driver",
 	.id		= I2C_DRIVERID_ADM9240,
 	.flags		= I2C_DF_NOTIFY,
@@ -358,16 +299,6 @@ static int adm9240_detect(struct i2c_adapter *adapter, int address,
 	const char *type_name = "";
 	const char *client_name = "";
 
-	/* Make sure we aren't probing the ISA bus!! This is just a safety check
-	   at this moment; i2c_detect really won't call us. */
-#ifdef DEBUG
-	if (i2c_is_isa_adapter(adapter)) {
-		printk
-		    ("adm9240.o: adm9240_detect called for an ISA bus adapter?!?\n");
-		return 0;
-	}
-#endif
-
 	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE_DATA))
 		goto ERROR0;
 
@@ -375,14 +306,13 @@ static int adm9240_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access adm9240_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct adm9240_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct adm9240_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct adm9240_data *) (new_client + 1);
+
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -452,7 +382,8 @@ static int adm9240_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					adm9240_dir_table_template)) < 0) {
+					adm9240_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -469,7 +400,7 @@ static int adm9240_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -487,16 +418,15 @@ static int adm9240_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
-
 }
 
 
 static int adm9240_read_value(struct i2c_client *client, u8 reg)
 {
-	return 0xFF & i2c_smbus_read_byte_data(client, reg);
+	return i2c_smbus_read_byte_data(client, reg);
 }
 
 static int adm9240_write_value(struct i2c_client *client, u8 reg, u8 value)
@@ -504,48 +434,9 @@ static int adm9240_write_value(struct i2c_client *client, u8 reg, u8 value)
 	return i2c_smbus_write_byte_data(client, reg, value);
 }
 
-/* Called when we have found a new ADM9240. It should set limits, etc. */
+/* Called when we have found a new ADM9240. */
 static void adm9240_init_client(struct i2c_client *client)
 {
-	/* Reset all except Watchdog values and last conversion values
-	   This sets fan-divs to 2, among others. This makes most other
-	   initializations unnecessary */
-	adm9240_write_value(client, ADM9240_REG_CONFIG, 0x80);
-
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(0),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_0, 0));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(0),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_0, 0));
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(1),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_1, 1));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(1),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_1, 1));
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(2),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_2, 2));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(2),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_2, 2));
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(3),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_3, 3));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(3),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_3, 3));
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(4),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_4, 4));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(4),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_4, 4));
-	adm9240_write_value(client, ADM9240_REG_IN_MIN(5),
-			    IN_TO_REG(ADM9240_INIT_IN_MIN_5, 5));
-	adm9240_write_value(client, ADM9240_REG_IN_MAX(5),
-			    IN_TO_REG(ADM9240_INIT_IN_MAX_5, 5));
-	adm9240_write_value(client, ADM9240_REG_FAN1_MIN,
-			    FAN_TO_REG(ADM9240_INIT_FAN_MIN_1, 2));
-	adm9240_write_value(client, ADM9240_REG_FAN2_MIN,
-			    FAN_TO_REG(ADM9240_INIT_FAN_MIN_2, 2));
-	adm9240_write_value(client, ADM9240_REG_TOS,
-			    TEMP_LIMIT_TO_REG(ADM9240_INIT_TEMP_OS_MAX));
-	adm9240_write_value(client, ADM9240_REG_THYST,
-			    TEMP_LIMIT_TO_REG(ADM9240_INIT_TEMP_OS_HYST));
-	adm9240_write_value(client, ADM9240_REG_TEMP_CONFIG, 0x00);
-
 	/* Start monitoring */
 	adm9240_write_value(client, ADM9240_REG_CONFIG, 0x01);
 }
@@ -814,6 +705,7 @@ static void __exit sm_adm9240_exit(void)
 
 MODULE_AUTHOR
     ("Frodo Looijaard <frodol@dds.nl> and Philip Edelbrock <phil@netroedge.com>");
+MODULE_LICENSE("GPL");
 MODULE_DESCRIPTION("ADM9240 driver");
 
 module_init(sm_adm9240_init);
