@@ -52,8 +52,8 @@
 #endif
 
 /* I2C addresses to scan */
-static unsigned short normal_i2c[] = { SENSORS_I2C_END };
-static unsigned short normal_i2c_range[] = { 0x28, 0x2f, SENSORS_I2C_END };
+static unsigned short normal_i2c[] = { 0x2d, SENSORS_I2C_END };
+static unsigned short normal_i2c_range[] = { SENSORS_I2C_END };
 
 /* ISA addresses to scan (none) */
 static unsigned int normal_isa[] = { SENSORS_ISA_END };
@@ -244,7 +244,6 @@ static void asb100_pwm(struct i2c_client *client, int operation,
 		int ctl_name, int *nrels_mag, long *results);
 
 static struct i2c_driver asb100_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "asb100",
 	.id		= I2C_DRIVERID_ASB100,
 	.flags		= I2C_DF_NOTIFY,
@@ -548,7 +547,7 @@ static int asb100_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((data->sysctl_id = i2c_register_entry(new_client, "asb100",
-			asb100_dir_table_template)) < 0) {
+			asb100_dir_table_template, THIS_MODULE)) < 0) {
 		err = data->sysctl_id;
 		goto ERROR3;
 	}
