@@ -21,15 +21,16 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/ipmi.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
+/* for kernel thread ... */
+#include <asm/atomic.h>
+#include <asm/semaphore.h>
+#include <linux/smp_lock.h>
+#include <asm/errno.h>
 #include "version.h"
 
 /*
@@ -63,25 +64,24 @@ static int bmcsensors_command(struct i2c_client *client, unsigned int cmd,
 			   void *arg);
 
 static void bmcsensors_update_client(struct i2c_client *client);
-static int bmcsensors_find(int *address);
 static void bmcsensors_reserve_sdr(void);
+static void bmc_do_pause(unsigned int amount); /* YJ for debug */
 
 
 static void bmcsensors_all(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
+#if 0
 static void bmcsensors_alarms(struct i2c_client *client, int operation,
 			   int ctl_name, int *nrels_mag, long *results);
 static void bmcsensors_fan_div(struct i2c_client *client, int operation,
 			    int ctl_name, int *nrels_mag, long *results);
 static void bmcsensors_pwm(struct i2c_client *client, int operation,
 			int ctl_name, int *nrels_mag, long *results);
+#endif
 static void bmcsensors_get_sdr(u16 resid, u16 record, u8 offset);
 static void bmcsensors_get_reading(struct i2c_client *client, int i);
 
-static int bmcsensors_id = 0;
-
 static struct i2c_driver bmcsensors_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "BMC Sensors driver",
 	.id		= I2C_DRIVERID_BMCSENSORS,
 	.flags		= I2C_DF_NOTIFY,
@@ -102,9 +102,9 @@ struct i2c_client bmc_client = {
 	0
 };
 
-static bmcsensors_initialized;
+static int bmcsensors_initialized;
 
-#define MAX_SDR_ENTRIES 50
+#define MAX_SDR_ENTRIES 100
 #define SDR_LIMITS 8
 #define SDR_MAX_ID_LENGTH 16
 #define SDR_MAX_UNPACKED_ID_LENGTH ((SDR_MAX_ID_LENGTH * 4 / 3) + 2)
@@ -123,7 +123,7 @@ struct sdrdata {
 	u8 k;
 	u8 nominal;
 	u8 limits[SDR_LIMITS];
-	int lim1, lim2;
+	int lim1, lim2;		/* index into limits for reported upper and lower limit */
 	u8 lim1_write, lim2_write;
 	u8 string_type;
 	u8 id_length;
@@ -163,7 +163,8 @@ static u16 nextrecord;
 static int errorcount;
 
 enum states {STATE_INIT, STATE_RESERVE, STATE_SDR, STATE_SDRPARTIAL,
-             STATE_READING, STATE_UNCANCEL, STATE_DONE};
+             STATE_READING, STATE_UNCANCEL, STATE_PROCTABLE, STATE_DONE};
+/* YJ : added extra state STATE_PROCTABLE for thread activity */
 static int state;
 static int receive_counter;
 
@@ -186,10 +187,15 @@ static int receive_counter;
 /* do we really need maximums per-type? */
 #define STYPE_MAX	4		/* the last sensor type we are interested in */
 static u8 bmcs_count[STYPE_MAX + 1];
-static const u8 bmcs_max[STYPE_MAX + 1] = {0, 20, 20, 20, 20};
+static const u8 bmcs_max[STYPE_MAX + 1] = {0, 20, 40, 20, 20};
+/* YJ: on poweredge 1750, we need                 ^^            */
 
 /************************************/
-
+/* YJ ... */
+static int thread_pid= 0;
+static DECLARE_MUTEX_LOCKED(bmc_sem);
+/* ... YJ */
+/************************************/
 
 /* unpack based on string type, convert to normal, null terminate */
 static void ipmi_sprintf(u8 * to, u8 * from, u8 type, u8 length)
@@ -238,69 +244,78 @@ static const char * threshold_text[] = {
 
 /* select two out of the 8 possible readable thresholds, and place indexes into the limits
    array into lim1 and lim2. Set writable flags */
-static void bmcsensors_select_thresholds(int i)
+static void bmcsensors_select_thresholds(struct sdrdata * sd)
 {
-	u8 capab = sdrd[i].capab;
-	u16 mask = sdrd[i].thresh_mask;
+	u8 capab = sd->capab;
+	u16 mask = sd->thresh_mask;
+	int tmp;
 
-	sdrd[i].lim1 = -1;
-	sdrd[i].lim2 = -1;
-	sdrd[i].lim1_write = 0;
-	sdrd[i].lim2_write = 0;
+	sd->lim1 = -1;
+	sd->lim2 = -1;
+	sd->lim1_write = 0;
+	sd->lim2_write = 0;
 
 	if(((capab & 0x0c) == 0x04) ||	/* readable thresholds ? */
 	   ((capab & 0x0c) == 0x08)) {
 		/* select upper threshold */
 		if(mask & 0x10) {			/* upper crit */
-			sdrd[i].lim1 = 1;
+			sd->lim1 = 1;
 			if((capab & 0x0c) == 0x08 && (mask & 0x1000))
-				sdrd[i].lim1_write = 1;
+				sd->lim1_write = 1;
 		}
 		else if(mask & 0x20) {		/* upper non-recov */
-			sdrd[i].lim1 = 0;
+			sd->lim1 = 0;
 			if((capab & 0x0c) == 0x08 && (mask & 0x2000))
-				sdrd[i].lim1_write = 1;
+				sd->lim1_write = 1;
 		}
 		else if(mask & 0x08) {		/* upper non-crit */
-			sdrd[i].lim1 = 2;
+			sd->lim1 = 2;
 			if((capab & 0x0c) == 0x08 && (mask & 0x0800))
-				sdrd[i].lim1_write = 1;
+				sd->lim1_write = 1;
 		}
 
 		/* select lower threshold */
-		if(((capab & 0x30) == 0x10) ||	/* readable hysteresis ? */
-		   ((capab & 0x30) == 0x20))	/* pos hyst */
-			sdrd[i].lim2 = 6;
+		if((((capab & 0x30) == 0x10) ||		/* readable ? */
+		    ((capab & 0x30) == 0x20)) &&	/* pos hyst */
+		   sd->stype == STYPE_TEMP)
+			sd->lim2 = 6;
 		else if(mask & 0x02) {		/* lower crit */
-			sdrd[i].lim2 = 4;
+			sd->lim2 = 4;
 			if((capab & 0x0c) == 0x08 && (mask & 0x0200))
-				sdrd[i].lim2_write = 1;
+				sd->lim2_write = 1;
 		}
 		else if(mask & 0x04) {		/* lower non-recov */
-			sdrd[i].lim2 = 3;
+			sd->lim2 = 3;
 			if((capab & 0x0c) == 0x08 && (mask & 0x0400))
-				sdrd[i].lim2_write = 1;
+				sd->lim2_write = 1;
 		}
 		else if(mask & 0x01) {		/* lower non-crit */
-			sdrd[i].lim2 = 5;
+			sd->lim2 = 5;
 			if((capab & 0x0c) == 0x08 && (mask & 0x0100))
-				sdrd[i].lim2_write = 1;
+				sd->lim2_write = 1;
 		}
 	}
 
-	if(sdrd[i].lim1 >= 0)
-		printk(KERN_INFO "bmcsensors.o: sensor %d: using %s for upper limit\n",
-			i, threshold_text[sdrd[i].lim1]);
+	/* swap lim1/lim2 if m < 0 or function is 1/x (but not both!) */
+	if(m < 0 && sd->linear != 7 || m >= 0 && sd->linear == 7) {
+		tmp = sd->lim1;
+		sd->lim1 = sd->lim2;
+		sd->lim2 = tmp;
+	}
+
+	if(sd->lim1 >= 0)
+		printk(KERN_INFO "bmcsensors.o: using %s for upper limit\n",
+			threshold_text[sd->lim1]);
 #ifdef DEBUG
 	else
-		printk(KERN_INFO "bmcsensors.o: sensor %d: no readable upper limit\n", i);
+		printk(KERN_INFO "bmcsensors.o: no readable upper limit\n");
 #endif
-	if(sdrd[i].lim2 >= 0)
-		printk(KERN_INFO "bmcsensors.o: sensor %d: using %s for lower limit\n",
-			i, threshold_text[sdrd[i].lim2]);
+	if(sd->lim2 >= 0)
+		printk(KERN_INFO "bmcsensors.o: using %s for lower limit\n",
+			threshold_text[sd->lim2]);
 #ifdef DEBUG
 	else
-		printk(KERN_INFO "bmcsensors.o: sensor %d: no readable lower limit\n", i);
+		printk(KERN_INFO "bmcsensors.o: no readable lower limit\n");
 #endif
 }
 
@@ -313,6 +328,8 @@ static void bmcsensors_build_proc_table()
 	int temps = 0, volts = 0, currs = 0, fans = 0;
 	u8 id[SDR_MAX_UNPACKED_ID_LENGTH];
 
+	
+	printk(KERN_INFO "bmcsensors.o: building proc table\n");
 	if(!(bmcsensors_dir_table = kmalloc((sdrd_count + 1) * sizeof(struct ctl_table), GFP_KERNEL))) {
 		printk(KERN_ERR "bmcsensors.o: no memory\n");
 		return; /* do more than this */	/* ^^ add 1 or more for alarms, etc. */
@@ -368,8 +385,8 @@ static void bmcsensors_build_proc_table()
 			printk(KERN_INFO "bmcsensors.o: sensors.conf: label %s \"%s\"\n",
 				bmcsensors_dir_table[i].procname, id);
 		}
-		bmcsensors_select_thresholds(i);
-		if(sdrd[i].linear != 0) {
+		bmcsensors_select_thresholds(sdrd + i);
+		if(sdrd[i].linear != 0 && sdrd[i].linear != 7) {
 			printk(KERN_INFO
 			       "bmcsensors.o: sensor %d: nonlinear function 0x%.2x unsupported, expect bad results\n",
 			       i, sdrd[i].linear);
@@ -391,7 +408,8 @@ static void bmcsensors_build_proc_table()
 	bmcsensors_dir_table[sdrd_count].ctl_name = 0;
 
 	if ((i = i2c_register_entry(&bmc_client, "bmc",
-				    bmcsensors_dir_table)) < 0) {
+				    bmcsensors_dir_table,
+				    THIS_MODULE)) < 0) {
 		printk(KERN_INFO "bmcsensors.o: i2c registration failed.\n");
 		kfree(bmcsensors_dir_table);
 		kfree(bmcsensors_proc_name_pool);
@@ -440,12 +458,11 @@ static int bmcsensors_rcv_reading_msg(struct ipmi_msg *msg)
 static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 {
 	u16 record;
-	int type, length, owner, lun, number, entity, instance, init;
-	int stype, code;
+	int type;
+	int stype;
 	int id_length;
 	int i;
 	int rstate = STATE_SDR;
-	struct ipmi_msg txmsg;
 	unsigned char * data;
 	u8 id[SDR_MAX_UNPACKED_ID_LENGTH];
 
@@ -455,6 +472,7 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 		ipmi_sdr_partial_size /= 2;
 		if(ipmi_sdr_partial_size < 8) {
 			printk(KERN_INFO "bmcsensors.o: IPMI buffers too small, giving up\n");
+			up (&bmc_sem); /* should wait for thread exit ! */
 			return STATE_DONE;
 		}
 #ifdef DEBUG
@@ -486,6 +504,9 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 	}
 
 	nextrecord = (data[2] << 8) | data[1];
+	/* printk(KERN_INFO "bmcsensors.o: nextrecord %d \n", nextrecord); */
+
+
 	type = data[6];
 	if(type == 1 || type == 2) {		/* known SDR type */
 /*
@@ -559,6 +580,8 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 				}
 				bmcs_count[stype]++;
 				sdrd_count++;
+				if (sdrd_count>=MAX_SDR_ENTRIES) nextrecord = 0xffff; /*YJ*/
+
 			}
 		}
 #ifdef DEBUG
@@ -585,16 +608,26 @@ static int bmcsensors_rcv_sdr_msg(struct ipmi_msg *msg, int state)
 		printk(KERN_INFO "bmcsensors.o: Skipping SDR type 0x%x\n", type);
 #endif
 	}
+	if (nextrecord>=6224) {
+	  nextrecord = 0xffff; /*YJ stop sensor scan on poweredge 1750 */
+	}
 			
 	if(nextrecord == 0xFFFF) {
 		if(sdrd_count == 0) {
 			printk(KERN_INFO "bmcsensors.o: No recognized sensors found.\n");
 			/* unregister?? */
+			rstate = STATE_DONE;
+			up (&bmc_sem); /* should wait for thread exit !!! */
 		} else {
-			bmcsensors_build_proc_table();
+  	          /* YJ ...*/
+		  printk(KERN_INFO "bmcsensors.o: all sensors detected\n");
+		  rstate = STATE_PROCTABLE;
+		  /* YJ bmcsensors_build_proc_table() call by thread */
+		  /* ... YJ */
 		}
-		rstate = STATE_DONE;
+
 	} else {
+
 		bmcsensors_get_sdr(0, nextrecord, 0);
 	}
 	return rstate;
@@ -607,7 +640,7 @@ static void bmcsensors_rcv_msg(struct ipmi_msg *msg)
 	switch(state) {
 		case STATE_INIT:
 		case STATE_RESERVE:
-			resid = (((u16)msg->data[2]) << 8) || msg->data[1];
+			resid = (((u16)msg->data[2]) << 8) | msg->data[1];
 #ifdef DEBUG
 			printk(KERN_DEBUG "bmcsensors.o: Got first resid 0x%.4x\n", resid);
 #endif
@@ -618,6 +651,17 @@ static void bmcsensors_rcv_msg(struct ipmi_msg *msg)
 		case STATE_SDR:
 		case STATE_SDRPARTIAL:
 			state = bmcsensors_rcv_sdr_msg(msg, state);
+			/*YJ ...*/
+			if (state==STATE_PROCTABLE){
+
+#ifdef DEBUG
+			  printk(KERN_DEBUG "releasing thread\n");
+#endif
+
+			  up (&bmc_sem);
+			}
+			 /*YJ bmcsensors_build_proc_table() called by thread */ 
+			/* ... YJ */
 			break;
 
 		case STATE_READING:
@@ -625,7 +669,7 @@ static void bmcsensors_rcv_msg(struct ipmi_msg *msg)
 			break;
 
 		case STATE_UNCANCEL:
-			resid = (((u16)msg->data[2]) << 8) || msg->data[1];
+			resid = (((u16)msg->data[2]) << 8) | msg->data[1];
 #ifdef DEBUG
 			printk(KERN_DEBUG "bmcsensors.o: Got new resid 0x%.4x\n", resid);
 #endif
@@ -635,6 +679,7 @@ static void bmcsensors_rcv_msg(struct ipmi_msg *msg)
 			break;
 
 		case STATE_DONE:
+	        case STATE_PROCTABLE:
 			break;
 
 		default:
@@ -653,6 +698,7 @@ static void bmcsensors_msg_handler(struct ipmi_recv_msg *msg,
 			printk(KERN_ERR
 			       "bmcsensors.o: Too many reservations cancelled, giving up\n");
 			state = STATE_DONE;
+			up (&bmc_sem); /* YJ : should make sure thread exited ! */
 		} else {
 #ifdef DEBUG
 			printk(KERN_DEBUG
@@ -661,7 +707,9 @@ static void bmcsensors_msg_handler(struct ipmi_recv_msg *msg,
 			bmcsensors_reserve_sdr();
 			state = STATE_UNCANCEL;
 		}
-	} else if (msg->msg.data[0] != 0 && msg->msg.data[0] != 0xca) {
+	} else if (msg->msg.data[0] != 0 && msg->msg.data[0] != 0xca &&
+	           msg->msg.data[0] != 0xce) {
+	  /* YJ : accept  0xce */
 		printk(KERN_ERR
 		       "bmcsensors.o: Error 0x%x on cmd 0x%x/0x%x; state = %d; probably fatal.\n",
 		       msg->msg.data[0], msg->msg.netfn & 0xfe, msg->msg.cmd, state);
@@ -699,6 +747,7 @@ static void bmcsensors_reserve_sdr(void)
 	tx_message.cmd = IPMI_RESERVE_SDR;
 	tx_message.data_len = 0;
 	tx_message.data = NULL;
+	printk(KERN_INFO "bmcsensors.o: reserve_sdr...\n");
 	bmcsensors_send_message(&tx_message);
 }
 
@@ -737,8 +786,12 @@ static void bmcsensors_get_reading(struct i2c_client *client, int i)
 
 static int bmcsensors_attach_adapter(struct i2c_adapter *adapter)
 {
-	if(adapter->algo->id != I2C_ALGO_IPMI)
+	printk(KERN_INFO "bmcsensors.o: attach_adapter...\n");
+ 
+	if(adapter->algo->id != I2C_ALGO_IPMI){
+	  printk(KERN_INFO "bmcsensors.o: attach_adapter, expected 0x%x, got 0x%x\n", I2C_ALGO_IPMI, adapter->algo->id);
 		return 0;
+	}
 
 	if(bmcsensors_initialized >= 2) {
 		printk(KERN_INFO "bmcsensors.o: Additional IPMI adapter not supported\n");
@@ -848,51 +901,69 @@ static void bmcsensors_update_client(struct i2c_client *client)
 /************* /proc callback helper functions *********/
 
 /* need better way to map from sysctl to sdrd record number */
-static int find_sdrd(int sysctl)
+static struct sdrdata * find_sdrd(int sysctl)
 {
 	int i;
 
 	for(i = 0; i < sdrd_count; i++)
 		if(sdrd[i].sysctl == sysctl)
-			return i;
-	return -1;
+			return sdrd + i;
+	return NULL;
 }
 
 /* IPMI V1.5 Section 30 */
 static const int exps[] = {1, 10, 100, 1000, 10000, 100000, 1000000, 10000000};
 
-static int decplaces(int i)
+/* Return 0 for fan, 2 for temp, 3 for voltage
+   We could make it variable based on the accuracy (= log10(m * 10**k2));
+   this would work for /proc output, however libsensors resolution
+   is statically set in lib/chips.c */
+static int decplaces(struct sdrdata *sd)
 {
-	u8 k2;
-
-	k2 = sdrd[i].k >> 4;
-	if(k2 < 8)
+	switch(sd->stype) {
+	case STYPE_TEMP:
+		return 2;
+	case STYPE_CURR:
+	case STYPE_VOLT:
+		return 3;
+	case STYPE_FAN:
+	default:
 		return 0;
-	else
-		return 16 - k2;
+	}
 }
 
-static long convert_value(u8 value, int i)
+/* convert a raw value to a reading. IMPI V1.5 Section 30 */
+/* 1/x is the only "linearization function" supported */
+static long conv_val(int value, struct sdrdata *sd)
 {
 	u8 k1, k2;
 	long r;
 
-/* fixme signed/unsigned */
-	r = value * sdrd[i].m;
-
-	k1 = sdrd[i].k & 0x0f;
-	k2 = sdrd[i].k >> 4;
+	r = value * sd->m;
+	k1 = sd->k & 0x0f;
+	k2 = sd->k >> 4;
 	if(k1 < 8)
-		r += sdrd[i].b * exps[k1];
+		r += sd->b * exps[k1];
 	else
-		r += sdrd[i].b / exps[16 - k1];
-	if(k2 < 8)
-		r *= exps[k2];
-/*
-	taken care of by nrels_mag
-	else
-		r /= exps[16 - k2];
-*/
+		r += sd->b / exps[16 - k1];
+	r *= exps[decplaces(sd)];
+	if(k2 < 8) {
+		if(sd->linear != 7)
+			r *= exps[k2];
+		else
+			// this will always truncate to 0: r = 1 / (exps[k2] * r);
+			r = 0;
+	} else {
+		if(sd->linear != 7)
+			r /= exps[16 - k2];
+		else {
+			if(r != 0)
+				// 1 / x * 10 ** (-m) == 10 ** m / x
+				r = exps[16 - k2] / r;
+			else
+				r = 0;
+		}
+	}
 	return r;
 }
 
@@ -902,49 +973,59 @@ static long convert_value(u8 value, int i)
 static void bmcsensors_all(struct i2c_client *client, int operation, int ctl_name,
 		 int *nrels_mag, long *results)
 {
-	int i;
+	struct sdrdata *sd;
+/*
 	struct bmcsensors_data *data = client->data;
+*/
 
-	if((i = find_sdrd(ctl_name)) < 0) {
+	if((sd = find_sdrd(ctl_name)) == NULL) {
 		*nrels_mag = 0;
 		return;		
 	}
 	if (operation == SENSORS_PROC_REAL_INFO)
-		*nrels_mag = decplaces(i);
+		*nrels_mag = decplaces(sd);
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		bmcsensors_update_client(client);
-		if(sdrd[i].stype == STYPE_FAN) { /* lower limit only */
-			if(sdrd[i].lim2 >= 0)
-				results[0] = convert_value(sdrd[i].limits[sdrd[i].lim2], i);
-			else
-				results[0] = 0;
-			results[1] = convert_value(sdrd[i].reading, i);
+		if(sd->lim2 >= 0) {
+			if(sd->stype == STYPE_TEMP)   /* upper limit first */
+				results[0] =
+				    conv_val(sd->limits[sd->lim1], sd);
+			else			      /* lower limit first */
+				results[0] =
+				    conv_val(sd->limits[sd->lim2], sd);
+		} else
+			results[0] = 0;
+		if(sd->stype == STYPE_FAN) { /* lower limit only */
+			results[1] = conv_val(sd->reading, sd);
 			*nrels_mag = 2;
 		} else {
-			if(sdrd[i].lim1 >= 0)
-				results[0] = convert_value(sdrd[i].limits[sdrd[i].lim1], i);
-			else
-				results[0] = 0;
-			results[2] = convert_value(sdrd[i].reading, i);
-			if(sdrd[i].lim2 >= 0) {
-				results[1] = convert_value(sdrd[i].limits[sdrd[i].lim2], i);
-				if(sdrd[i].lim2 == 6) /* pos. threshold */
-					results[1] = results[0] - results[1];
+			if(sd->lim1 >= 0) {
+				if(sd->stype == STYPE_TEMP) {	/* lower 2nd */
+					results[1] =
+					   conv_val(sd->limits[sd->lim2], sd);
+					if(sd->lim2 == 6)    /* pos. thresh. */
+						results[1] = results[0] -
+						             results[1];
+				} else				/* upper 2nd */
+					results[1] =
+					   conv_val(sd->limits[sd->lim1], sd);
 			} else
 				results[1] = 0;
+			results[2] = conv_val(sd->reading, sd);
 			*nrels_mag = 3;
 		}
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
+			/* unimplemented */
 		}
 	}
 }
 
+#if 0
 static void bmcsensors_alarms(struct i2c_client *client, int operation, int ctl_name,
 		    int *nrels_mag, long *results)
 {
 	struct bmcsensors_data *data = client->data;
-#if 0
 	if (operation == SENSORS_PROC_REAL_INFO)
 		*nrels_mag = 0;
 	else if (operation == SENSORS_PROC_REAL_READ) {
@@ -952,18 +1033,93 @@ static void bmcsensors_alarms(struct i2c_client *client, int operation, int ctl_
 		results[0] = data->alarms;
 		*nrels_mag = 1;
 	}
-#endif
 }
+#endif
+
+/* YJ ... */
+static int bmc_thread(void *dummy){
+
+  lock_kernel();
+  daemonize();
+  unlock_kernel();
+
+  strcpy(current->comm, "bmc-sensors");
+
+  if(down_interruptible(&bmc_sem)) {
+
+   printk("exiting...");
+
+   thread_pid= 0;
+   up (&bmc_sem);
+
+   return 0;
+  }
+
+  if (state == STATE_PROCTABLE){
+
+    bmcsensors_build_proc_table();
+    
+    state = STATE_DONE;
+    
+    printk(KERN_INFO "bmcsensors.o: bmcsensor thread done\n" );
+    
+  }
+
+  thread_pid= 0;
+ 
+  up (&bmc_sem);
+
+  return 0;
+}
+/* ... YJ */
 
 static int __init sm_bmcsensors_init(void)
 {
 	printk(KERN_INFO "bmcsensors.o version %s (%s)\n", LM_VERSION, LM_DATE);
+	/* YJ ... */
+	init_MUTEX_LOCKED(&bmc_sem);
+
+	thread_pid= kernel_thread(bmc_thread, NULL, 0);
+
+	if (thread_pid<0){
+	  printk(KERN_ERR "bmcsensors.o : Could not initialize bmc thread.  Aborting\n");
+	  return 0;
+	}
+	/* ... YJ */
+
 	return i2c_add_driver(&bmcsensors_driver);
 }
 
 static void __exit sm_bmcsensors_exit(void)
 {
-	i2c_del_driver(&bmcsensors_driver);
+  int j;
+  j= 0;
+  /* YJ ... */
+  printk(KERN_INFO "bmcsensors.o sleeping a while\n");
+  while( (j++ < 50)){
+    bmc_do_pause(HZ / 25);
+  }
+  if (thread_pid > 0){
+    printk(KERN_INFO "bmcsensors.o stopping kernel thread\n");
+    state= STATE_DONE;
+    j= 0;
+    up (&bmc_sem);
+    printk(KERN_INFO "bmcsensors.o waiting...\n");
+    /* make sure kernel thread does not access driver memory any more */
+    while((thread_pid > 0)&& (j++ < 100)){
+      bmc_do_pause(HZ / 25);
+    }
+    printk(KERN_INFO "bmcsensors.o OK\n");
+  }
+  j= 0;
+  /* sleep for debug... not necessary ? */
+  printk(KERN_INFO "bmcsensors.o sleeping again\n");
+  while( (j++ < 50)){
+    bmc_do_pause(HZ / 25);
+  }
+  /* ...YJ */
+  printk(KERN_INFO "bmcsensors.o i2c cleanup\n");
+  i2c_del_driver(&bmcsensors_driver);
 }
 
 
