@@ -60,6 +60,7 @@ SENSORS_INSMOD_1(bt869);
 
 /* Each client has this additional data */
 struct bt869_data {
+	struct i2c_client client;
 	int sysctl_id;
 
 	struct semaphore update_lock;
@@ -102,7 +103,6 @@ static void bt869_update_client(struct i2c_client *client);
 
 /* This is the driver that will be inserted */
 static struct i2c_driver bt869_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "BT869 video-output chip driver",
 	.id		= I2C_DRIVERID_BT869,
 	.flags		= I2C_DF_NOTIFY,
@@ -503,15 +503,12 @@ int bt869_detect(struct i2c_adapter *adapter, int address,
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access bt869_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct bt869_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct bt869_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data =
-	    (struct bt869_data *) (((struct i2c_client *) new_client) + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -551,7 +548,8 @@ int bt869_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					bt869_dir_table_template)) < 0) {
+					bt869_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -567,7 +565,7 @@ int bt869_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -585,7 +583,7 @@ static int bt869_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
