@@ -74,6 +74,7 @@ MODULE_PARM_DESC(checksum, "Only accept eeproms whose checksum is correct");
 
 /* Each client has this additional data */
 struct ddcmon_data {
+	struct i2c_client client;
 	int sysctl_id;
 
 	struct semaphore update_lock;
@@ -116,7 +117,6 @@ static void ddcmon_update_client(struct i2c_client *client);
 
 /* This is the driver that will be inserted */
 static struct i2c_driver ddcmon_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "DDCMON READER",
 	.id		= I2C_DRIVERID_DDCMON,
 	.flags		= I2C_DF_NOTIFY,
@@ -215,26 +215,25 @@ int ddcmon_detect(struct i2c_adapter *adapter, int address,
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access ddcmon_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct ddcmon_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct ddcmon_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct ddcmon_data *) (new_client + 1);
-	memset(data, 0xff, DDCMON_SIZE);
+	new_client = &data->client;
+	memset(data->data, 0xff, DDCMON_SIZE);
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
 	new_client->driver = &ddcmon_driver;
 	new_client->flags = 0;
 
+	/* prevent 24RF08 corruption (just in case) */
+	i2c_smbus_write_quick(new_client, 0);
+
 	/* Now, we do the remaining detection. */
 	if (checksum) {
 		int cs = 0;
-		/* prevent 24RF08 corruption (just in case) */
-		i2c_smbus_write_quick(new_client, 0);
 		for (i = 0; i < 0x80; i++)
 			cs += i2c_smbus_read_byte_data(new_client, i);
 		if ((cs & 0xff) != 0)
@@ -271,7 +270,8 @@ int ddcmon_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					ddcmon_dir_table_template)) < 0) {
+					ddcmon_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -283,7 +283,7 @@ int ddcmon_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -299,7 +299,7 @@ static int ddcmon_detach_client(struct i2c_client *client)
 		    ("ddcmon.o: Client deregistration failed, client not detached.\n");
 		return err;
 	}
-	kfree(client);
+	kfree(client->data);
 	return 0;
 }
 
@@ -318,19 +318,21 @@ static void ddcmon_update_client(struct i2c_client *client)
 			for (i=0; i<DDCMON_SIZE; i+=I2C_SMBUS_I2C_BLOCK_MAX)
 				if (i2c_smbus_read_i2c_block_data(client,
 				                           i, data->data + i)
-				                    != I2C_SMBUS_I2C_BLOCK_MAX)
+				                    != I2C_SMBUS_I2C_BLOCK_MAX) {
+					printk(KERN_WARNING "ddcmon.o: block read fail at 0x%.2x!\n", i);
 					goto DONE;
+				}
 		} else {
 			if (i2c_smbus_write_byte(client, 0)) {
-#ifdef DEBUG
-				printk("ddcmon read start has failed!\n");
-#endif
+				printk(KERN_WARNING "ddcmon.o: read start fail at 0!\n");
 				goto DONE;
 			}
 			for (i = 0; i < DDCMON_SIZE; i++) {
 				j = i2c_smbus_read_byte(client);
-				if (j < 0)
+				if (j < 0) {
+					printk(KERN_WARNING "eeprom.o: read fail at 0x%.2x!\n", i);
 					goto DONE;
+				}
 				data->data[i] = (u8) j;
 			}
 		}
