@@ -163,7 +163,6 @@ static void fscher_wdog(struct i2c_client *client, int operation,
 static int fscher_id = 0;
 
 static struct i2c_driver fscher_driver = {
-  .owner		= THIS_MODULE,
   .name		        = "FSCHER sensor driver",
   .id		        = I2C_DRIVERID_FSCHER,
   .flags		= I2C_DF_NOTIFY,
@@ -308,7 +307,8 @@ int fscher_detect(struct i2c_adapter *adapter, int address,
 
   /* Register a new directory entry with module sensors */
   if ((i = i2c_register_entry(new_client, type_name,
-                              fscher_dir_table_template)) < 0) {
+                              fscher_dir_table_template,
+			      THIS_MODULE)) < 0) {
     err = i;
     goto ERROR4;
   }
