@@ -162,7 +162,6 @@ static void fscpos_wdog(struct i2c_client *client, int operation,
 static int fscpos_id = 0;
 
 static struct i2c_driver fscpos_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "FSCPOS sensor driver",
 	.id		= I2C_DRIVERID_FSCPOS,
 	.flags		= I2C_DF_NOTIFY,
@@ -291,7 +290,8 @@ int fscpos_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					fscpos_dir_table_template)) < 0) {
+					fscpos_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
