@@ -220,7 +220,6 @@ static void fscscy_intrusion(struct i2c_client *client, int operation,
 static int fscscy_id = 0;
 
 static struct i2c_driver fscscy_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "FSCSCY sensor driver",
 	.id		= I2C_DRIVERID_FSCSCY,
 	.flags		= I2C_DF_NOTIFY,
@@ -370,7 +369,8 @@ int fscscy_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					fscscy_dir_table_template)) < 0) {
+					fscscy_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
