@@ -109,40 +109,9 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define BEEPS_TO_REG(val) ((val) & 0x7f)
 #define BEEPS_FROM_REG(val) ((val) & 0x7f)
 
-/* Initial values */
-#define GL518_INIT_TEMP_OVER 600
-#define GL518_INIT_TEMP_HYST 500
-#define GL518_INIT_FAN_MIN_1 3000
-#define GL518_INIT_FAN_MIN_2 3000
-
-/* These are somewhat sane */
-#define GL518_INIT_VIN_1 330	/* 3.3 V */
-#define GL518_INIT_VIN_2 286	/* 12 V */
-#define GL518_INIT_VIN_3 260	/* Vcore */
-#define GL518_INIT_VDD 500	/* 5 V */
-
-#define GL518_INIT_PERCENTAGE 10
-
-#define GL518_INIT_VIN_MIN_1 \
-        (GL518_INIT_VIN_1 - GL518_INIT_VIN_1 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VIN_MAX_1 \
-        (GL518_INIT_VIN_1 + GL518_INIT_VIN_1 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VIN_MIN_2 \
-        (GL518_INIT_VIN_2 - GL518_INIT_VIN_2 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VIN_MAX_2 \
-        (GL518_INIT_VIN_2 + GL518_INIT_VIN_2 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VIN_MIN_3 \
-        (GL518_INIT_VIN_3 - GL518_INIT_VIN_3 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VIN_MAX_3 \
-        (GL518_INIT_VIN_3 + GL518_INIT_VIN_3 * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VDD_MIN \
-        (GL518_INIT_VDD - GL518_INIT_VDD * GL518_INIT_PERCENTAGE / 100)
-#define GL518_INIT_VDD_MAX \
-        (GL518_INIT_VDD + GL518_INIT_VDD * GL518_INIT_PERCENTAGE / 100)
-
-
 /* Each client has this additional data */
 struct gl518_data {
+	struct i2c_client client;
 	int sysctl_id;
 	enum chips type;
 
@@ -179,7 +148,6 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 static void gl518_init_client(struct i2c_client *client);
 static int gl518_detach_client(struct i2c_client *client);
 
-static u16 swap_bytes(u16 val);
 static int gl518_read_value(struct i2c_client *client, u8 reg);
 static int gl518_write_value(struct i2c_client *client, u8 reg, u16 value);
 static void gl518_update_client(struct i2c_client *client);
@@ -206,7 +174,6 @@ static void gl518_iterate(struct i2c_client *client, int operation,
 
 /* This is the driver that will be inserted */
 static struct i2c_driver gl518_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "GL518SM sensor chip driver",
 	.id		= I2C_DRIVERID_GL518,
 	.flags		= I2C_DF_NOTIFY,
@@ -311,14 +278,12 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access gl518_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct gl518_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct gl518_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct gl518_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -391,7 +356,8 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					gl518_dir_table_template)) < 0) {
+					gl518_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -421,7 +387,7 @@ static int gl518_detect(struct i2c_adapter *adapter, int address,
 			gl518_list[i] = NULL;
       ERROR2:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -440,28 +406,6 @@ static void gl518_init_client(struct i2c_client *client)
 	/* Never interrupts */
 	gl518_write_value(client, GL518_REG_MASK, 0x00);
 
-	gl518_write_value(client, GL518_REG_TEMP_HYST,
-			  TEMP_TO_REG(GL518_INIT_TEMP_HYST));
-	gl518_write_value(client, GL518_REG_TEMP_OVER,
-			  TEMP_TO_REG(GL518_INIT_TEMP_OVER));
-	gl518_write_value(client, GL518_REG_MISC, (DIV_TO_REG(2) << 6) |
-			  (DIV_TO_REG(2) << 4));
-	gl518_write_value(client, GL518_REG_FAN_LIMIT,
-			  (FAN_TO_REG(GL518_INIT_FAN_MIN_1, 2) << 8) |
-			  FAN_TO_REG(GL518_INIT_FAN_MIN_2, 2));
-	gl518_write_value(client, GL518_REG_VIN1_LIMIT,
-			  (IN_TO_REG(GL518_INIT_VIN_MAX_1) << 8) |
-			  IN_TO_REG(GL518_INIT_VIN_MIN_1));
-	gl518_write_value(client, GL518_REG_VIN2_LIMIT,
-			  (IN_TO_REG(GL518_INIT_VIN_MAX_2) << 8) |
-			  IN_TO_REG(GL518_INIT_VIN_MIN_2));
-	gl518_write_value(client, GL518_REG_VIN3_LIMIT,
-			  (IN_TO_REG(GL518_INIT_VIN_MAX_3) << 8) |
-			  IN_TO_REG(GL518_INIT_VIN_MIN_3));
-	gl518_write_value(client, GL518_REG_VDD_LIMIT,
-			  (VDD_TO_REG(GL518_INIT_VDD_MAX) << 8) |
-			  VDD_TO_REG(GL518_INIT_VDD_MIN));
-
 	/* Clear status register (bit 5=1), start (bit6=1) */
 	gl518_write_value(client, GL518_REG_CONF, 0x24);
 	gl518_write_value(client, GL518_REG_CONF, 0x44);
@@ -495,24 +439,19 @@ static int gl518_detach_client(struct i2c_client *client)
 		wake_up_interruptible(&data->wq);
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
 
 
-static u16 swap_bytes(u16 val)
-{
-	return (val >> 8) | (val << 8);
-}
-
 /* Registers 0x07 to 0x0c are word-sized, others are byte-sized 
    GL518 uses a high-byte first convention, which is exactly opposite to
    the usual practice. */
 static int gl518_read_value(struct i2c_client *client, u8 reg)
 {
 	if ((reg >= 0x07) && (reg <= 0x0c))
-		return swap_bytes(i2c_smbus_read_word_data(client, reg));
+		return swab16(i2c_smbus_read_word_data(client, reg));
 	else
 		return i2c_smbus_read_byte_data(client, reg);
 }
@@ -523,8 +462,7 @@ static int gl518_read_value(struct i2c_client *client, u8 reg)
 static int gl518_write_value(struct i2c_client *client, u8 reg, u16 value)
 {
 	if ((reg >= 0x07) && (reg <= 0x0c))
-		return i2c_smbus_write_word_data(client, reg,
-						 swap_bytes(value));
+		return i2c_smbus_write_word_data(client, reg, swab16(value));
 	else
 		return i2c_smbus_write_byte_data(client, reg, value);
 }
