@@ -170,7 +170,6 @@ static void gl520_config(struct i2c_client *client, int operation,
 
 /* This is the driver that will be inserted */
 static struct i2c_driver gl520_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "GL520SM sensor chip driver",
 	.id		= I2C_DRIVERID_GL520,
 	.flags		= I2C_DF_NOTIFY,
@@ -332,7 +331,8 @@ static int gl520_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					gl520_dir_table_template)) < 0) {
+					gl520_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
