@@ -157,7 +157,6 @@ static void lm80_fan_div(struct i2c_client *client, int operation,
 static int lm80_id = 0;
 
 static struct i2c_driver lm80_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "LM80 sensor driver",
 	.id		= I2C_DRIVERID_LM80,
 	.flags		= I2C_DF_NOTIFY,
@@ -314,7 +313,8 @@ int lm80_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					lm80_dir_table_template)) < 0) {
+					lm80_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
