@@ -80,11 +80,12 @@ SENSORS_INSMOD_1(lm83);
 
 /*
  * Conversions and various macros
- * The LM83 uses signed 8-bit values.
+ * The LM83 uses signed 8-bit values with LSB = 1 degree Celcius.
  */
 
-#define TEMP_FROM_REG(val)  (val > 127 ? val-256 : val)
-#define TEMP_TO_REG(val)    (val < 0 ? val+256 : val)
+#define TEMP_FROM_REG(val)	(val)
+#define TEMP_TO_REG(val)	((val) <= -50 ? -50 : \
+				 (val) >= 127 ? 127 : (val))
 
 static const u8 LM83_REG_R_TEMP[] = {
 	LM83_REG_R_LOCAL_TEMP,
@@ -128,7 +129,6 @@ static void lm83_alarms(struct i2c_client *client, int operation, int
  */
  
 static struct i2c_driver lm83_driver = {
-	.owner          = THIS_MODULE,
 	.name           = "LM83 sensor driver",
 	.id             = I2C_DRIVERID_LM83,
 	.flags          = I2C_DF_NOTIFY,
@@ -150,7 +150,7 @@ struct lm83_data
 	unsigned long last_updated; /* in jiffies */
 
 	/* registers values */
-	u8 temp[4], temp_high[4], tcrit;
+	s8 temp[4], temp_high[4], tcrit;
 	u16 alarms; /* bitvector, combined */
 };
 
@@ -276,6 +276,10 @@ static int lm83_detect(struct i2c_adapter *adapter, int address, unsigned
 	 * are skipped.
 	 */
 
+	/* Default to an LM83 if forced */
+	if (kind == 0)
+		kind = lm83;
+
 	if (kind < 0) /* detection */
 	{
 		if (((i2c_smbus_read_byte_data(new_client, LM83_REG_R_STATUS1)
@@ -350,7 +354,7 @@ static int lm83_detect(struct i2c_adapter *adapter, int address, unsigned
 	 */
 
 	if ((err = i2c_register_entry(new_client, type_name,
-	     lm83_dir_table_template)) < 0)
+	     lm83_dir_table_template, THIS_MODULE)) < 0)
 	{
 #ifdef DEBUG
 		printk("lm83.o: Failed registering directory entry.\n");
