@@ -27,6 +27,7 @@
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
+#include <linux/delay.h>
 #include "version.h"
 #include "sensors_vid.h"
 
@@ -153,9 +154,6 @@ SENSORS_INSMOD_1(lm87);
    these macros are called: arguments may be evaluated more than once.
    Fixing this is just not worth it. */
 
-#define IN_TO_REG(val,nr) (SENSORS_LIMIT(((val) & 0xff),0,255))
-#define IN_FROM_REG(val,nr) (val)
-
 static inline u8 FAN_TO_REG(long rpm, int div)
 {
 	if (rpm == 0)
@@ -168,32 +166,17 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define FAN_FROM_REG(val,div) ((val)==0?-1:\
                                (val)==255?0:1350000/((div)*(val)))
 
-#define TEMP_FROM_REG(temp)  (temp * 10)
+#define TEMP_FROM_REG(val) ((val)*10)
 
-#define TEMP_LIMIT_FROM_REG(val) (((val)>0x80?(val)-0x100:(val))*10)
-
-#define TEMP_LIMIT_TO_REG(val) SENSORS_LIMIT(((val)<0?(((val)-5)/10):\
-                                                      ((val)+5)/10),0,255)
-#if 0
-#define TEMP_FROM_REG(temp) \
-   ((temp)<256?((((temp)&0x1fe) >> 1) * 10)      + ((temp) & 1) * 5:  \
-               ((((temp)&0x1fe) >> 1) -255) * 10 - ((temp) & 1) * 5)  \
-
-#define TEMP_LIMIT_FROM_REG(val) (val)
-
-#define TEMP_LIMIT_TO_REG(val) SENSORS_LIMIT((val),0,255)
-#endif
-
-
-#define ALARMS_FROM_REG(val) (val)
+#define TEMP_LIMIT_TO_REG(val) SENSORS_LIMIT(((val)<0?(val)-5:\
+                                              (val)+5)/10,-128,127)
 
 #define DIV_FROM_REG(val) (1 << (val))
 #define DIV_TO_REG(val) ((val)==1?0:((val)==8?3:((val)==4?2:1)))
 
-/* For each registered LM87, we need to keep some data in memory. That
-   data is pointed to by LM87_list[NR]->data. The structure itself is
-   dynamically allocated, at the same time when a new LM87 client is
-   allocated. */
+/* For each registered LM87, we need to keep some data in memory. The
+   structure is dynamically allocated whenever a new LM87 client is
+   found. */
 struct lm87_data {
 	struct i2c_client client;
 	int sysctl_id;
@@ -218,9 +201,9 @@ struct lm87_data {
 	u8  fan2;		/* Register value */
 	u8  fan2_min;		/* Register value */
 	u8  fan2_div;		/* Register encoding, shifted right */
-	int ext2_temp;		/* Temp, shifted right */
-	int ext_temp;           /* Temp, shifted right */
-	int int_temp;		/* Temp, shifted right */
+	s8  ext2_temp;		/* Register value */
+	s8  ext_temp;		/* Register value */
+	s8  int_temp;		/* Register value */
 	u8  ext_temp_max;       /* Register value */
 	u8  ext_temp_min;       /* Register value */
 	u8  ext2_temp_max; 	/* Register value */
@@ -270,7 +253,6 @@ static void lm87_vrm(struct i2c_client *client, int operation,
 static int lm87_id = 0;
 
 static struct i2c_driver LM87_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "LM87 sensor driver",
 	.id		= I2C_DRIVERID_LM87,
 	.flags		= I2C_DF_NOTIFY,
@@ -289,9 +271,9 @@ static struct i2c_driver LM87_driver = {
 #define LM87_SYSCTL_AIN2       1007
 #define LM87_SYSCTL_FAN1       1102
 #define LM87_SYSCTL_FAN2       1103
-#define LM87_SYSCTL_TEMP1  1250 /* Degrees Celcius * 100 */
-#define LM87_SYSCTL_TEMP2   1251 /* Degrees Celcius * 100 */
-#define LM87_SYSCTL_TEMP3   1252 /* Degrees Celcius * 100 */
+#define LM87_SYSCTL_TEMP1      1250 /* Degrees Celcius * 10 */
+#define LM87_SYSCTL_TEMP2      1251 /* Degrees Celcius * 10 */
+#define LM87_SYSCTL_TEMP3      1252 /* Degrees Celcius * 10 */
 #define LM87_SYSCTL_FAN_DIV    2000 /* 1, 2, 4 or 8 */
 #define LM87_SYSCTL_ALARMS     2001 /* bitvector */
 #define LM87_SYSCTL_ANALOG_OUT 2002
@@ -456,7 +438,8 @@ static int lm87_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client,
 					type_name,
-					LM87_dir_table_template)) < 0) {
+					LM87_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -486,8 +469,8 @@ static int lm87_detach_client(struct i2c_client *client)
 				 sysctl_id);
 
 	if ((err = i2c_detach_client(client))) {
-		printk
-		    ("lm87.o: Client deregistration failed, client not detached.\n");
+		printk(KERN_ERR "lm87.o: Client deregistration failed, "
+		       "client not detached\n");
 		return err;
 	}
 
@@ -496,9 +479,25 @@ static int lm87_detach_client(struct i2c_client *client)
 	return 0;
 }
 
+#define MAX_RETRIES 5
+
 static int lm87_read_value(struct i2c_client *client, u8 reg)
 {
-	return 0xFF & i2c_smbus_read_byte_data(client, reg);
+	int value, i;
+
+	/* Retry in case of read errors */
+	for (i = 1; i <= MAX_RETRIES; i++) {
+		if ((value = i2c_smbus_read_byte_data(client, reg)) >= 0)
+			return value;
+
+		printk(KERN_WARNING "lm87.o: Read byte data failed, "
+		       "address 0x%02x\n", reg);
+		mdelay(i + 3);
+	}
+
+	/* <TODO> what to return in case of error? */
+	printk(KERN_ERR "lm87.o: All read byte retries failed!!\n");
+	return 0;
 }
 
 static int lm87_write_value(struct i2c_client *client, u8 reg, u8 value)
@@ -506,15 +505,11 @@ static int lm87_write_value(struct i2c_client *client, u8 reg, u8 value)
 	return i2c_smbus_write_byte_data(client, reg, value);
 }
 
-/* Called when we have found a new LM87. It should set limits, etc. */
+/* Called when we have found a new LM87. */
 static void lm87_init_client(struct i2c_client *client)
 {
 	struct lm87_data *data = client->data;
-
-	/* Reset all except Watchdog values and last conversion values
-	   This sets fan-divs to 2, among others. This makes most other
-	   initializations unnecessary */
-	lm87_write_value(client, LM87_REG_CONFIG, 0x80);
+	u8 reg;
 
         /* Setup Channel Mode register for configuration of monitoring 
 	 * Default is 00000000b
@@ -528,8 +523,10 @@ static void lm87_init_client(struct i2c_client *client)
 	 * 	bit 7 - Configures VID/IRQ input as interrupts if = 1
 	 */
 
+	/* Preserve 4 MSB */
+	reg = lm87_read_value(client, LM87_REG_CHANNEL_MODE);
 /* I know, not clean, but it works. :'p */
-	lm87_write_value(client, LM87_REG_CHANNEL_MODE,
+	lm87_write_value(client, LM87_REG_CHANNEL_MODE, (reg & 0xf0) |
 #ifdef LM87_AIN1
  0x01
 #else
@@ -558,7 +555,12 @@ static void lm87_init_client(struct i2c_client *client)
 	data->vrm = DEFAULT_VRM;
 
 	/* Start monitoring */
-	lm87_write_value(client, LM87_REG_CONFIG, 0x01);
+	reg = lm87_read_value(client, LM87_REG_CONFIG);
+	if (!(reg & 0x01)) {
+		printk(KERN_INFO "lm87.o: Monitoring starts\n");
+		lm87_write_value(client, LM87_REG_CONFIG,
+				 (reg & 0x7e) | 0x01);
+	}
 }
 
 static void lm87_update_client(struct i2c_client *client)
@@ -673,23 +675,20 @@ void lm87_in(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 2;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		lm87_update_client(client);
-		results[0] =
-		    ((long)data->in_min[nr] * scales[nr]) / 192;
-		results[1] =
-		    ((long)data->in_max[nr] * scales[nr]) / 192;
-		results[2] =
-		    ((long)data->in[nr] * scales[nr]) / 192;
+		results[0] = (data->in_min[nr] * scales[nr] + 96) / 192;
+		results[1] = (data->in_max[nr] * scales[nr] + 96) / 192;
+		results[2] = (data->in[nr] * scales[nr] + 96) / 192;
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->in_min[nr] =
-			    (results[0] * 192) / scales[nr];
+			data->in_min[nr] = (results[0] * 192 + scales[nr] / 2)
+					 / scales[nr];
 			lm87_write_value(client, LM87_REG_IN_MIN(nr),
 					    data->in_min[nr]);
 		}
 		if (*nrels_mag >= 2) {
-			data->in_max[nr] =
-			    (results[1] * 192) / scales[nr];
+			data->in_max[nr] = (results[1] * 192 + scales[nr] / 2)
+					 / scales[nr];
 			lm87_write_value(client, LM87_REG_IN_MAX(nr),
 					    data->in_max[nr]);
 		}
@@ -798,20 +797,20 @@ void lm87_temp(struct i2c_client *client, int operation, int ctl_name,
 	   /* find out which temp. is being requested */
 	   if (ctl_name == LM87_SYSCTL_TEMP3) 
 	   {
-		results[0] = TEMP_LIMIT_FROM_REG(data->ext2_temp_max);
-		results[1] = TEMP_LIMIT_FROM_REG(data->ext2_temp_min);
+		results[0] = TEMP_FROM_REG(data->ext2_temp_max);
+		results[1] = TEMP_FROM_REG(data->ext2_temp_min);
 		results[2] = TEMP_FROM_REG(data->ext2_temp);
 	   }
 	   else if(ctl_name == LM87_SYSCTL_TEMP2)
 	   {
-		results[0] = TEMP_LIMIT_FROM_REG(data->ext_temp_max);
-		results[1] = TEMP_LIMIT_FROM_REG(data->ext_temp_min);
+		results[0] = TEMP_FROM_REG(data->ext_temp_max);
+		results[1] = TEMP_FROM_REG(data->ext_temp_min);
 		results[2] = TEMP_FROM_REG(data->ext_temp);
 	   }
 	   else if(ctl_name == LM87_SYSCTL_TEMP1)
 	   {
-		results[0] = TEMP_LIMIT_FROM_REG(data->int_temp_max);
-		results[1] = TEMP_LIMIT_FROM_REG(data->int_temp_min);
+		results[0] = TEMP_FROM_REG(data->int_temp_max);
+		results[1] = TEMP_FROM_REG(data->int_temp_min);
 		results[2] = TEMP_FROM_REG(data->int_temp);
 	   }
 	   *nrels_mag = 3;
@@ -861,7 +860,7 @@ void lm87_alarms(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 0;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		lm87_update_client(client);
-		results[0] = ALARMS_FROM_REG(data->alarms);
+		results[0] = data->alarms;
 		*nrels_mag = 1;
 	}
 }
@@ -964,7 +963,7 @@ void lm87_vrm(struct i2c_client *client, int operation, int ctl_name,
 
 static int __init sm_lm87_init(void)
 {
-	printk("lm87.o version %s (%s)\n", LM_VERSION, LM_DATE);
+	printk(KERN_INFO "lm87.o version %s (%s)\n", LM_VERSION, LM_DATE);
 	return i2c_add_driver(&LM87_driver);
 }
 
