@@ -1,7 +1,7 @@
 /*
  * lm90.c - Part of lm_sensors, Linux kernel modules for hardware
  *          monitoring
- * Copyright (c) 2003  Jean Delvare <khali@linux-fr.org>
+ * Copyright (C) 2003-2004  Jean Delvare <khali@linux-fr.org>
  *
  * Based on the lm83 driver. The LM90 is a sensor chip made by National
  * Semiconductor. It reports up to two temperatures (its own plus up to
@@ -10,6 +10,42 @@
  * obtained from National's website at:
  *   http://www.national.com/pf/LM/LM90.html
  *
+ * This driver also supports the LM89 and LM99, two other sensor chips
+ * made by National Semiconductor. Both have an increased remote
+ * temperature measurement accuracy (1 degree), and the LM99
+ * additionally shifts remote temperatures (measured and limits) by 16
+ * degrees, which allows for higher temperatures measurement. The
+ * driver doesn't handle it since it can be done easily in user-space.
+ * Complete datasheets can be obtained from National's website at:
+ *   http://www.national.com/pf/LM/LM89.html
+ *   http://www.national.com/pf/LM/LM99.html
+ * Note that there is no way to differenciate between both chips.
+ *
+ * This driver also supports the LM86, another sensor chip made by
+ * National Semiconductor. It is exactly similar to the LM90 except it
+ * has a higher accuracy.
+ * Complete datasheet can be obtained from National's website at:
+ *   http://www.national.com/pf/LM/LM86.html
+ *
+ * This driver also supports the ADM1032, a sensor chip made by Analog
+ * Devices. That chip is similar to the LM90, with a few differences
+ * that are not handled by this driver. Complete datasheet can be
+ * obtained from Analog's website at:
+ *   http://products.analog.com/products/info.asp?product=ADM1032
+ * Among others, it has a higher accuracy than the LM90, much like the
+ * LM86 does.
+ *
+ * This driver also supports the MAX6657 and MAX6658, sensor chips made
+ * by Maxim. These chips are similar to the LM86. Complete datasheet
+ * can be obtained at Maxim's website at:
+ *   http://www.maxim-ic.com/quick_view2.cfm/qv_pk/2578
+ * Note that there is no way to differenciate between both chips (but
+ * no need either).
+ *
+ * Since the LM90 was the first chipset supported by this driver, most
+ * comments will refer to this chipset, but are actually general and
+ * concern all supported chipsets, unless mentioned otherwise.
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -32,12 +68,18 @@
 #include <linux/init.h>
 #include "version.h"
 
+#ifndef I2C_DRIVERID_LM90
+#define I2C_DRIVERID_LM90	1042
+#endif
+
 /*
  * Addresses to scan
  * Address is fully defined internally and cannot be changed.
+ * LM86, LM89, LM90, LM99, ADM1032, MAX6657 and MAX6658 have address 0x4c.
+ * LM89-1, and LM99-1 have address 0x4d.
  */
 
-static unsigned short normal_i2c[] = { 0x4c, SENSORS_I2C_END };
+static unsigned short normal_i2c[] = { 0x4c, 0x4d, SENSORS_I2C_END };
 static unsigned short normal_i2c_range[] = { SENSORS_I2C_END };
 static unsigned int normal_isa[] = { SENSORS_ISA_END };
 static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
@@ -46,11 +88,10 @@ static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
  * Insmod parameters
  */
 
-SENSORS_INSMOD_1(lm90);
+SENSORS_INSMOD_5(lm90, adm1032, lm99, lm86, max6657);
 
 /*
  * The LM90 registers
- * Manufacturer ID is 0x01 for National Semiconductor.
  */
 
 #define LM90_REG_R_MAN_ID        0xFE
@@ -89,25 +130,22 @@ SENSORS_INSMOD_1(lm90);
 #define LM90_REG_W_TCRIT_HYST    0x21
 
 /*
- * Conversions, initial values and various macros
- * The LM90 uses signed 8-bit values for the local temperatures,
- * and signed 11-bit values for the remote temperatures (except
- * T_CRIT). The 11-bit conversion formulas may not round negative
- * numbers perfectly, but who cares?
+ * Conversions and various macros
+ * For local temperatures and limits, critical limits and the hysteresis
+ * value, the LM90 uses signed 8-bit values with LSB = 1 degree Celcius.
+ * For remote temperatures and limits, it uses signed 11-bit values with
+ * LSB = 0.125 degree Celcius, left-justified in 16-bit registers.
  */
 
-#define TEMP1_FROM_REG(val)  (val & 0x80 ? val-0x100 : val)
-#define TEMP1_TO_REG(val)    (val < 0 ? val+0x100 : val)
-#define TEMP2_FROM_REG(val)  (((val & 0x8000 ? val-0x10000 : val) \
-                             * 10 + 128) >> 8)
-#define TEMP2_TO_REG(val)    (((val << 8) / 10 + (val < 0 ? \
-                             0x10000 : 0)) & 0xFFE0)
-#define HYST_TO_REG(val)     (val < 0 ? 0 : val > 31 ? 31 : val)
-
-#define LM90_INIT_LOW        5 /* degrees */
-#define LM90_INIT_HIGH      70
-#define LM90_INIT_CRIT      85
-#define LM90_INIT_HYST      10
+#define TEMP1_FROM_REG(val)	(val)
+#define TEMP1_TO_REG(val)	((val) <= -128 ? -128 : \
+				 (val) >= 127 ? 127 : (val))
+#define TEMP2_FROM_REG(val)	((val) / 32 * 125 / 100)
+#define TEMP2_TO_REG(val)	((val) <= -1280 ? 0x8000 : \
+				 (val) >= 1270 ? 0x7FE0 : \
+				 ((val) * 100 / 125 * 32))
+#define HYST_TO_REG(val)	((val) <= 0 ? 0 : \
+				 (val) >= 31 ? 31 : (val))
 
 /*
  * Functions declaration
@@ -126,7 +164,9 @@ static void lm90_local_tcrit(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results);
 static void lm90_remote_tcrit(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results);
-static void lm90_hyst(struct i2c_client *client, int operation,
+static void lm90_local_hyst(struct i2c_client *client, int operation,
+	int ctl_name, int *nrels_mag, long *results);
+static void lm90_remote_hyst(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results);
 static void lm90_alarms(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results);
@@ -134,10 +174,9 @@ static void lm90_alarms(struct i2c_client *client, int operation,
 /*
  * Driver data (common to all clients)
  */
- 
+
 static struct i2c_driver lm90_driver = {
-	.owner          = THIS_MODULE,
-	.name           = "LM90 sensor driver",
+	.name           = "LM90/ADM1032 sensor driver",
 	.id             = I2C_DRIVERID_LM90,
 	.flags          = I2C_DF_NOTIFY,
 	.attach_adapter = lm90_attach_adapter,
@@ -150,6 +189,7 @@ static struct i2c_driver lm90_driver = {
 
 struct lm90_data
 {
+	struct i2c_client client;
 	int sysctl_id;
 
 	struct semaphore update_lock;
@@ -157,11 +197,11 @@ struct lm90_data
 	unsigned long last_updated; /* in jiffies */
 
 	/* registers values */
-	u8 local_temp, local_high, local_low;
-	u16 remote_temp, remote_high, remote_low; /* combined */
-	u8 local_crit, remote_crit;
-	u8 hyst;
-	u16 alarms; /* bitvector, combined */
+	s8 local_temp, local_high, local_low;
+	s16 remote_temp, remote_high, remote_low; /* combined */
+	s8 local_crit, remote_crit;
+	u8 hyst; /* linked to two sysctl files (hyst1 RW, hyst2 RO) */
+	u8 alarms; /* bitvector */
 };
 
 /*
@@ -175,7 +215,8 @@ struct lm90_data
 #define LM90_SYSCTL_REMOTE_TEMP   1201
 #define LM90_SYSCTL_LOCAL_TCRIT   1204
 #define LM90_SYSCTL_REMOTE_TCRIT  1205
-#define LM90_SYSCTL_HYST          1207
+#define LM90_SYSCTL_LOCAL_HYST    1207
+#define LM90_SYSCTL_REMOTE_HYST   1208
 #define LM90_SYSCTL_ALARMS        1210
 
 #define LM90_ALARM_LOCAL_HIGH     0x40
@@ -199,8 +240,10 @@ static ctl_table lm90_dir_table_template[] =
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_local_tcrit},
 	{LM90_SYSCTL_REMOTE_TCRIT, "tcrit2", NULL, 0, 0644, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_remote_tcrit},
-	{LM90_SYSCTL_HYST, "hyst", NULL, 0, 0644, NULL,
-	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_hyst},
+	{LM90_SYSCTL_LOCAL_HYST, "hyst1", NULL, 0, 0644, NULL,
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_local_hyst},
+	{LM90_SYSCTL_REMOTE_HYST, "hyst2", NULL, 0, 0444, NULL,
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_remote_hyst},
 	{LM90_SYSCTL_ALARMS, "alarms", NULL, 0, 0444, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &lm90_alarms},
 	{0}
@@ -225,7 +268,6 @@ static int lm90_attach_adapter(struct i2c_adapter *adapter)
  * The following function does more than just detection. If detection
  * succeeds, it also registers the new chip.
  */
-
 static int lm90_detect(struct i2c_adapter *adapter, int address,
 	unsigned short flags, int kind)
 {
@@ -252,21 +294,21 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 		return 0;
 	}
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-		sizeof(struct lm90_data), GFP_KERNEL)))
+	if (!(data = kmalloc(sizeof(struct lm90_data), GFP_KERNEL)))
 	{
 		printk("lm90.o: Out of memory in lm90_detect (new_client).\n");
 		return -ENOMEM;
 	}
 
 	/*
-	 * The LM90-specific data is placed right after the common I2C
-	 * client data, and is pointed to by the data field from the I2C
-	 * client data.
+	 * The common I2C client data is placed right before the
+	 * LM90-specific data. The LM90-specific data is pointed to by the
+	 * data field from the I2C client data.
 	 */
 
+	new_client = &data->client;
 	new_client->addr = address;
-	new_client->data = data = (struct lm90_data *) (new_client + 1);
+	new_client->data = data;
 	new_client->adapter = adapter;
 	new_client->driver = &lm90_driver;
 	new_client->flags = 0;
@@ -274,8 +316,7 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	/*
 	 * Now we do the remaining detection. A negative kind means that
 	 * the driver was loaded with no force parameter (default), so we
-	 * must both detect and identify the chip (actually there is only
-	 * one possible kind of chip for now, LM90). A zero kind means that
+	 * must both detect and identify the chip. A zero kind means that
 	 * the driver was loaded with the force parameter, the detection
 	 * step shall be skipped. A positive kind means that the driver
 	 * was loaded with the force parameter and a given kind of chip is
@@ -283,33 +324,58 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	 * are skipped.
 	 */
 
-	if (kind < 0) /* detection */
-	{
-		if (((i2c_smbus_read_byte_data(new_client, LM90_REG_R_CONFIG1)
-		      & 0x2A) != 0x00)
-		||  (i2c_smbus_read_byte_data(new_client, LM90_REG_R_CONVRATE)
-			 > 9)
-		||  ((i2c_smbus_read_byte_data(new_client, LM90_REG_R_CONFIG2)
-		      & 0xF8) != 0x00))
-		{
-#ifdef DEBUG
-			printk("lm90.o: LM90 detection failed at 0x%02x.\n",
-				address);
-#endif
-			goto ERROR1;
-		}
-	}
+	/* Default to an LM90 if forced */
+	if (kind == 0)
+		kind = lm90;
 
-	if (kind <= 0) /* identification */
+	if (kind < 0) /* detection and identification */
 	{
-		u8 man_id, chip_id;
+		u8 man_id, chip_id, reg_config1, reg_convrate;
 
-		man_id = i2c_smbus_read_byte_data(new_client, LM90_REG_R_MAN_ID);
-		chip_id = i2c_smbus_read_byte_data(new_client, LM90_REG_R_CHIP_ID);
+		man_id = i2c_smbus_read_byte_data(new_client,
+			LM90_REG_R_MAN_ID);
+		chip_id = i2c_smbus_read_byte_data(new_client,
+			LM90_REG_R_CHIP_ID);
+		reg_config1 = i2c_smbus_read_byte_data(new_client,
+			LM90_REG_R_CONFIG1);
+		reg_convrate = i2c_smbus_read_byte_data(new_client,
+			LM90_REG_R_CONVRATE);
+		
 		if (man_id == 0x01) /* National Semiconductor */
 		{
-			if (chip_id >= 0x21 && chip_id < 0x30)
-				kind = lm90;
+			u8 reg_config2;
+
+			reg_config2 = i2c_smbus_read_byte_data(new_client,
+				LM90_REG_R_CONFIG2);
+
+			if ((reg_config1 & 0x2A) == 0x00
+			 && (reg_config2 & 0xF8) == 0x00
+			 && reg_convrate <= 0x09)
+			{
+				if (address == 0x4C
+				 && (chip_id & 0xF0) == 0x20) /* LM90 */
+					kind = lm90;
+				else if ((chip_id & 0xF0) == 0x30) /* LM89/LM99 */
+					kind = lm99;
+				else if (address == 0x4C
+				 && (chip_id & 0xF0) == 0x10) /* LM86 */
+					kind = lm99;
+			}
+		}
+		else if (man_id == 0x41) /* Analog Devices */
+		{
+			if (address == 0x4C
+			 && (chip_id & 0xF0) == 0x40 /* ADM1032 */
+			 && (reg_config1 & 0x3F) == 0x00
+			 && reg_convrate <= 0x0A)
+				kind = adm1032;
+		}
+		else if (man_id == 0x4D) /* Maxim */
+		{
+			if (address == 0x4C
+			 && (reg_config1 & 0x1F) == 0
+			 && reg_convrate <= 0x09)
+				kind = max6657;
 		}
 	}
 
@@ -324,6 +390,26 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 		type_name = "lm90";
 		client_name = "LM90 chip";
 	}
+	else if (kind == adm1032)
+	{
+		type_name = "adm1032";
+		client_name = "ADM1032 chip";
+	}
+	else if (kind == lm99)
+	{
+		type_name = "lm99";
+		client_name = "LM99 chip";
+	}
+	else if (kind == lm86)
+	{
+		type_name = "lm86";
+		client_name = "LM86 chip";
+	}
+	else if (kind == max6657)
+	{
+		type_name = "max6657";
+		client_name = "MAX6657 chip";
+	}
 	else
 	{
 		printk("lm90.o: Unknown kind %d.\n", kind);
@@ -357,7 +443,7 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	 */
 
 	if ((err = i2c_register_entry(new_client, type_name,
-	     lm90_dir_table_template)) < 0)
+	     lm90_dir_table_template, THIS_MODULE)) < 0)
 	{
 #ifdef DEBUG
 		printk("lm90.o: Failed registering directory entry.\n");
@@ -376,7 +462,7 @@ static int lm90_detect(struct i2c_adapter *adapter, int address,
 	ERROR2:
 	i2c_detach_client(new_client);
 	ERROR1:
-	kfree(new_client);
+	kfree(data);
 	return err;
 }
 
@@ -385,30 +471,7 @@ static void lm90_init_client(struct i2c_client *client)
 	u8 config;
 
 	/*
-	 * Set limits.
-	 */
-
-	i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_HIGH,
-		TEMP1_TO_REG(LM90_INIT_HIGH));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_LOW,
-		TEMP1_TO_REG(LM90_INIT_LOW));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_LOCAL_CRIT,
-		TEMP1_TO_REG(LM90_INIT_CRIT));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_HIGHH,
-		TEMP1_TO_REG(LM90_INIT_HIGH));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_HIGHL,
-		0);
-	i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_LOWH,
-		TEMP1_TO_REG(LM90_INIT_LOW));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_LOWL,
-		0);
-	i2c_smbus_write_byte_data(client, LM90_REG_W_REMOTE_CRIT,
-		TEMP1_TO_REG(LM90_INIT_CRIT));
-	i2c_smbus_write_byte_data(client, LM90_REG_W_TCRIT_HYST,
-		HYST_TO_REG(LM90_INIT_HYST));
-
-	/*
-	 * Starts the conversions.
+	 * Start the conversions.
 	 */
 
 	i2c_smbus_write_byte_data(client, LM90_REG_W_CONVRATE,
@@ -432,7 +495,7 @@ static int lm90_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 	return 0;
 }
 
@@ -447,7 +510,7 @@ static void lm90_update_client(struct i2c_client *client)
 	{
 		u8 oldh, newh;
 #ifdef DEBUG
-		printk("lm90.o: Updating LM90 data.\n");
+		printk("lm90.o: Updating data.\n");
 #endif
 
 		data->local_temp =
@@ -629,7 +692,22 @@ static void lm90_remote_tcrit(struct i2c_client *client, int operation,
 	}
 }
 
-static void lm90_hyst(struct i2c_client *client, int operation,
+/*
+ * One quick note about hysteresis. Internally, the hysteresis value
+ * is held in a single register by the LM90, as a relative value.
+ * This relative value applies to both the local critical temperature
+ * and the remote critical temperature. Since all temperatures exported
+ * through procfs have to be absolute, we have to do some conversions.
+ * The solution retained here is to export two absolute values, one for
+ * each critical temperature. In order not to confuse the users too
+ * much, only one file is writable. Would we fail to do so, users
+ * would probably attempt to write to both files, as if they were
+ * independant, and since they aren't, they wouldn't understand why
+ * setting one affects the other one (and would probably claim there's
+ * a bug in the driver).
+ */
+
+static void lm90_local_hyst(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results)
 {
 	struct lm90_data *data = client->data;
@@ -639,20 +717,37 @@ static void lm90_hyst(struct i2c_client *client, int operation,
 	else if (operation == SENSORS_PROC_REAL_READ)
 	{
 		lm90_update_client(client);
-		results[0] = TEMP1_FROM_REG(data->hyst);
+		results[0] = TEMP1_FROM_REG(data->local_crit) -
+			TEMP1_FROM_REG(data->hyst);
 		*nrels_mag = 1;
 	}
 	else if (operation == SENSORS_PROC_REAL_WRITE)
 	{
 		if (*nrels_mag >= 1)
 		{
-			data->hyst = HYST_TO_REG(results[0]);
+			data->hyst = HYST_TO_REG(data->local_crit - results[0]);
 			i2c_smbus_write_byte_data(client, LM90_REG_W_TCRIT_HYST,
 				data->hyst);
 		}
 	}
 }
 
+static void lm90_remote_hyst(struct i2c_client *client, int operation,
+	int ctl_name, int *nrels_mag, long *results)
+{
+	struct lm90_data *data = client->data;
+
+	if (operation == SENSORS_PROC_REAL_INFO)
+		*nrels_mag = 0; /* magnitude */
+	else if (operation == SENSORS_PROC_REAL_READ)
+	{
+		lm90_update_client(client);
+		results[0] = TEMP1_FROM_REG(data->remote_crit) -
+			TEMP1_FROM_REG(data->hyst);
+		*nrels_mag = 1;
+	}
+}
+
 static void lm90_alarms(struct i2c_client *client, int operation,
 	int ctl_name, int *nrels_mag, long *results)
 {
@@ -680,7 +775,7 @@ static void __exit sm_lm90_exit(void)
 }
 
 MODULE_AUTHOR("Jean Delvare <khali@linux-fr.org>");
-MODULE_DESCRIPTION("LM90 sensor driver");
+MODULE_DESCRIPTION("LM90/ADM1032 sensor driver");
 MODULE_LICENSE("GPL");
 
 module_init(sm_lm90_init);
