@@ -66,12 +66,16 @@ MODULE_PARM_DESC(disable_block,
 
 static int init = 1;
 MODULE_PARM(init, "i");
-MODULE_PARM_DESC(init, "Set to zero to bypass chip initialization");
+MODULE_PARM_DESC(init, "Set to zero to bypass most chip initialization");
 
 static int vccp_limit_type[2] /* = {0,0} */ ;
 MODULE_PARM(vccp_limit_type, "2-2i");
 MODULE_PARM_DESC(vccp_limit_type, "Configures in7 and in8 limit modes");
 
+static int vid_agtl = 0;
+MODULE_PARM(vid_agtl, "i");
+MODULE_PARM_DESC(vid_agtl, "Configures VID pin input thresholds");
+
 /* -- SENSORS SYSCTL START -- */
 /* volts * 100 */
 #define LM93_SYSCTL_IN1				1001
@@ -378,6 +382,7 @@ static u8 lm93_read_byte(struct i2c_client *client, u8 reg)
 	}
 
 	/* <TODO> what to return in case of error? */
+	printk(KERN_ERR "lm93.o: All read byte retries failed!!\n");
 	return 0;
 }
 
@@ -412,6 +417,7 @@ static u16 lm93_read_word(struct i2c_client *client, u8 reg)
 	}
 
 	/* <TODO> what to return in case of error? */
+	printk(KERN_ERR "lm93.o: All read word retries failed!!\n");
 	return 0;
 }
 
@@ -438,13 +444,15 @@ static u8 lm93_block_buffer[I2C_SMBUS_BLOCK_MAX];
 */
 static void lm93_read_block(struct i2c_client *client, u8 fbn, u8 *values)
 {
-	int i, result;
+	int i, result=0;
 
 	for (i = 1; i <= MAX_RETRIES; i++) {
 		result = i2c_smbus_read_block_data(client, 
 			lm93_block_read_cmds[fbn].cmd, lm93_block_buffer);
 
-		if (result != lm93_block_read_cmds[fbn].len) {
+		if (result == lm93_block_read_cmds[fbn].len) {
+			break;
+		} else {
 			printk(KERN_WARNING "lm93.o: block read data failed, "
 				"command 0x%02x.\n", 
 				lm93_block_read_cmds[fbn].cmd);
@@ -481,6 +489,7 @@ static void lm93_update_client_common(struct lm93_data *data,
 		struct i2c_client *client)
 {
 	int i;
+	u8 *ptr;
 
 	/* temp1 - temp4: limits */
 	for (i = 0; i < 4; i++) {
@@ -540,6 +549,10 @@ static void lm93_update_client_common(struct lm93_data *data,
 	data->sfc2 = lm93_read_byte(client, LM93_REG_SFC2);
 	data->sf_tach_to_pwm = lm93_read_byte(client,
 			LM93_REG_SF_TACH_TO_PWM);
+
+	/* write back alarm values to clear */
+	for (i = 0, ptr = (u8 *)(&data->block1); i < 8; i++)
+		lm93_write_byte(client, LM93_REG_HOST_ERROR_1 + i, *(ptr + i));
 }
 
 /* update routine which uses SMBus block data commands */
@@ -837,7 +850,18 @@ static void lm93_in(struct i2c_client *client, int operation, int ctl_name,
 static unsigned LM93_ALARMS_FROM_REG(struct block1_t b1)
 {
 	unsigned result;
-	result  = b1.host_status_2;
+	result  = b1.host_status_2 & 0x3f;
+
+	if (vccp_limit_type[0])
+		result |= (b1.host_status_4 & 0x10) << 2;
+	else
+		result |= b1.host_status_2 & 0x40;
+
+	if (vccp_limit_type[1])
+		result |= (b1.host_status_4 & 0x20) << 2;
+	else
+		result |= b1.host_status_2 & 0x80;
+
 	result |= b1.host_status_3 << 8;
 	result |= (b1.fan_status & 0x0f) << 16;
 	result |= (b1.p1_prochot_status & 0x80) << 13;
@@ -1685,10 +1709,12 @@ static void lm93_fan_smart_tach(struct i2c_client *client, int operation,
 	if (operation == SENSORS_PROC_REAL_INFO)
 		*nrels_mag = 0;
 	else if (operation == SENSORS_PROC_REAL_READ) {
+		int mapping;
+
 		lm93_update_client(client);
 
 		/* extract the relevant mapping */
-		int mapping = (data->sf_tach_to_pwm >> (nr * 2)) & 0x03;
+		mapping = (data->sf_tach_to_pwm >> (nr * 2)) & 0x03;
 
 		/* if there's a mapping and it's enabled */
 		if (mapping && ((data->sfc2 >> nr) & 0x01))
@@ -2112,10 +2138,12 @@ static ctl_table lm93_dir_table_template[] = {
 static void lm93_init_client(struct i2c_client *client)
 {
 	int i;
-	u8 reg = lm93_read_byte(client, LM93_REG_CONFIG);
+	u8 reg;
 
-	/* start monitoring */
-	lm93_write_byte(client, LM93_REG_CONFIG, reg | 0x01);
+	/* configure VID pin input thresholds */
+	reg = lm93_read_byte(client, LM93_REG_GPI_VID_CTL);
+	lm93_write_byte(client, LM93_REG_GPI_VID_CTL,
+			reg | (vid_agtl ? 0x03 : 0x00));
 
 	if (init) {
 		/* enable #ALERT pin */
@@ -2128,8 +2156,19 @@ static void lm93_init_client(struct i2c_client *client)
 
 		/* set sleep state to S0 */
 		lm93_write_byte(client, LM93_REG_SLEEP_CONTROL, 0);
+
+		/* unmask #VRDHOT and dynamic VCCP (if nec) error events */
+		reg = lm93_read_byte(client, LM93_REG_MISC_ERR_MASK);
+		reg &= ~0x03;
+		reg &= ~(vccp_limit_type[0] ? 0x10 : 0);
+		reg &= ~(vccp_limit_type[1] ? 0x20 : 0);
+		lm93_write_byte(client, LM93_REG_MISC_ERR_MASK, reg);
 	}
-		
+
+	/* start monitoring */
+	reg = lm93_read_byte(client, LM93_REG_CONFIG);
+	lm93_write_byte(client, LM93_REG_CONFIG, reg | 0x01);
+
 	/* spin until ready */
 	for (i=0; i<20; i++) {
 		mdelay(10);
@@ -2245,7 +2284,7 @@ static int lm93_detect(struct i2c_adapter *adapter, int address,
 
 	/* register a new directory entry with module sensors */
 	if ((data->sysctl_id = i2c_register_entry(client, "lm93", 
-			lm93_dir_table_template)) < 0) {
+			lm93_dir_table_template, THIS_MODULE)) < 0) {
 		err = data->sysctl_id;
 		goto ERROR2;
 	}
@@ -2286,7 +2325,6 @@ static int lm93_detach_client(struct i2c_client *client)
 }
 
 static struct i2c_driver lm93_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "LM93 sensor driver",
 	.id		= I2C_DRIVERID_LM93,
 	.flags		= I2C_DF_NOTIFY,
