@@ -53,15 +53,9 @@ static const char *version_str = "2.00 29/2/2000 Fons Rademakers";
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
-#include <asm/io.h>
 #include "version.h"
 
 MODULE_LICENSE("GPL");
@@ -244,6 +238,7 @@ enum sensor_type { fan, temp, vid, pll, lcd, alarm };
    client is allocated. We assume MaxiLife will only be present on the
    SMBus and not on the ISA bus. */
 struct maxi_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 	enum maxi_type type;
@@ -311,7 +306,6 @@ static void maxi_alarms(struct i2c_client *client, int operation,
 /* The driver. I choose to use type i2c_driver, as at is identical to
    the smbus_driver. */
 static struct i2c_driver maxi_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "HP MaxiLife driver",
 	.id		= I2C_DRIVERID_MAXILIFE,
 	.flags		= I2C_DF_NOTIFY,
@@ -445,15 +439,13 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access maxi_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct maxi_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct maxi_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
 	/* Fill the new client structure with data */
-	data = (struct maxi_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -622,7 +614,8 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((err = i2c_register_entry(new_client, type_name,
-					  maxi_dir_table_template)) < 0)
+					  maxi_dir_table_template,
+					  THIS_MODULE)) < 0)
 		goto ERROR4;
 	data->sysctl_id = err;
 
@@ -635,7 +628,7 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
       ERROR4:
 	i2c_detach_client(new_client);
       ERROR2:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -656,7 +649,7 @@ static int maxi_detach_client(struct i2c_client *client)
 		    ("maxilife: Client deregistration failed, client not detached.\n");
 		return err;
 	}
-	kfree(client);
+	kfree(client->data);
 	return 0;
 }
 
