@@ -1,7 +1,7 @@
 /*
    mtp008.c - Part of lm_sensors, Linux kernel modules for hardware
    monitoring
-   Copyright (c) 2001  Kris Van Hees <aedil@alchar.org>
+   Copyright (C) 2001, 2004  Kris Van Hees <aedil@alchar.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,15 +20,9 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
-#include <asm/io.h>
 #include "version.h"
 
 MODULE_LICENSE("GPL");
@@ -102,7 +96,7 @@ SENSORS_INSMOD_1(mtp008);
 #define PIIDIODE	3
 
 /*
- * Conversion routines and macros.  Rounding and limit checking is only done on
+ * Conversion routines and macros.  Limit checking is only done on
  * the TO_REG variants.
  *
  * Note that IN values are expressed as 100 times the actual voltage to avoid
@@ -110,7 +104,7 @@ SENSORS_INSMOD_1(mtp008);
  * 409 (0V to 4.096V).
  */
 #define IN_TO_REG(val)		(SENSORS_LIMIT((((val) * 10 + 8) / 16), 0, 255))
-#define IN_FROM_REG(val)	(((val) * 16) / 10)
+#define IN_FROM_REG(val)	(((val) * 16 + 5) / 10)
 
 /*
  * The fan cotation count (as stored in the register) is calculated using the
@@ -193,57 +187,13 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define PWM_TO_REG(val)		(val)
 #define PWMENABLE_FROM_REG(nr, val)	(((val) >> ((nr) + 3)) & 1)
 
-/* Initial limits */
-#define MTP008_INIT_IN_0	(vid)				/* VCore 1 */
-#define MTP008_INIT_IN_1	330				/* +3.3V */
-#define MTP008_INIT_IN_2	(1200 * 10 / 38)		/* +12V */
-#define MTP008_INIT_IN_3	(vid)				/* VCore 2 */
-#define MTP008_INIT_IN_5	((11861 + 7 * (-1200)) / 36)	/* -12V */
-#define MTP008_INIT_IN_6	150				/* Vtt */
-
-#define MTP008_INIT_IN_PCT	10
-
-#define MTP008_INIT_IN_MIN_0	(MTP008_INIT_IN_0 -			      \
-				 MTP008_INIT_IN_0 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_0	(MTP008_INIT_IN_0 +			      \
-				 MTP008_INIT_IN_0 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MIN_1	(MTP008_INIT_IN_1 -			      \
-				 MTP008_INIT_IN_1 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_1	(MTP008_INIT_IN_1 +			      \
-				 MTP008_INIT_IN_1 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MIN_2	(MTP008_INIT_IN_2 -			      \
-				 MTP008_INIT_IN_2 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_2	(MTP008_INIT_IN_2 +			      \
-				 MTP008_INIT_IN_2 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MIN_3	(MTP008_INIT_IN_3 -			      \
-				 MTP008_INIT_IN_3 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_3	(MTP008_INIT_IN_3 +			      \
-				 MTP008_INIT_IN_3 * MTP008_INIT_IN_PCT / 100)
-
-#define MTP008_INIT_IN_MIN_5	(MTP008_INIT_IN_5 -			      \
-				 MTP008_INIT_IN_5 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_5	(MTP008_INIT_IN_5 +			      \
-				 MTP008_INIT_IN_5 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MIN_6	(MTP008_INIT_IN_6 -			      \
-				 MTP008_INIT_IN_6 * MTP008_INIT_IN_PCT / 100)
-#define MTP008_INIT_IN_MAX_6	(MTP008_INIT_IN_6 +			      \
-				 MTP008_INIT_IN_6 * MTP008_INIT_IN_PCT / 100)
-
-#define MTP008_INIT_FAN_MIN_1	3000
-#define MTP008_INIT_FAN_MIN_2	3000
-#define MTP008_INIT_FAN_MIN_3	3000
-
-#define MTP008_INIT_TEMP_OVER	700			/* 70 Celsius */
-#define MTP008_INIT_TEMP_HYST	500			/* 50 Celsius */
-#define MTP008_INIT_TEMP2_OVER	700			/* 70 Celsius */
-#define MTP008_INIT_TEMP2_HYST	500			/* 50 Celsius */
-
 /*
  * For each registered MTP008, we need to keep some data in memory.  The
  * structure itself is dynamically allocated, at the same time when a new
  * mtp008 client is allocated.
  */
 struct mtp008_data {
+	struct i2c_client client;
 	int sysctl_id;
 	enum chips type;
 
@@ -306,7 +256,6 @@ static int mtp008_id = 0;
 
 static struct i2c_driver mtp008_driver =
 {
-	.owner		= THIS_MODULE,
 	.name		= "MTP008 sensor driver",
 	.id		= I2C_DRIVERID_MTP008,
 	.flags		= I2C_DF_NOTIFY,
@@ -387,9 +336,9 @@ static ctl_table mtp008_dir_table_template[] =
 	{MTP008_SYSCTL_TEMP1, "temp1", NULL, 0, 0644, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_temp},
 	{MTP008_SYSCTL_TEMP2, "temp2", NULL, 0, 0644, NULL,
-       &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_temp_add},
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_temp_add},
 	{MTP008_SYSCTL_TEMP3, "temp3", NULL, 0, 0644, NULL,
-       &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_temp_add},
+	 &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_temp_add},
 	{MTP008_SYSCTL_VID, "vid", NULL, 0, 0444, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &mtp008_vid},
 	{MTP008_SYSCTL_FAN_DIV, "fan_div", NULL, 0, 0644, NULL,
@@ -453,13 +402,12 @@ int mtp008_detect(struct i2c_adapter *adapter, int address,
 	 * structure, even though we cannot fill it completely yet.  But it
 	 * allows us to use mtp008_(read|write)_value().
 	 */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct mtp008_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct mtp008_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
-	data = (struct mtp008_data *) (new_client + 1);
+
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -495,7 +443,8 @@ int mtp008_detect(struct i2c_adapter *adapter, int address,
 	 * Register a new directory entry with the sensors module.
 	 */
 	if ((sysid = i2c_register_entry(new_client, type_name,
-					    mtp008_dir_table_template)) < 0) {
+					    mtp008_dir_table_template,
+					    THIS_MODULE)) < 0) {
 		err = sysid;
 		goto ERROR2;
 	}
@@ -514,7 +463,7 @@ int mtp008_detect(struct i2c_adapter *adapter, int address,
       ERROR2:
 	i2c_detach_client(new_client);
       ERROR1:
-	kfree(new_client);
+	kfree(data);
 
       ERROR0:
 	return err;
@@ -532,7 +481,7 @@ static int mtp008_detach_client(struct i2c_client *client)
 		       "client not detached.\n");
 		return err;
 	}
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -551,7 +500,6 @@ static int mtp008_write_value(struct i2c_client *client, u8 reg, u8 value)
 /* Called when we have found a new MTP008. It should set limits, etc. */
 static void mtp008_init_client(struct i2c_client *client)
 {
-	int vid;
 	u8 save1, save2;
 	struct mtp008_data *data;
 
@@ -570,65 +518,6 @@ static void mtp008_init_client(struct i2c_client *client)
 
 	mtp008_getsensortype(data, save2);
 
-	/*
-	 * Retrieve the VID setting (needed for the default limits).
-	 */
-	vid = mtp008_read_value(client, MTP008_REG_VID_FANDIV) & 0x0f;
-	vid |= (mtp008_read_value(client, MTP008_REG_RESET_VID4) & 0x01) << 4;
-	vid = VID_FROM_REG(vid);
-
-	/*
-	 * Set the default limits.
-	 *
-	 * Setting temp sensors is done as follows:
-	 *
-	 *          Register 0x57: 0 0 0 0 x x x x
-	 *                                 | \ / +-- AIN5/VT3
-	 *                                 |  +----- AIN4/VT2/PII
-	 *                                 +-------- VT1/PII
-	 */
-
-	mtp008_write_value(client, MTP008_REG_IN_MAX(0),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_0));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(0),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_0));
-	mtp008_write_value(client, MTP008_REG_IN_MAX(1),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_1));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(1),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_1));
-	mtp008_write_value(client, MTP008_REG_IN_MAX(2),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_2));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(2),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_2));
-	mtp008_write_value(client, MTP008_REG_IN_MAX(3),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_3));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(3),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_3));
-
-	mtp008_write_value(client, MTP008_REG_IN_MAX(5),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_5));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(5),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_5));
-	mtp008_write_value(client, MTP008_REG_IN_MAX(6),
-			   IN_TO_REG(MTP008_INIT_IN_MAX_6));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(6),
-			   IN_TO_REG(MTP008_INIT_IN_MIN_6));
-
-	mtp008_write_value(client, MTP008_REG_TEMP_MAX,
-			   TEMP_TO_REG(MTP008_INIT_TEMP_OVER));
-	mtp008_write_value(client, MTP008_REG_TEMP_MIN,
-			   TEMP_TO_REG(MTP008_INIT_TEMP_HYST));
-	mtp008_write_value(client, MTP008_REG_IN_MAX(4),
-			   TEMP_TO_REG(MTP008_INIT_TEMP2_OVER));
-	mtp008_write_value(client, MTP008_REG_IN_MIN(4),
-			   TEMP_TO_REG(MTP008_INIT_TEMP2_HYST));
-
-	mtp008_write_value(client, MTP008_REG_FAN_MIN(1),
-			   FAN_TO_REG(MTP008_INIT_FAN_MIN_1, 2));
-	mtp008_write_value(client, MTP008_REG_FAN_MIN(2),
-			   FAN_TO_REG(MTP008_INIT_FAN_MIN_2, 2));
-	mtp008_write_value(client, MTP008_REG_FAN_MIN(3),
-			   FAN_TO_REG(MTP008_INIT_FAN_MIN_3, 2));
 
 	/*
 	 * Start monitoring.
@@ -920,12 +809,14 @@ void mtp008_temp_add(struct i2c_client *client, int operation, int ctl_name,
 		if(data->sens[nr - 3] != VOLTAGE) {
 			if (*nrels_mag >= 1) {
 				data->in_max[nr] = TEMP_TO_REG(results[0]);
-				mtp008_write_value(client, MTP008_REG_TEMP_MAX,
+				mtp008_write_value(client,
+						   MTP008_REG_IN_MAX(nr),
 						   data->in_max[nr]);
 			}
 			if (*nrels_mag >= 2) {
 				data->in_min[nr] = TEMP_TO_REG(results[1]);
-				mtp008_write_value(client, MTP008_REG_TEMP_MIN,
+				mtp008_write_value(client,
+						   MTP008_REG_IN_MIN(nr),
 						   data->in_min[nr]);
 			}
 		}
