@@ -70,9 +70,8 @@ SENSORS_INSMOD_1(pcf8591);
 
 
 struct pcf8591_data {
-        struct semaphore lock;
+	struct i2c_client client;
         int sysctl_id;
-        enum chips type;
 
         struct semaphore update_lock;
         char valid;             /* !=0 if following fields are valid */
@@ -103,7 +102,6 @@ static void pcf8591_aout(struct i2c_client *client, int operation,
 
 /* This is the driver that will be inserted */
 static struct i2c_driver pcf8591_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "PCF8591 sensor chip driver",
 	.id		= I2C_DRIVERID_PCF8591,
 	.flags		= I2C_DF_NOTIFY,
@@ -179,19 +177,18 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
         }
 #endif
 
-        if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE))
+        if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE
+                                     | I2C_FUNC_SMBUS_WRITE_BYTE_DATA))
                 goto ERROR0;
 
         /* OK. For now, we presume we have a valid client. We now create the
            client structure, even though we cannot fill it completely yet. */
-        if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-                                   sizeof(struct pcf8591_data),
-                                   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct pcf8591_data), GFP_KERNEL))) {
                 err = -ENOMEM;
                 goto ERROR0;
         }
 
-        data = (struct pcf8591_data *) (new_client + 1);
+	new_client = &data->client;
         new_client->addr = address;
         new_client->data = data;
         new_client->adapter = adapter;
@@ -205,16 +202,8 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
         if (kind <= 0)
                 kind = pcf8591;
 
-        if (kind == pcf8591) {
-                type_name = "pcf8591";
-                client_name = "PCF8591 chip";
-        } else {
-#ifdef DEBUG
-                printk(KERN_ERR "pcf8591.o: Internal error: unknown kind (%d)?!?",
-                       kind);
-#endif
-                goto ERROR1;
-        }
+	type_name = "pcf8591";
+	client_name = "PCF8591 chip";
 
         /* Fill in the remaining client fields and put it into the global list */
         strcpy(new_client->name, client_name);
@@ -230,7 +219,8 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
         /* Register a new directory entry with module sensors */
         if ((i = i2c_register_entry(new_client,
                                         type_name,
-                                        pcf8591_dir_table_template)) < 0) {
+                                        pcf8591_dir_table_template,
+					THIS_MODULE)) < 0) {
                 err = i;
                 goto ERROR4;
         }
@@ -246,8 +236,7 @@ int pcf8591_detect(struct i2c_adapter *adapter, int address,
       ERROR4:
         i2c_detach_client(new_client);
       ERROR3:
-      ERROR1:
-        kfree(new_client);
+	kfree(data);
       ERROR0:
         return err;
 }
@@ -265,7 +254,7 @@ static int pcf8591_detach_client(struct i2c_client *client)
                 return err;
         }
 
-        kfree(client);
+	kfree(client->data);
 
         return 0;
 }
