@@ -126,7 +126,6 @@ static void smartbatt_update_client(struct i2c_client *client);
 
 /* This is the driver that will be inserted */
 static struct i2c_driver smartbatt_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "Smart Battery chip driver",
 	.id		= I2C_DRIVERID_SMARTBATT,
 	.flags		= I2C_DF_NOTIFY,
@@ -280,7 +279,8 @@ int smartbatt_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry(new_client, type_name,
-					smartbatt_dir_table_template)) < 0) {
+					smartbatt_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
