@@ -86,7 +86,7 @@ superio_exit(void)
  * The 47M15x and 47M192 chips "with hardware monitoring block"
  * can do much more besides (device id 0x60).
  */
-#define SMSC_DEVID_MATCH(id) ((id) == 0x51 || (id) == 0x59 || (id) == 0x5F)
+#define SMSC_DEVID_MATCH(id) ((id) == 0x51 || (id) == 0x59 || (id) == 0x5F || (id) == 0x60)
 
 #define SMSC_ACT_REG 0x30
 #define SMSC_BASE_REG 0x60
@@ -163,7 +163,6 @@ static void smsc47m1_pwm(struct i2c_client *client, int operation,
 static int smsc47m1_id = 0;
 
 static struct i2c_driver smsc47m1_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "SMSC 47M1xx fan monitor",
 	.id		= I2C_DRIVERID_SMSC47M1,
 	.flags		= I2C_DF_NOTIFY,
@@ -285,7 +284,8 @@ int smsc47m1_detect(struct i2c_adapter *adapter, int address,
 
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					smsc47m1_dir_table_template)) < 0) {
+					smsc47m1_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
