@@ -23,14 +23,10 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
 #include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
 #include "version.h"
 #include "sensors_vid.h"
@@ -179,6 +175,7 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define FAN_FROM_REG(val,div) ((val)==0?0:(val)==255?0:1310720/((val)*(div)))
 
 struct vt1211_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 
@@ -238,7 +235,6 @@ static void vt1211_temp(struct i2c_client *client, int operation,
 static int vt1211_id = 0;
 
 static struct i2c_driver vt1211_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "VT1211 sensors driver",
 	.id		= I2C_DRIVERID_VT1211,
 	.flags		= I2C_DF_NOTIFY,
@@ -305,10 +301,13 @@ static ctl_table vt1211_dir_table_template[] = {
 	 &i2c_sysctl_real, NULL, &vt1211_in},
 	{VT1211_SYSCTL_IN5, "in5", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt1211_in},
+/*
+    datasheet says these are reserved
 	{VT1211_SYSCTL_IN6, "in6", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt1211_in},
 	{VT1211_SYSCTL_TEMP, "temp1", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt1211_temp},
+*/
 	{VT1211_SYSCTL_TEMP2, "temp2", NULL, 0, 0644, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &vt1211_temp},
 	{VT1211_SYSCTL_TEMP3, "temp3", NULL, 0, 0644, NULL,
@@ -410,13 +409,11 @@ int vt1211_detect(struct i2c_adapter *adapter, int address,
 		superio_outb(VT1211_ACT_REG, 1);
 	superio_exit();
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct vt1211_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct vt1211_data), GFP_KERNEL))) {
 		return -ENOMEM;
 	}
 
-	data = (struct vt1211_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	init_MUTEX(&data->lock);
 	new_client->data = data;
@@ -436,7 +433,8 @@ int vt1211_detect(struct i2c_adapter *adapter, int address,
 
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					vt1211_dir_table_template)) < 0) {
+					vt1211_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -449,7 +447,7 @@ int vt1211_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
 	release_region(address, VT1211_EXTENT);
-	kfree(new_client);
+	kfree(data);
 	return err;
 }
 
@@ -467,7 +465,7 @@ static int vt1211_detach_client(struct i2c_client *client)
 	}
 
 	release_region(client->addr, VT1211_EXTENT);
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -502,7 +500,7 @@ static void vt1211_update_client(struct i2c_client *client)
 	if ((jiffies - data->last_updated > HZ + HZ / 2) ||
 	    (jiffies < data->last_updated) || !data->valid) {
 		data->uch_config = vt_rdval(client, VT1211_REG_UCH_CONFIG);
-		for (i = 0; i <= 6; i++) {
+		for (i = 0; i <= 5; i++) {
 			if(ISVOLT(i, data->uch_config)) {
 				data->in[i] = vt_rdval(client, VT1211_REG_IN(i));
 				data->in_min[i] = vt_rdval(client,
@@ -520,7 +518,7 @@ static void vt1211_update_client(struct i2c_client *client)
 			data->fan_min[i - 1] = vt_rdval(client,
 						     VT1211_REG_FAN_MIN(i));
 		}
-		for (i = 1; i <= 7; i++) {
+		for (i = 2; i <= 7; i++) {
 			if(ISTEMP(i, data->uch_config)) {
 				data->temp[i - 1] = vt_rdval(client,
 					             VT1211_REG_TEMP(i)) << 2;
@@ -783,13 +781,13 @@ void vt1211_uch(struct i2c_client *client, int operation, int ctl_name,
 	if (operation == SENSORS_PROC_REAL_INFO)
 		*nrels_mag = 0;
 	else if (operation == SENSORS_PROC_REAL_READ) {
-		results[0] = data->uch_config;
+		results[0] = data->uch_config & 0x7c;
 		*nrels_mag = 1;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->uch_config = results[0] & 0x7c;
+			data->uch_config = (data->uch_config & 0x83)|(results[0] & 0x7c);
 			vt1211_write_value(client, VT1211_REG_UCH_CONFIG,
-			                   results[0] & 0x7c);
+			                   data->uch_config);
 		}
 	}
 }
