@@ -19,19 +19,15 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-/* Supports VIA VT8231 Super I/O embedded sensors */
+/* Supports VIA VT8231 South Bridge embedded sensors */
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
 #include <linux/ioport.h>
-#include <linux/sysctl.h>
 #include <linux/pci.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
 #include "version.h"
 #include "sensors_vid.h"
@@ -138,6 +134,7 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define FAN_FROM_REG(val,div) ((val)==0?0:(val)==255?0:1310720/((val)*(div)))
 
 struct vt8231_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 
@@ -197,7 +194,6 @@ static void vt8231_temp(struct i2c_client *client, int operation,
 static int vt8231_id = 0;
 
 static struct i2c_driver vt8231_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "VT8231 sensors driver",
 	.id		= I2C_DRIVERID_VT8231,
 	.flags		= I2C_DF_NOTIFY,
@@ -264,10 +260,13 @@ static ctl_table vt8231_dir_table_template[] = {
 	 &i2c_sysctl_real, NULL, &vt8231_in},
 	{VT8231_SYSCTL_IN5, "in5", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt8231_in},
+/*
+    not in 8231
 	{VT8231_SYSCTL_IN6, "in6", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt8231_in},
 	{VT8231_SYSCTL_TEMP, "temp1", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &vt8231_temp},
+*/
 	{VT8231_SYSCTL_TEMP2, "temp2", NULL, 0, 0644, NULL,
 	 &i2c_proc_real, &i2c_sysctl_real, NULL, &vt8231_temp},
 	{VT8231_SYSCTL_TEMP3, "temp3", NULL, 0, 0644, NULL,
@@ -374,14 +373,12 @@ int vt8231_detect(struct i2c_adapter *adapter, int address,
 			return -ENODEV;
 	}
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct vt8231_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct vt8231_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct vt8231_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	init_MUTEX(&data->lock);
 	new_client->data = data;
@@ -406,7 +403,8 @@ int vt8231_detect(struct i2c_adapter *adapter, int address,
 	/* Register a new directory entry with module sensors */
 	if ((i = i2c_register_entry((struct i2c_client *) new_client,
 					type_name,
-					vt8231_dir_table_template)) < 0) {
+					vt8231_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR4;
 	}
@@ -419,7 +417,7 @@ int vt8231_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
 	release_region(address, VIA686A_EXTENT);
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -438,7 +436,7 @@ static int vt8231_detach_client(struct i2c_client *client)
 	}
 
 	release_region(client->addr, VIA686A_EXTENT);
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -474,7 +472,7 @@ static void vt8231_update_client(struct i2c_client *client)
 	if ((jiffies - data->last_updated > HZ + HZ / 2) ||
 	    (jiffies < data->last_updated) || !data->valid) {
 		data->uch_config = vt_rdval(client, VT8231_REG_UCH_CONFIG);
-		for (i = 0; i <= 6; i++) {
+		for (i = 0; i <= 5; i++) {
 			if(ISVOLT(i, data->uch_config)) {
 				data->in[i] = vt_rdval(client, VT8231_REG_IN(i));
 				data->in_min[i] = vt_rdval(client,
@@ -492,7 +490,7 @@ static void vt8231_update_client(struct i2c_client *client)
 			data->fan_min[i - 1] = vt_rdval(client,
 						     VT8231_REG_FAN_MIN(i));
 		}
-		for (i = 1; i <= 7; i++) {
+		for (i = 2; i <= 7; i++) {
 			if(ISTEMP(i, data->uch_config)) {
 				data->temp[i - 1] = vt_rdval(client,
 					             VT8231_REG_TEMP(i)) << 2;
@@ -755,13 +753,13 @@ void vt8231_uch(struct i2c_client *client, int operation, int ctl_name,
 	if (operation == SENSORS_PROC_REAL_INFO)
 		*nrels_mag = 0;
 	else if (operation == SENSORS_PROC_REAL_READ) {
-		results[0] = data->uch_config;
+		results[0] = data->uch_config & 0x7c;
 		*nrels_mag = 1;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->uch_config = results[0] & 0x7c;
+			data->uch_config = (data->uch_config & 0x83)|(results[0] & 0x7c);
 			vt8231_write_value(client, VT8231_REG_UCH_CONFIG,
-			                   results[0] & 0x7c);
+			                   data->uch_config);
 		}
 	}
 }
