@@ -26,27 +26,26 @@
     Chip	#vin	#fanin	#pwm	#temp	wchipid	vendid	i2c	ISA
     w83627hf	9	3	2	3	0x20	0x5ca3	no	yes(LPC)
     w83627thf	7	3	3	3	0x90	0x5ca3	no	yes(LPC)
+    w83637hf	7	3	3	3	0x80	0x5ca3	no	yes(LPC)
     w83697hf	8	2	2	2	0x60	0x5ca3	no	yes(LPC)
 
     For other winbond chips, and for i2c support in the above chips,
-    use w83627hf.c.
+    use w83781d.c.
 
-    Note: automatic ("cruise") fan control for 697 not supported yet.
+    Note: automatic ("cruise") fan control for 697, 637 & 627thf not
+    supported yet.
 */
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
 #include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
 #include "version.h"
 #include "sensors_vid.h"
+#include "lm75.h"
 
 static int force_addr;
 MODULE_PARM(force_addr, "i");
@@ -64,7 +63,7 @@ static unsigned int normal_isa[] = { 0, SENSORS_ISA_END };
 static unsigned int normal_isa_range[] = { SENSORS_ISA_END };
 
 /* Insmod parameters */
-SENSORS_INSMOD_3(w83627hf, w83627thf, w83697hf);
+SENSORS_INSMOD_4(w83627hf, w83627thf, w83697hf, w83637hf);
 
 static int init = 1;
 MODULE_PARM(init, "i");
@@ -74,9 +73,30 @@ MODULE_PARM_DESC(init, "Set to zero to bypass chip initialization");
 #define	REG	0x2e	/* The register to read/write */
 #define	DEV	0x07	/* Register: Logical device select */
 #define	VAL	0x2f	/* The value to read/write */
-#define PME	0x0b	/* The device with the hardware monitor */
+
+/* logical device numbers for superio_select (below) */
+#define W83627HF_LD_FDC		0x00
+#define W83627HF_LD_PRT		0x01
+#define W83627HF_LD_UART1	0x02
+#define W83627HF_LD_UART2	0x03
+#define W83627HF_LD_KBC		0x05
+#define W83627HF_LD_CIR		0x06 /* w83627hf only */
+#define W83627HF_LD_GAME	0x07
+#define W83627HF_LD_MIDI	0x07
+#define W83627HF_LD_GPIO1	0x07
+#define W83627HF_LD_GPIO5	0x07 /* w83627thf only */
+#define W83627HF_LD_GPIO2	0x08
+#define W83627HF_LD_GPIO3	0x09
+#define W83627HF_LD_GPIO4	0x09 /* w83627thf only */
+#define W83627HF_LD_ACPI	0x0a
+#define W83627HF_LD_HWM		0x0b
+
 #define	DEVID	0x20	/* Register: Device ID */
 
+#define W83627THF_GPIO5_IOSR	0xf3 /* w83627thf only */
+#define W83627THF_GPIO5_DR	0xf4 /* w83627thf only */
+#define W83627THF_GPIO5_INVR	0xf5 /* w83627thf only */
+
 static inline void
 superio_outb(int reg, int val)
 {
@@ -92,10 +112,10 @@ superio_inb(int reg)
 }
 
 static inline void
-superio_select(void)
+superio_select(int ld)
 {
 	outb(DEV, REG);
-	outb(PME, VAL);
+	outb(ld, VAL);
 }
 
 static inline void
@@ -114,6 +134,7 @@ superio_exit(void)
 #define W627_DEVID 0x52
 #define W627THF_DEVID 0x82
 #define W697_DEVID 0x60
+#define W637_DEVID 0x70
 #define WINB_ACT_REG 0x30
 #define WINB_BASE_REG 0x60
 /* Constants specified below */
@@ -171,40 +192,42 @@ superio_exit(void)
 
 #define W83781D_REG_VBAT 0x5D
 
-#define W83781D_REG_PWM1 0x5B	/* 782d and 783s/627hf datasheets disagree */
-				/* on which is which; */
-#define W83781D_REG_PWM2 0x5A	/* We follow the 782d convention here, */
-				/* However 782d is probably wrong. */
-#define W83781D_REG_PWMCLK12 0x5C
-#define W83627THF_REG_PWM1 0x01		/* 697HF too */
-#define W83627THF_REG_PWM2 0x03		/* 697HF too */
-#define W83627THF_REG_PWM3 0x11
-#define W83697HF_REG_PWMCLK1 0x00
-#define W83697HF_REG_PWMCLK2 0x02
-static const u8 regpwm[] = { W83781D_REG_PWM1, W83781D_REG_PWM2 };
-static const u8 regpwm_thf[] = { W83627THF_REG_PWM1, W83627THF_REG_PWM2,
-                                 W83627THF_REG_PWM3 };
+#define W83627HF_REG_PWM1 0x5A
+#define W83627HF_REG_PWM2 0x5B
+#define W83627HF_REG_PWMCLK12 0x5C
+
+#define W83627THF_REG_PWM1		0x01	/* 697HF and 637HF too */
+#define W83627THF_REG_PWM2		0x03	/* 697HF and 637HF too */
+#define W83627THF_REG_PWM3		0x11	/* 637HF too */
+
+#define W83627THF_REG_VRM_OVT_CFG 	0x18	/* 637HF too */
+
+static const u8 regpwm_627hf[] = { W83627HF_REG_PWM1, W83627HF_REG_PWM2 };
+static const u8 regpwm[] = { W83627THF_REG_PWM1, W83627THF_REG_PWM2,
+                             W83627THF_REG_PWM3 };
 #define W836X7HF_REG_PWM(type, nr) (((type) == w83627hf) ? \
-                                     regpwm[(nr) - 1] : regpwm_thf[(nr) - 1])
+                                     regpwm_627hf[(nr) - 1] : regpwm[(nr) - 1])
 
 #define W83781D_REG_I2C_ADDR 0x48
 #define W83781D_REG_I2C_SUBADDR 0x4A
 
-/* The following are undocumented in the data sheets however we
-   received the information in an email from Winbond tech support */
-/* Sensor selection - not on 781d */
+/* Sensor selection */
 #define W83781D_REG_SCFG1 0x5D
 static const u8 BIT_SCFG1[] = { 0x02, 0x04, 0x08 };
 #define W83781D_REG_SCFG2 0x59
 static const u8 BIT_SCFG2[] = { 0x10, 0x20, 0x40 };
 #define W83781D_DEFAULT_BETA 3435
 
-/* Conversions. Rounding and limit checking is only done on the TO_REG
+/* Conversions. Limit checking is only done on the TO_REG
    variants. Note that you should be a bit careful with which arguments
    these macros are called: arguments may be evaluated more than once.
    Fixing this is just not worth it. */
 #define IN_TO_REG(val)  (SENSORS_LIMIT((((val) * 10 + 8)/16),0,255))
-#define IN_FROM_REG(val) (((val) * 16) / 10)
+#define IN_FROM_REG(val) (((val) * 16 + 5) / 10)
+
+#define IN_TO_REG_VRM9(val) \
+	(SENSORS_LIMIT((((val) * 1000 - 70000 + 244) / 488), 0, 255))
+#define IN_FROM_REG_VRM9(reg)	(((reg) * 488 + 70000 + 500) / 1000)
 
 static inline u8 FAN_TO_REG(long rpm, int div)
 {
@@ -215,16 +238,25 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 			     254);
 }
 
+#define TEMP_MIN (-1280)
+#define TEMP_MAX ( 1270)
+
+/* TEMP: 1/10 degrees C (-128C to +127C)
+   REG: 1C/bit, two's complement */
+static u8 TEMP_TO_REG(int temp)
+{
+        int ntemp = SENSORS_LIMIT(temp, TEMP_MIN, TEMP_MAX);
+        ntemp += (ntemp<0 ? -5 : 5);
+        return (u8)(ntemp / 10);
+}
+                                                                                
+static int TEMP_FROM_REG(u8 reg)
+{
+        return (s8)reg * 10;
+}
+
 #define FAN_FROM_REG(val,div) ((val)==0?-1:(val)==255?0:1350000/((val)*(div)))
 
-#define TEMP_TO_REG(val) (SENSORS_LIMIT(((val)<0?(((val)-5)/10):\
-                                                 ((val)+5)/10),0,255))
-#define TEMP_FROM_REG(val) (((val)>0x80?(val)-0x100:(val))*10)
-
-#define TEMP_ADD_TO_REG(val)   (SENSORS_LIMIT(((((val) + 2) / 5) << 7),\
-                                              0,0xffff))
-#define TEMP_ADD_FROM_REG(val) (((val) >> 7) * 5)
-
 #define PWM_TO_REG(val) (SENSORS_LIMIT((val),0,255))
 #define BEEPS_TO_REG(val) ((val) & 0xffffff)
 
@@ -245,124 +277,11 @@ static inline u8 DIV_TO_REG(long val)
 	return ((u8) i);
 }
 
-/* Initial limits */
-#define W83781D_INIT_IN_0 (vid==3500?280:vid/10)
-#define W83781D_INIT_IN_1 (vid==3500?280:vid/10)
-#define W83781D_INIT_IN_2 330
-#define W83781D_INIT_IN_3 (((500)   * 100)/168)
-#define W83781D_INIT_IN_4 (((1200)  * 10)/38)
-#define W83781D_INIT_IN_5 (((-1200) * -604)/2100)
-#define W83781D_INIT_IN_6 (((-500)  * -604)/909)
-#define W83781D_INIT_IN_7 (((500)   * 100)/168)
-#define W83781D_INIT_IN_8 300
-/* Initial limits for 782d/783s negative voltages */
-/* Note level shift. Change min/max below if you change these. */
-#define W83782D_INIT_IN_5 ((((-1200) + 1491) * 100)/514)
-#define W83782D_INIT_IN_6 ((( (-500)  + 771) * 100)/314)
-
-#define W83781D_INIT_IN_PERCENTAGE 10
-
-#define W83781D_INIT_IN_MIN_0 \
-        (W83781D_INIT_IN_0 - W83781D_INIT_IN_0 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_0 \
-        (W83781D_INIT_IN_0 + W83781D_INIT_IN_0 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_1 \
-        (W83781D_INIT_IN_1 - W83781D_INIT_IN_1 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_1 \
-        (W83781D_INIT_IN_1 + W83781D_INIT_IN_1 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_2 \
-        (W83781D_INIT_IN_2 - W83781D_INIT_IN_2 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_2 \
-        (W83781D_INIT_IN_2 + W83781D_INIT_IN_2 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_3 \
-        (W83781D_INIT_IN_3 - W83781D_INIT_IN_3 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_3 \
-        (W83781D_INIT_IN_3 + W83781D_INIT_IN_3 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_4 \
-        (W83781D_INIT_IN_4 - W83781D_INIT_IN_4 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_4 \
-        (W83781D_INIT_IN_4 + W83781D_INIT_IN_4 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_5 \
-        (W83781D_INIT_IN_5 - W83781D_INIT_IN_5 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_5 \
-        (W83781D_INIT_IN_5 + W83781D_INIT_IN_5 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_6 \
-        (W83781D_INIT_IN_6 - W83781D_INIT_IN_6 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_6 \
-        (W83781D_INIT_IN_6 + W83781D_INIT_IN_6 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_7 \
-        (W83781D_INIT_IN_7 - W83781D_INIT_IN_7 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_7 \
-        (W83781D_INIT_IN_7 + W83781D_INIT_IN_7 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_8 \
-        (W83781D_INIT_IN_8 - W83781D_INIT_IN_8 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_8 \
-        (W83781D_INIT_IN_8 + W83781D_INIT_IN_8 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-/* Initial limits for 782d/783s negative voltages */
-/* These aren't direct multiples because of level shift */
-/* Beware going negative - check */
-#define W83782D_INIT_IN_MIN_5_TMP \
-        (((-1200 * (100 + W83781D_INIT_IN_PERCENTAGE)) + (1491 * 100))/514)
-#define W83782D_INIT_IN_MIN_5 \
-        ((W83782D_INIT_IN_MIN_5_TMP > 0) ? W83782D_INIT_IN_MIN_5_TMP : 0)
-#define W83782D_INIT_IN_MAX_5 \
-        (((-1200 * (100 - W83781D_INIT_IN_PERCENTAGE)) + (1491 * 100))/514)
-#define W83782D_INIT_IN_MIN_6_TMP \
-        ((( -500 * (100 + W83781D_INIT_IN_PERCENTAGE)) +  (771 * 100))/314)
-#define W83782D_INIT_IN_MIN_6 \
-        ((W83782D_INIT_IN_MIN_6_TMP > 0) ? W83782D_INIT_IN_MIN_6_TMP : 0)
-#define W83782D_INIT_IN_MAX_6 \
-        ((( -500 * (100 - W83781D_INIT_IN_PERCENTAGE)) +  (771 * 100))/314)
-
-#define W83781D_INIT_FAN_MIN_1 3000
-#define W83781D_INIT_FAN_MIN_2 3000
-#define W83781D_INIT_FAN_MIN_3 3000
-
-#define W83781D_INIT_TEMP_OVER 600
-#define W83781D_INIT_TEMP_HYST 1270	/* must be 127 for ALARM to work */
-#define W83781D_INIT_TEMP2_OVER 600
-#define W83781D_INIT_TEMP2_HYST 500
-#define W83781D_INIT_TEMP3_OVER 600
-#define W83781D_INIT_TEMP3_HYST 500
-
-/* There are some complications in a module like this. First off, W83781D chips
-   may be both present on the SMBus and the ISA bus, and we have to handle
-   those cases separately at some places. Second, there might be several
-   W83781D chips available (well, actually, that is probably never done; but
-   it is a clean illustration of how to handle a case like that). Finally,
-   a specific chip may be attached to *both* ISA and SMBus, and we would
-   not like to detect it double. Fortunately, in the case of the W83781D at
-   least, a register tells us what SMBus address we are on, so that helps
-   a bit - except if there could be more than one SMBus. Groan. No solution
-   for this yet. */
-
-/* This module may seem overly long and complicated. In fact, it is not so
-   bad. Quite a lot of bookkeeping is done. A real driver can often cut
-   some corners. */
-
-/* For each registered W83781D, we need to keep some data in memory. That
+/* For each registered chip, we need to keep some data in memory. That
    data is pointed to by w83627hf_list[NR]->data. The structure itself is
-   dynamically allocated, at the same time when a new w83627hf client is
-   allocated. */
+   dynamically allocated, at the same time when a new client is allocated. */
 struct w83627hf_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 	enum chips type;
@@ -374,9 +293,9 @@ struct w83627hf_data {
 	struct i2c_client *lm75;	/* for secondary I2C addresses */
 	/* pointer to array of 2 subclients */
 
-	u8 in[9];		/* Register value - 8 & 9 for 782D only */
-	u8 in_max[9];		/* Register value - 8 & 9 for 782D only */
-	u8 in_min[9];		/* Register value - 8 & 9 for 782D only */
+	u8 in[9];		/* Register value */
+	u8 in_max[9];		/* Register value */
+	u8 in_min[9];		/* Register value */
 	u8 fan[3];		/* Register value */
 	u8 fan_min[3];		/* Register value */
 	u8 temp;
@@ -398,6 +317,7 @@ struct w83627hf_data {
 				   Default = 3435. 
 				   Other Betas unimplemented */
 	u8 vrm;
+	u8 vrm_ovt;		/* Register value, 627thf & 637hf only */
 };
 
 
@@ -439,8 +359,7 @@ static void w83627hf_sens(struct i2c_client *client, int operation,
 static int w83627hf_id = 0;
 
 static struct i2c_driver w83627hf_driver = {
-	.owner		= THIS_MODULE,
-	.name		= "W83781D sensor driver",
+	.name		= "W83627HF sensor driver",
 	.id		= I2C_DRIVERID_W83627HF,
 	.flags		= I2C_DF_NOTIFY,
 	.attach_adapter	= w83627hf_attach_adapter,
@@ -608,6 +527,7 @@ static ctl_table w83697hf_dir_table_template[] = {
 };
 
 /* no in5 and in6 */
+/* We use this one for W83637HF too */
 static ctl_table w83627thf_dir_table_template[] = {
 	{W83781D_SYSCTL_IN0, "in0", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &w83627hf_in},
@@ -676,12 +596,12 @@ static int w83627hf_find(int *address)
 
 	superio_enter();
 	val= superio_inb(DEVID);
-	if(val != W627_DEVID && val !=W627THF_DEVID && val != W697_DEVID) {
+	if(val != W627_DEVID && val !=W627THF_DEVID && val != W697_DEVID && val != W637_DEVID) {
 		superio_exit();
 		return -ENODEV;
 	}
 
-	superio_select();
+	superio_select(W83627HF_LD_HWM);
 	val = (superio_inb(WINB_BASE_REG) << 8) |
 	       superio_inb(WINB_BASE_REG + 1);
 	*address = val & ~(WINB_EXTENT - 1);
@@ -719,7 +639,7 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
 	if(force_addr) {
 		printk("w83627hf.o: forcing ISA address 0x%04X\n", address);
 		superio_enter();
-		superio_select();
+		superio_select(W83627HF_LD_HWM);
 		superio_outb(WINB_BASE_REG, address >> 8);
 		superio_outb(WINB_BASE_REG+1, address & 0xff);
 		superio_exit();
@@ -733,8 +653,10 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
 		kind = w83697hf;
 	else if(val == W627THF_DEVID)
 		kind = w83627thf;
+	else if(val == W637_DEVID)
+		kind = w83637hf;
 		
-	superio_select();
+	superio_select(W83627HF_LD_HWM);
 	if((val = 0x01 & superio_inb(WINB_ACT_REG)) == 0)
 		superio_outb(WINB_ACT_REG, 1);
 	superio_exit();
@@ -743,14 +665,12 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access w83627hf_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct w83627hf_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct w83627hf_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct w83627hf_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	init_MUTEX(&data->lock);
 	new_client->data = data;
@@ -768,6 +688,9 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
 	} else if (kind == w83697hf) {
 		type_name = "w83697hf";
 		client_name = "W83697HF chip";
+	} else if (kind == w83637hf) {
+		type_name = "w83637hf";
+		client_name = "W83637HF chip";
 	} else {
 		goto ERROR1;
 	}
@@ -794,7 +717,9 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
 				   w83697hf_dir_table_template :
 				(kind == w83627hf) ?
 				   w83627hf_dir_table_template :
-				   w83627thf_dir_table_template)) < 0) {
+				   /* w83627thf table also used for 637 */
+				   w83627thf_dir_table_template,
+				THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR7;
 	}
@@ -812,7 +737,7 @@ int w83627hf_detect(struct i2c_adapter *adapter, int address,
       ERROR3:
 	release_region(address, WINB_EXTENT);
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -831,7 +756,7 @@ static int w83627hf_detach_client(struct i2c_client *client)
 	}
 
 	release_region(client->addr, WINB_EXTENT);
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -877,6 +802,20 @@ static int w83627hf_read_value(struct i2c_client *client, u16 reg)
 	return res;
 }
 
+static int w83627thf_read_gpio5(struct i2c_client *client)
+{
+	int res, inv;
+
+	down(&(((struct w83627hf_data *) (client->data))->lock));
+	superio_enter();
+	superio_select(W83627HF_LD_GPIO5);
+	res = superio_inb(W83627THF_GPIO5_DR);
+	inv = superio_inb(W83627THF_GPIO5_INVR);
+	superio_exit();
+	up(&(((struct w83627hf_data *) (client->data))->lock));
+	return res;
+}
+
 static int w83627hf_write_value(struct i2c_client *client, u16 reg, u16 value)
 {
 	int word_sized;
@@ -914,39 +853,35 @@ static int w83627hf_write_value(struct i2c_client *client, u16 reg, u16 value)
 static void w83627hf_init_client(struct i2c_client *client)
 {
 	struct w83627hf_data *data = client->data;
-	int vid = 0, i;
+	int i;
 	int type = data->type;
 	u8 tmp;
 
-	if(init) {
-		/* save this register */
-		i = w83627hf_read_value(client, W83781D_REG_BEEP_CONFIG);
-		/* Reset all except Watchdog values and last conversion values
-		   This sets fan-divs to 2, among others */
-		w83627hf_write_value(client, W83781D_REG_CONFIG, 0x80);
-		/* Restore the register and disable power-on abnormal beep.
-		   This saves FAN 1/2/3 input/output values set by BIOS. */
-		w83627hf_write_value(client, W83781D_REG_BEEP_CONFIG, i | 0x80);
-		/* Disable master beep-enable (reset turns it on).
-		   Individual beeps should be reset to off but for some reason
-		   disabling this bit helps some people not get beeped */
-		w83627hf_write_value(client, W83781D_REG_BEEP_INTS2, 0);
-	}
-
 	/* Minimize conflicts with other winbond i2c-only clients...  */
 	/* disable i2c subclients... how to disable main i2c client?? */
 	/* force i2c address to relatively uncommon address */
 	w83627hf_write_value(client, W83781D_REG_I2C_SUBADDR, 0x89);
 	w83627hf_write_value(client, W83781D_REG_I2C_ADDR, force_i2c);
 
-	if (type != w83697hf) {
-		vid = w83627hf_read_value(client, W83781D_REG_VID_FANDIV) & 0x0f;
-		vid |=
-		    (w83627hf_read_value(client, W83781D_REG_CHIPID) & 0x01) << 4;
-		data->vrm = DEFAULT_VRM;
-		vid = vid_from_reg(vid, data->vrm);
+	/* Read VID only once */
+	if (w83627hf == data->type || w83637hf == data->type) {
+		int lo = w83627hf_read_value(client, W83781D_REG_VID_FANDIV);
+		int hi = w83627hf_read_value(client, W83781D_REG_CHIPID);
+		data->vid = (lo & 0x0f) | ((hi & 0x01) << 4);
+	} else if (w83627thf == data->type) {
+		data->vid = w83627thf_read_gpio5(client) & 0x1f;
 	}
 
+	/* Read VRM & OVT Config only once */
+	if (w83627thf == data->type || w83637hf == data->type)
+		data->vrm_ovt =
+			w83627hf_read_value(client, W83627THF_REG_VRM_OVT_CFG);
+	else
+		data->vrm_ovt = 0;
+
+	/* Choose VRM based on "VRM & OVT" register */
+	data->vrm = (data->vrm_ovt & 0x01) ? 90 : 82;
+
 	tmp = w83627hf_read_value(client, W83781D_REG_SCFG1);
 	for (i = 1; i <= 3; i++) {
 		if (!(tmp & BIT_SCFG1[i - 1])) {
@@ -968,87 +903,17 @@ static void w83627hf_init_client(struct i2c_client *client)
 	data->pwmenable[2] = 1;
 
 	if(init) {
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(0),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_0));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(0),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_0));
-		if (type != w83697hf) {
-			w83627hf_write_value(client, W83781D_REG_IN_MIN(1),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_1));
-			w83627hf_write_value(client, W83781D_REG_IN_MAX(1),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_1));
-		}
-
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(2),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_2));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(2),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_2));
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(3),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_3));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(3),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_3));
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(4),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_4));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(4),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_4));
-		if (type != w83627thf) {
-			w83627hf_write_value(client, W83781D_REG_IN_MIN(5),
-					    IN_TO_REG(W83782D_INIT_IN_MIN_5));
-			w83627hf_write_value(client, W83781D_REG_IN_MAX(5),
-					    IN_TO_REG(W83782D_INIT_IN_MAX_5));
-			w83627hf_write_value(client, W83781D_REG_IN_MIN(6),
-					    IN_TO_REG(W83782D_INIT_IN_MIN_6));
-			w83627hf_write_value(client, W83781D_REG_IN_MAX(6),
-					    IN_TO_REG(W83782D_INIT_IN_MAX_6));
-		}
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(7),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_7));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(7),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_7));
-		w83627hf_write_value(client, W83781D_REG_IN_MIN(8),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_8));
-		w83627hf_write_value(client, W83781D_REG_IN_MAX(8),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_8));
-		w83627hf_write_value(client, W83781D_REG_VBAT,
-		    (w83627hf_read_value(client, W83781D_REG_VBAT) | 0x01));
-		w83627hf_write_value(client, W83781D_REG_FAN_MIN(1),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_1, 2));
-		w83627hf_write_value(client, W83781D_REG_FAN_MIN(2),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_2, 2));
-		if (type != w83697hf) {
-			w83627hf_write_value(client, W83781D_REG_FAN_MIN(3),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_3, 2));
-		}
-
-		w83627hf_write_value(client, W83781D_REG_TEMP_OVER,
-				    TEMP_TO_REG(W83781D_INIT_TEMP_OVER));
-		w83627hf_write_value(client, W83781D_REG_TEMP_HYST,
-				    TEMP_TO_REG(W83781D_INIT_TEMP_HYST));
-
-		w83627hf_write_value(client, W83781D_REG_TEMP2_OVER,
-				    TEMP_ADD_TO_REG
-				    (W83781D_INIT_TEMP2_OVER));
-		w83627hf_write_value(client, W83781D_REG_TEMP2_HYST,
-				    TEMP_ADD_TO_REG
-				    (W83781D_INIT_TEMP2_HYST));
-		w83627hf_write_value(client, W83781D_REG_TEMP2_CONFIG, 0x00);
-
-		if (type != w83697hf) {
-			w83627hf_write_value(client, W83781D_REG_TEMP3_OVER,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_OVER));
-			w83627hf_write_value(client, W83781D_REG_TEMP3_HYST,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_HYST));
-			w83627hf_write_value(client, W83781D_REG_TEMP3_CONFIG,
-					    0x00);
+		if (type == w83627hf) {
 			/* enable PWM2 control (can't hurt since PWM reg
 		           should have been reset to 0xff) */
-			w83627hf_write_value(client, W83781D_REG_PWMCLK12, 0x19);
+			w83627hf_write_value(client, W83627HF_REG_PWMCLK12, 0x19);
 		}
 		/* enable comparator mode for temp2 and temp3 so
 	           alarm indication will work correctly */
-		w83627hf_write_value(client, W83781D_REG_IRQ, 0x41);
+		i = w83627hf_read_value(client, W83781D_REG_IRQ);
+		if (!(i & 0x40))
+			w83627hf_write_value(client, W83781D_REG_IRQ,
+					    i | 0x40);
 	}
 
 	/* Start monitoring */
@@ -1070,7 +935,8 @@ static void w83627hf_update_client(struct i2c_client *client)
 		for (i = 0; i <= 8; i++) {
 			/* skip missing sensors */
 			if (((data->type == w83697hf) && (i == 1)) ||
-			    ((data->type == w83627thf) && ((i == 4) || (i == 5))))
+			    ((data->type == w83627thf || data->type == w83637hf) &&
+			     (i == 4 || i == 5)))
 				continue;
 			data->in[i] =
 			    w83627hf_read_value(client, W83781D_REG_IN(i));
@@ -1089,10 +955,12 @@ static void w83627hf_update_client(struct i2c_client *client)
 					       W83781D_REG_FAN_MIN(i));
 		}
 		for (i = 1; i <= 3; i++) {
-			data->pwm[i - 1] =
-			    w83627hf_read_value(client,
-					       W836X7HF_REG_PWM(data->type, i));
-			if(i == 2 && data->type != w83627thf)
+			u8 tmp = w83627hf_read_value(client,
+				W836X7HF_REG_PWM(data->type, i));
+			if (data->type == w83627thf)
+				tmp &= 0xf0; /* bits 0-3 are reserved  in 627THF */
+			data->pwm[i - 1] = tmp;
+			if(i == 2 && (data->type == w83627hf || data->type == w83697hf))
 				break;
 		}
 
@@ -1115,13 +983,8 @@ static void w83627hf_update_client(struct i2c_client *client)
 			data->temp_add_hyst[1] =
 			    w83627hf_read_value(client, W83781D_REG_TEMP3_HYST);
 		}
+
 		i = w83627hf_read_value(client, W83781D_REG_VID_FANDIV);
-		if (data->type != w83697hf) {
-			data->vid = i & 0x0f;
-			data->vid |=
-			    (w83627hf_read_value(client, W83781D_REG_CHIPID) & 0x01)
-			    << 4;
-		}
 		data->fan_div[0] = (i >> 4) & 0x03;
 		data->fan_div[1] = (i >> 6) & 0x03;
 		if (data->type != w83697hf) {
@@ -1159,18 +1022,42 @@ void w83627hf_in(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 2;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		w83627hf_update_client(client);
-		results[0] = IN_FROM_REG(data->in_min[nr]);
-		results[1] = IN_FROM_REG(data->in_max[nr]);
-		results[2] = IN_FROM_REG(data->in[nr]);
+
+		if (nr == 0 && (data->vrm_ovt & 0x01)) {
+			/* use VRM9 calculation */
+			results[0] = IN_FROM_REG_VRM9(data->in_min[0]);
+			results[1] = IN_FROM_REG_VRM9(data->in_max[0]);
+			results[2] = IN_FROM_REG_VRM9(data->in[0]);
+
+		} else {
+			/* use VRM8 (standard) calculation */
+			results[0] = IN_FROM_REG(data->in_min[nr]);
+			results[1] = IN_FROM_REG(data->in_max[nr]);
+			results[2] = IN_FROM_REG(data->in[nr]);
+		}
+
 		*nrels_mag = 3;
+
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->in_min[nr] = IN_TO_REG(results[0]);
+			if (nr == 0 && (data->vrm_ovt & 0x01))
+				/* use VRM9 calculation */
+				data->in_min[0] = IN_TO_REG_VRM9(results[0]);
+			else
+				/* use VRM8 (standard) calculation */
+				data->in_min[nr] = IN_TO_REG(results[0]);
+
 			w83627hf_write_value(client, W83781D_REG_IN_MIN(nr),
 					    data->in_min[nr]);
 		}
 		if (*nrels_mag >= 2) {
-			data->in_max[nr] = IN_TO_REG(results[1]);
+			if (nr == 0 && (data->vrm_ovt & 0x01))
+				/* use VRM9 calculation */
+				data->in_min[0] = IN_TO_REG_VRM9(results[1]);
+			else
+				/* use VRM8 (standard) calculation */
+				data->in_max[nr] = IN_TO_REG(results[1]);
+
 			w83627hf_write_value(client, W83781D_REG_IN_MAX(nr),
 					    data->in_max[nr]);
 		}
@@ -1241,15 +1128,15 @@ void w83627hf_temp_add(struct i2c_client *client, int operation,
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		w83627hf_update_client(client);
 			results[0] =
-			    TEMP_ADD_FROM_REG(data->temp_add_over[nr]);
+			    LM75_TEMP_FROM_REG(data->temp_add_over[nr]);
 			results[1] =
-			    TEMP_ADD_FROM_REG(data->temp_add_hyst[nr]);
-			results[2] = TEMP_ADD_FROM_REG(data->temp_add[nr]);
+			    LM75_TEMP_FROM_REG(data->temp_add_hyst[nr]);
+			results[2] = LM75_TEMP_FROM_REG(data->temp_add[nr]);
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
 				data->temp_add_over[nr] =
-				    TEMP_ADD_TO_REG(results[0]);
+				    LM75_TEMP_TO_REG(results[0]);
 			w83627hf_write_value(client,
 					    nr ? W83781D_REG_TEMP3_OVER :
 					    W83781D_REG_TEMP2_OVER,
@@ -1257,7 +1144,7 @@ void w83627hf_temp_add(struct i2c_client *client, int operation,
 		}
 		if (*nrels_mag >= 2) {
 				data->temp_add_hyst[nr] =
-				    TEMP_ADD_TO_REG(results[1]);
+				    LM75_TEMP_TO_REG(results[1]);
 			w83627hf_write_value(client,
 					    nr ? W83781D_REG_TEMP3_HYST :
 					    W83781D_REG_TEMP2_HYST,
@@ -1421,10 +1308,20 @@ void w83627hf_pwm(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 2;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->pwm[nr - 1] = PWM_TO_REG(results[0]);
-			w83627hf_write_value(client,
-			                     W836X7HF_REG_PWM(data->type, nr),
-					     data->pwm[nr - 1]);
+			if (data->type == w83627thf) {
+				/* bits 0-3 are reserved  in 627THF */
+				data->pwm[nr - 1] = PWM_TO_REG(results[0]) & 0xf0;
+				w83627hf_write_value(client,
+						     W836X7HF_REG_PWM(data->type, nr),
+						     data->pwm[nr - 1] |
+						     (w83627hf_read_value(client,
+						      W836X7HF_REG_PWM(data->type, nr)) & 0x0f));
+			} else {
+				data->pwm[nr - 1] = PWM_TO_REG(results[0]);
+				w83627hf_write_value(client,
+						     W836X7HF_REG_PWM(data->type, nr),
+						     data->pwm[nr - 1]);
+			}
 		}
 	}
 }
