@@ -332,7 +332,6 @@ static void w83781d_rt(struct i2c_client *client, int operation,
 static int w83781d_id = 0;
 
 static struct i2c_driver w83781d_driver = {
-	.owner		= THIS_MODULE,
 	.name		= "W83781D sensor driver",
 	.id		= I2C_DRIVERID_W83781D,
 	.flags		= I2C_DF_NOTIFY,
@@ -1050,7 +1049,8 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
                                             w83791d_dir_table_template :
                                         (is_isa || kind == w83627hf) ?
 					   w83782d_isa_dir_table_template :
-					   w83782d_i2c_dir_table_template)) < 0) {
+					   w83782d_i2c_dir_table_template,
+					THIS_MODULE)) < 0) {
 		err = i;
 		goto ERROR7;
 	}
