@@ -184,6 +184,38 @@
 #define SENSORS_ADM1021_ALARMS 81 /* R */
 #define SENSORS_ADM1021_DIE_CODE 90 /* R */
 
+
+/* ADM1030 and ADM1031 chips */
+
+#define SENSORS_ADM1030_PREFIX "adm1030"
+#define SENSORS_ADM1031_PREFIX "adm1031"
+
+#define SENSORS_ADM1031_TEMP1      51 /* R */
+#define SENSORS_ADM1031_TEMP1_MIN  52 /* RW */
+#define SENSORS_ADM1031_TEMP1_MAX  53 /* RW */
+#define SENSORS_ADM1031_TEMP1_CRIT 54 /* RW */
+
+#define SENSORS_ADM1031_TEMP2      61 /* R */
+#define SENSORS_ADM1031_TEMP2_MIN  62 /* RW */
+#define SENSORS_ADM1031_TEMP2_MAX  63 /* RW */
+#define SENSORS_ADM1031_TEMP2_CRIT 64 /* RW */
+
+#define SENSORS_ADM1031_TEMP3      71 /* R */
+#define SENSORS_ADM1031_TEMP3_MIN  72 /* RW */
+#define SENSORS_ADM1031_TEMP3_MAX  73 /* RW */
+#define SENSORS_ADM1031_TEMP3_CRIT 74 /* RW */
+
+
+#define SENSORS_ADM1031_FAN1       80 /* R */
+#define SENSORS_ADM1031_FAN1_MIN   81 /* RW */
+#define SENSORS_ADM1031_FAN1_DIV   82 /* RW */
+#define SENSORS_ADM1031_FAN2       90 /* R */
+#define SENSORS_ADM1031_FAN2_MIN   91 /* RW */
+#define SENSORS_ADM1031_FAN2_DIV   92 /* RW */
+
+#define SENSORS_ADM1031_ALARMS    100 /* R */
+
+
 /* MAX1617 chips. */
 
 #define SENSORS_MAX1617_PREFIX "max1617"
@@ -443,9 +475,13 @@
 #define SENSORS_LM85_ZONE3_RANGE           100  /* RW -- zone3_range  */
 #define SENSORS_LM85_ZONE3_SMOOTH          101  /* RW -- zone3_smooth  */
 
-/* LM90 chips */
+/* LM86/LM89/LM90/LM99/ADM1032/MAX6657 chips */
 
 #define SENSORS_LM90_PREFIX "lm90"
+#define SENSORS_ADM1032_PREFIX "adm1032"
+#define SENSORS_LM99_PREFIX "lm99"
+#define SENSORS_LM86_PREFIX "lm86"
+#define SENSORS_MAX6657_PREFIX "max6657"
 
 #define SENSORS_LM90_LOCAL_TEMP 51 /* R */
 #define SENSORS_LM90_LOCAL_HIGH 52 /* RW */
@@ -455,9 +491,27 @@
 #define SENSORS_LM90_REMOTE_HIGH 58 /* RW */
 #define SENSORS_LM90_REMOTE_LOW 59 /* RW */
 #define SENSORS_LM90_REMOTE_TCRIT 60 /* RW */
-#define SENSORS_LM90_TCRIT_HYST 79 /* RW */
+#define SENSORS_LM90_LOCAL_TCRIT_HYST 79 /* RW */
+#define SENSORS_LM90_REMOTE_TCRIT_HYST 80 /* R, see driver source */
 #define SENSORS_LM90_ALARMS 81 /* R */
 
+/* LM63 chips */
+
+#define SENSORS_LM63_PREFIX "lm63"
+
+#define SENSORS_LM63_LOCAL_TEMP			51 /* R  */
+#define SENSORS_LM63_LOCAL_HIGH			52 /* RW */
+#define SENSORS_LM63_REMOTE_TEMP		57 /* R  */
+#define SENSORS_LM63_REMOTE_HIGH		58 /* RW */
+#define SENSORS_LM63_REMOTE_LOW			59 /* RW */
+#define SENSORS_LM63_REMOTE_TCRIT		60 /* R  */
+#define SENSORS_LM63_REMOTE_TCRIT_HYST		80 /* RW */
+#define SENSORS_LM63_ALARMS			81 /* R  */
+#define SENSORS_LM63_FAN			84 /* R  */
+#define SENSORS_LM63_FAN_LOW			85 /* RW */
+#define SENSORS_LM63_PWM			87 /* RW */
+#define SENSORS_LM63_PWM_ENABLE			88 /* RW */
+
 /* Winbond W83781D chips */
 
 #define SENSORS_W83781D_PREFIX "w83781d"
@@ -517,6 +571,8 @@
 
 #define SENSORS_W83782D_PREFIX "w83782d"
 #define SENSORS_W83627HF_PREFIX "w83627hf"
+#define SENSORS_W83627THF_PREFIX "w83627thf"
+#define SENSORS_W83637HF_PREFIX "w83637hf"
 
 #define SENSORS_W83791D_PREFIX "w83791d"
 
@@ -702,6 +758,14 @@
 #define SENSORS_W83783S_SENS2 92 /* RW */
 
 
+/* W83L785TS-S chips */
+
+#define SENSORS_W83L785TS_PREFIX "w83l785ts"
+
+#define SENSORS_W83L785TS_TEMP		51 /* R */
+#define SENSORS_W83L785TS_TEMP_OVER	52 /* R for now */
+
+
 /* Analog Devices ADM9240 chips */
 
 #define SENSORS_ADM9240_PREFIX "adm9240"
@@ -955,6 +1019,7 @@
 /* ADM1025 chip */
 
 #define SENSORS_ADM1025_PREFIX "adm1025"
+#define SENSORS_NE1619_PREFIX "ne1619"
 
 #define SENSORS_ADM1025_IN0 1 /* R */
 #define SENSORS_ADM1025_IN1 2 /* R */
@@ -1197,6 +1262,9 @@
 #define SENSORS_EEPROM_EDID_HEADER 32
 /* 33 to 39: reserved, do not use! */
 
+#define SENSORS_EEPROM_SHUTTLE 40
+/* 41 to 45: reserved, do not use! */
+
 /* Analog Devices LM87 chips */
 
 #define SENSORS_LM87_PREFIX "lm87"
@@ -1348,6 +1416,7 @@
 /* IT87xx chips */
 
 #define SENSORS_IT87_PREFIX "it87"
+#define SENSORS_IT8712_PREFIX "it8712"
 
 #define SENSORS_IT87_IN0 1 /* R */
 #define SENSORS_IT87_IN1 2 /* R */
@@ -1397,6 +1466,9 @@
 #define SENSORS_IT87_ALARMS_VIN 82 /* R */
 #define SENSORS_IT87_ALARMS_TEMP 83 /* R */
 #define SENSORS_IT87_ALARMS 84 /* R */
+#define SENSORS_IT87_SENS1 91 /* RW */
+#define SENSORS_IT87_SENS2 92 /* RW */
+#define SENSORS_IT87_SENS3 93 /* RW */
 
 
 /* fsc poseidon chip */
@@ -1489,6 +1561,38 @@
 #define SENSORS_FSCSCY_WDOG_STATE   52 /* RW */
 #define SENSORS_FSCSCY_WDOG_CONTROL 52 /* RW */
 
+/* fsc hermes chip */
+
+#define SENSORS_FSCHER_PREFIX "fscher"
+
+#define SENSORS_FSCHER_REV 1 /* R */
+#define SENSORS_FSCHER_EVENT 2 /* R */
+#define SENSORS_FSCHER_CONTROL 3 /* RW */
+#define SENSORS_FSCHER_FAN1 4 /* R */
+#define SENSORS_FSCHER_FAN2 5 /* R */
+#define SENSORS_FSCHER_FAN3 6 /* R */
+#define SENSORS_FSCHER_PWM1 7 /* RW */
+#define SENSORS_FSCHER_PWM2 8 /* RW */
+#define SENSORS_FSCHER_PWM3 9 /* RW */
+#define SENSORS_FSCHER_FAN1_STATE 10 /* RW */
+#define SENSORS_FSCHER_FAN2_STATE 11 /* RW */
+#define SENSORS_FSCHER_FAN3_STATE 12 /* RW */
+#define SENSORS_FSCHER_FAN1_RIPPLE 13 /* RW */
+#define SENSORS_FSCHER_FAN2_RIPPLE 14 /* RW */
+#define SENSORS_FSCHER_FAN3_RIPPLE 15 /* RW */
+#define SENSORS_FSCHER_TEMP1 16 /* R */
+#define SENSORS_FSCHER_TEMP2 17 /* R */
+#define SENSORS_FSCHER_TEMP3 18 /* R */
+#define SENSORS_FSCHER_TEMP1_STATE 19 /* RW */
+#define SENSORS_FSCHER_TEMP2_STATE 20 /* RW */
+#define SENSORS_FSCHER_TEMP3_STATE 21 /* RW */
+#define SENSORS_FSCHER_VOLTAGE1 22 /* R */
+#define SENSORS_FSCHER_VOLTAGE2 23 /* R */
+#define SENSORS_FSCHER_VOLTAGE3 24 /* R */
+#define SENSORS_FSCHER_WDOG_PRESET 25/* RW */
+#define SENSORS_FSCHER_WDOG_STATE 26/* RW */
+#define SENSORS_FSCHER_WDOG_CONTROL 27/* RW */
+
 /* PCF8591 chip. */
 
 #define SENSORS_PCF8591_PREFIX "pcf8591"
@@ -1567,6 +1671,107 @@
 #define SENSORS_SMSC47M1_FAN2_DIV 76 /* RW */
 #define SENSORS_SMSC47M1_ALARMS 81 /* R */
 
+#define SENSORS_PC87360_PREFIX "pc87360"
+#define SENSORS_PC87363_PREFIX "pc87363"
+#define SENSORS_PC87364_PREFIX "pc87364"
+#define SENSORS_PC87365_PREFIX "pc87365"
+#define SENSORS_PC87366_PREFIX "pc87366"
+
+#define SENSORS_PC87360_ALARMS_IN	10	/* R */
+#define SENSORS_PC87360_ALARMS_TEMP	11	/* R */
+
+#define SENSORS_PC87360_FAN1		31	/* R */
+#define SENSORS_PC87360_FAN2		32	/* R */
+#define SENSORS_PC87360_FAN3		33	/* R */
+#define SENSORS_PC87360_FAN1_MIN	41	/* RW */
+#define SENSORS_PC87360_FAN2_MIN	42	/* RW */
+#define SENSORS_PC87360_FAN3_MIN	43	/* RW */
+#define SENSORS_PC87360_FAN1_DIV	71	/* R */
+#define SENSORS_PC87360_FAN2_DIV	72	/* R */
+#define SENSORS_PC87360_FAN3_DIV	73	/* R */
+#define SENSORS_PC87360_FAN1_STATUS	81	/* R */
+#define SENSORS_PC87360_FAN2_STATUS	82	/* R */
+#define SENSORS_PC87360_FAN3_STATUS	83	/* R */
+
+#define SENSORS_PC87360_IN0		90	/* R */
+#define SENSORS_PC87360_IN1		91	/* R */
+#define SENSORS_PC87360_IN2		92	/* R */
+#define SENSORS_PC87360_IN3		93	/* R */
+#define SENSORS_PC87360_IN4		94	/* R */
+#define SENSORS_PC87360_IN5		95	/* R */
+#define SENSORS_PC87360_IN6		96	/* R */
+#define SENSORS_PC87360_IN7		97	/* R */
+#define SENSORS_PC87360_IN8		98	/* R */
+#define SENSORS_PC87360_IN9		99	/* R */
+#define SENSORS_PC87360_IN10		100	/* R */
+#define SENSORS_PC87360_IN0_MIN		110	/* RW */
+#define SENSORS_PC87360_IN1_MIN		111	/* RW */
+#define SENSORS_PC87360_IN2_MIN		112	/* RW */
+#define SENSORS_PC87360_IN3_MIN		113	/* RW */
+#define SENSORS_PC87360_IN4_MIN		114	/* RW */
+#define SENSORS_PC87360_IN5_MIN		115	/* RW */
+#define SENSORS_PC87360_IN6_MIN		116	/* RW */
+#define SENSORS_PC87360_IN7_MIN		117	/* RW */
+#define SENSORS_PC87360_IN8_MIN		118	/* RW */
+#define SENSORS_PC87360_IN9_MIN		119	/* RW */
+#define SENSORS_PC87360_IN10_MIN	120	/* RW */
+#define SENSORS_PC87360_IN0_MAX		130	/* RW */
+#define SENSORS_PC87360_IN1_MAX		131	/* RW */
+#define SENSORS_PC87360_IN2_MAX		132	/* RW */
+#define SENSORS_PC87360_IN3_MAX		133	/* RW */
+#define SENSORS_PC87360_IN4_MAX		134	/* RW */
+#define SENSORS_PC87360_IN5_MAX		135	/* RW */
+#define SENSORS_PC87360_IN6_MAX		136	/* RW */
+#define SENSORS_PC87360_IN7_MAX		137	/* RW */
+#define SENSORS_PC87360_IN8_MAX		138	/* RW */
+#define SENSORS_PC87360_IN9_MAX		139	/* RW */
+#define SENSORS_PC87360_IN10_MAX	140	/* RW */
+#define SENSORS_PC87360_IN0_STATUS	150	/* R */
+#define SENSORS_PC87360_IN1_STATUS	151	/* R */
+#define SENSORS_PC87360_IN2_STATUS	152	/* R */
+#define SENSORS_PC87360_IN3_STATUS	153	/* R */
+#define SENSORS_PC87360_IN4_STATUS	154	/* R */
+#define SENSORS_PC87360_IN5_STATUS	155	/* R */
+#define SENSORS_PC87360_IN6_STATUS	156	/* R */
+#define SENSORS_PC87360_IN7_STATUS	157	/* R */
+#define SENSORS_PC87360_IN8_STATUS	158	/* R */
+#define SENSORS_PC87360_IN9_STATUS	159	/* R */
+#define SENSORS_PC87360_IN10_STATUS	160	/* R */
+
+#define SENSORS_PC87360_TEMP1		171	/* R */
+#define SENSORS_PC87360_TEMP2		172	/* R */
+#define SENSORS_PC87360_TEMP3		173	/* R */
+#define SENSORS_PC87360_TEMP4		174	/* R */
+#define SENSORS_PC87360_TEMP5		175	/* R */
+#define SENSORS_PC87360_TEMP6		176	/* R */
+#define SENSORS_PC87360_TEMP1_MIN	181	/* RW */
+#define SENSORS_PC87360_TEMP2_MIN	182	/* RW */
+#define SENSORS_PC87360_TEMP3_MIN	183	/* RW */
+#define SENSORS_PC87360_TEMP4_MIN	184	/* RW */
+#define SENSORS_PC87360_TEMP5_MIN	185	/* RW */
+#define SENSORS_PC87360_TEMP6_MIN	186	/* RW */
+#define SENSORS_PC87360_TEMP1_MAX	191	/* RW */
+#define SENSORS_PC87360_TEMP2_MAX	192	/* RW */
+#define SENSORS_PC87360_TEMP3_MAX	193	/* RW */
+#define SENSORS_PC87360_TEMP4_MAX	194	/* RW */
+#define SENSORS_PC87360_TEMP5_MAX	195	/* RW */
+#define SENSORS_PC87360_TEMP6_MAX	196	/* RW */
+#define SENSORS_PC87360_TEMP1_CRIT	201	/* RW */
+#define SENSORS_PC87360_TEMP2_CRIT	202	/* RW */
+#define SENSORS_PC87360_TEMP3_CRIT	203	/* RW */
+#define SENSORS_PC87360_TEMP4_CRIT	204	/* RW */
+#define SENSORS_PC87360_TEMP5_CRIT	205	/* RW */
+#define SENSORS_PC87360_TEMP6_CRIT	206	/* RW */
+#define SENSORS_PC87360_TEMP1_STATUS	211	/* R */
+#define SENSORS_PC87360_TEMP2_STATUS	212	/* R */
+#define SENSORS_PC87360_TEMP3_STATUS	213	/* R */
+#define SENSORS_PC87360_TEMP4_STATUS	214	/* R */
+#define SENSORS_PC87360_TEMP5_STATUS	215	/* R */
+#define SENSORS_PC87360_TEMP6_STATUS	216	/* R */
+
+#define SENSORS_PC87360_VID		240	/* R */
+#define SENSORS_PC87360_VRM		241	/* RW */
+
 #define SENSORS_LM92_PREFIX "lm92"
 
 #define SENSORS_LM92_TEMP_HIGH		1	/* RW */
@@ -1648,6 +1853,127 @@
 #define SENSORS_BMC_CURR1_MIN 3101 /* RW */
 #define SENSORS_BMC_CURR1_MAX 3201 /* RW */
 
+#define SENSORS_LM93_PREFIX "lm93"
+
+#define SENSORS_LM93_IN1		1011
+#define SENSORS_LM93_IN1_MIN		1012
+#define SENSORS_LM93_IN1_MAX		1013
+#define SENSORS_LM93_IN2		1021
+#define SENSORS_LM93_IN2_MIN		1022
+#define SENSORS_LM93_IN2_MAX		1023
+#define SENSORS_LM93_IN3		1031
+#define SENSORS_LM93_IN3_MIN		1032
+#define SENSORS_LM93_IN3_MAX		1033
+#define SENSORS_LM93_IN4		1041
+#define SENSORS_LM93_IN4_MIN		1042
+#define SENSORS_LM93_IN4_MAX		1043
+#define SENSORS_LM93_IN5		1051
+#define SENSORS_LM93_IN5_MIN		1052
+#define SENSORS_LM93_IN5_MAX		1053
+#define SENSORS_LM93_IN6		1061
+#define SENSORS_LM93_IN6_MIN		1062
+#define SENSORS_LM93_IN6_MAX		1063
+#define SENSORS_LM93_IN7		1071
+#define SENSORS_LM93_IN7_MIN		1072
+#define SENSORS_LM93_IN7_MAX		1073
+#define SENSORS_LM93_IN8		1081
+#define SENSORS_LM93_IN8_MIN		1082
+#define SENSORS_LM93_IN8_MAX		1083
+#define SENSORS_LM93_IN9		1091
+#define SENSORS_LM93_IN9_MIN		1092
+#define SENSORS_LM93_IN9_MAX		1093
+#define SENSORS_LM93_IN10		1101
+#define SENSORS_LM93_IN10_MIN		1102
+#define SENSORS_LM93_IN10_MAX		1103
+#define SENSORS_LM93_IN11		1111
+#define SENSORS_LM93_IN11_MIN		1112
+#define SENSORS_LM93_IN11_MAX		1113
+#define SENSORS_LM93_IN12		1121
+#define SENSORS_LM93_IN12_MIN		1122
+#define SENSORS_LM93_IN12_MAX		1123
+#define SENSORS_LM93_IN13		1131
+#define SENSORS_LM93_IN13_MIN		1132
+#define SENSORS_LM93_IN13_MAX		1133
+#define SENSORS_LM93_IN14		1141
+#define SENSORS_LM93_IN14_MIN		1142
+#define SENSORS_LM93_IN14_MAX		1143
+#define SENSORS_LM93_IN15		1151
+#define SENSORS_LM93_IN15_MIN		1152
+#define SENSORS_LM93_IN15_MAX		1153
+#define SENSORS_LM93_IN16		1161
+#define SENSORS_LM93_IN16_MIN		1162
+#define SENSORS_LM93_IN16_MAX		1163
+#define SENSORS_LM93_TEMP1		2011
+#define SENSORS_LM93_TEMP1_MIN		2012
+#define SENSORS_LM93_TEMP1_MAX		2013
+#define SENSORS_LM93_TEMP2		2021
+#define SENSORS_LM93_TEMP2_MIN		2022
+#define SENSORS_LM93_TEMP2_MAX		2023
+#define SENSORS_LM93_TEMP3		2031
+#define SENSORS_LM93_TEMP3_MIN		2032
+#define SENSORS_LM93_TEMP3_MAX		2033
+#define SENSORS_LM93_FAN1		3011
+#define SENSORS_LM93_FAN1_MIN		3012
+#define SENSORS_LM93_FAN2		3021
+#define SENSORS_LM93_FAN2_MIN		3022
+#define SENSORS_LM93_FAN3		3031
+#define SENSORS_LM93_FAN3_MIN		3032
+#define SENSORS_LM93_FAN4		3041
+#define SENSORS_LM93_FAN4_MIN		3042
+#define SENSORS_LM93_VID1		4001
+#define SENSORS_LM93_VID2		4002
+#define SENSORS_LM93_ALARMS		5001
+
+#define SENSORS_ASB100_PREFIX "asb100"
+
+#define SENSORS_ASB100_IN0		0x01 /* R */
+#define SENSORS_ASB100_IN1		0x02 /* R */
+#define SENSORS_ASB100_IN2		0x03 /* R */
+#define SENSORS_ASB100_IN3		0x04 /* R */
+#define SENSORS_ASB100_IN4		0x05 /* R */
+#define SENSORS_ASB100_IN5		0x06 /* R */
+#define SENSORS_ASB100_IN6		0x07 /* R */
+#define SENSORS_ASB100_IN0_MIN		0x11 /* RW */
+#define SENSORS_ASB100_IN1_MIN		0x12 /* RW */
+#define SENSORS_ASB100_IN2_MIN		0x13 /* RW */
+#define SENSORS_ASB100_IN3_MIN		0x14 /* RW */
+#define SENSORS_ASB100_IN4_MIN		0x15 /* RW */
+#define SENSORS_ASB100_IN5_MIN		0x16 /* RW */
+#define SENSORS_ASB100_IN6_MIN		0x17 /* RW */
+#define SENSORS_ASB100_IN0_MAX		0x21 /* RW */
+#define SENSORS_ASB100_IN1_MAX		0x22 /* RW */
+#define SENSORS_ASB100_IN2_MAX		0x23 /* RW */
+#define SENSORS_ASB100_IN3_MAX		0x24 /* RW */
+#define SENSORS_ASB100_IN4_MAX		0x25 /* RW */
+#define SENSORS_ASB100_IN5_MAX		0x26 /* RW */
+#define SENSORS_ASB100_IN6_MAX		0x27 /* RW */
+#define SENSORS_ASB100_FAN1		0x31 /* R */
+#define SENSORS_ASB100_FAN2		0x32 /* R */
+#define SENSORS_ASB100_FAN3		0x33 /* R */
+#define SENSORS_ASB100_FAN1_MIN		0x41 /* RW */
+#define SENSORS_ASB100_FAN2_MIN		0x42 /* RW */
+#define SENSORS_ASB100_FAN3_MIN		0x43 /* RW */
+#define SENSORS_ASB100_TEMP1		0x51 /* R */
+#define SENSORS_ASB100_TEMP1_HYST	0x52 /* RW */
+#define SENSORS_ASB100_TEMP1_OVER	0x53 /* RW */
+#define SENSORS_ASB100_TEMP2		0x54 /* R */
+#define SENSORS_ASB100_TEMP2_HYST	0x55 /* RW */
+#define SENSORS_ASB100_TEMP2_OVER	0x56 /* RW */
+#define SENSORS_ASB100_TEMP3		0x57 /* R */
+#define SENSORS_ASB100_TEMP3_HYST	0x58 /* RW */
+#define SENSORS_ASB100_TEMP3_OVER	0x59 /* RW */
+#define SENSORS_ASB100_TEMP4		0x5a /* R */
+#define SENSORS_ASB100_TEMP4_HYST	0x5b /* RW */
+#define SENSORS_ASB100_TEMP4_OVER	0x5c /* RW */
+#define SENSORS_ASB100_VID		0x61 /* R */
+#define SENSORS_ASB100_VRM		0x62 /* RW */
+#define SENSORS_ASB100_FAN1_DIV		0x71 /* RW */
+#define SENSORS_ASB100_FAN2_DIV		0x72 /* RW */
+#define SENSORS_ASB100_FAN3_DIV		0x73 /* RW */
+#define SENSORS_ASB100_ALARMS		0x81 /* R */
+#define SENSORS_ASB100_BEEP_ENABLE	0x82 /* RW */
+#define SENSORS_ASB100_BEEPS		0x83 /* RW */
+
 #define SENSORS_XEONTEMP_PREFIX "xeontemp"
 
 #define SENSORS_XEONTEMP_REMOTE_TEMP 54 /* R */
@@ -1655,6 +1981,18 @@
 #define SENSORS_XEONTEMP_REMOTE_TEMP_OVER 56 /* RW */
 #define SENSORS_XEONTEMP_ALARMS 81 /* R */
 
+/* MAX1619 chip */
+
+#define SENSORS_MAX1619_PREFIX "max1619"
+
+#define SENSORS_MAX1619_LOCAL_TEMP      51      /* R */
+#define SENSORS_MAX1619_REMOTE_TEMP     52      /* R */
+#define SENSORS_MAX1619_REMOTE_LOW      53      /* RW */
+#define SENSORS_MAX1619_REMOTE_HIGH     54      /* RW */
+#define SENSORS_MAX1619_REMOTE_MAX      55      /* RW */
+#define SENSORS_MAX1619_REMOTE_HYST     56      /* RW */
+#define SENSORS_MAX1619_ALARMS          81      /* R */
+
 /* MAX6650 / 1 chips */
 
 #define SENSORS_MAX6650_PREFIX "max6650"
@@ -1665,4 +2003,16 @@
 #define SENSORS_MAX6650_FAN4_TACH  4 /* R */
 #define SENSORS_MAX6650_SPEED      5 /* RW */
 
+/* SMSC47B397-NC chip */
+#define SENSORS_SMSC47B397_PREFIX "smsc47b397"
+
+#define SENSORS_SMSC47B397_TEMP1	0x01 /* R */
+#define SENSORS_SMSC47B397_TEMP2	0x02 /* R */
+#define SENSORS_SMSC47B397_TEMP3	0x03 /* R */
+#define SENSORS_SMSC47B397_TEMP4	0x04 /* R */
+#define SENSORS_SMSC47B397_FAN1		0x11 /* R */
+#define SENSORS_SMSC47B397_FAN2		0x12 /* R */
+#define SENSORS_SMSC47B397_FAN3		0x13 /* R */
+#define SENSORS_SMSC47B397_FAN4		0x14 /* R */
+
 #endif /* def LIB_SENSORS_CHIPS_H */
