@@ -32,27 +32,30 @@ $temp = "mkpatch/.temp";
 sub print_diff
 {
   my ($package_root,$kernel_root,$kernel_file,$package_file) = @_;
-  my ($diff_command,$dummy);
+  my ($diff_command,$package_mtime,$kernel_mtime);
 
-  $diff_command = "diff -u2";
+  $diff_command = "diff -u";
   if ( -e "$kernel_root/$kernel_file") {
-    $diff_command .= " $kernel_root/$kernel_file ";
-  } else {
-    $diff_command .= " /dev/null ";
-  }
-  if ( -e "$package_root/$package_file") {
-    $diff_command .= " $package_root/$package_file ";
+    $diff_command .= " $kernel_root/$kernel_file";
+    $kernel_mtime = (stat("$kernel_root/$kernel_file"))[9];
   } else {
     $diff_command .= " /dev/null";
+    $kernel_mtime = 0;
+  }
+  if ( -e "$package_root/$package_file") {
+    $diff_command .= " $package_root/$package_file";
+    $package_mtime = (stat("$package_root/$package_file"))[9];
+  } else {
+    $diff_command .= " /dev/null";
+    $package_mtime = 0;
   }
   open INPUT, "$diff_command|" or die "Can't execute `$diff_command'";
-  $dummy = <INPUT>;
-  $dummy = <INPUT>;
-  print "--- linux-old/$kernel_file\t".`date`;
-  print "+++ linux/$kernel_file\t".`date`;
-    
-  while (<INPUT>) {
-    print;
+  if (<INPUT>) {
+    <INPUT>;
+    print "--- linux-old/$kernel_file\t".gmtime($kernel_mtime)."\n".
+          "+++ linux/$kernel_file\t".gmtime($package_mtime)."\n";
+
+    print while <INPUT>;
   }
   close INPUT;
 }
@@ -80,11 +83,13 @@ sub gen_Documentation_Configure_help
            m@Acer Labs ALI 1535@ or
            m@Acer Labs ALI 1533 and 1543C@ or
            m@AMD 756/766/768/8111@ or
+           m@SMBus multiplexing on the Tyan S4882@ or
            m@AMD 8111 SMBus 2.0@ or
            m@Apple Hydra Mac I/O@ or
            m@Intel I801@ or
            m@Intel I810/I815 based Mainboard@ or
            m@Intel 82371AB PIIX4\(E\)@ or
+           m@Nvidia Nforce2@ or
            m@Silicon Integrated Systems Corp. SiS5595 based Mainboard@ or
            m@VIA Technologies, Inc. VT82C586B@ or
            m@VIA Technologies, Inc. VT82C596, 596B, 686A/B, 8233@ or
@@ -96,23 +101,32 @@ sub gen_Documentation_Configure_help
            m@Analog Devices ADM1025@ or
            m@Analog Devices ADM1026@ or
            m@Analog Devices ADM9240 and compatibles@ or
+           m@Asus ASB100@ or
            m@Dallas DS1621 and DS1625@ or
            m@Fujitsu-Siemens Poseidon@ or
            m@Fujitsu-Siemens Scylla@ or
            m@Genesys Logic GL518SM@ or
            m@Genesys Logic GL520SM@ or
            m@HP Maxilife@ or
+           m@Intel Xeon Thermal Sensor@ or
            m@ITE 8705, 8712, Sis950@ or
+           m@Maxim MAX6650, MAX6651@ or
            m@Myson MTP008@ or
-           m@National Semiconductors LM75 and compatibles@ or
-           m@National Semiconductors LM78@ or
-           m@National Semiconductors LM80@ or
-           m@National Semiconductors LM85 and compatibles@ or
-           m@National Semiconductors LM87@ or
+           m@National Semiconductor LM75 and compatibles@ or
+           m@National Semiconductor LM78@ or
+           m@National Semiconductor LM80@ or
+           m@National Semiconductor LM83@ or
+           m@National Semiconductor LM85@ or
+           m@National Semiconductor LM87@ or
+           m@National Semiconductor LM90@ or
+           m@National Semiconductor LM92@ or
+           m@National Semiconductor PC8736x@ or
            m@Silicon Integrated Systems Corp. SiS5595 Sensor@ or
            m@Texas Instruments THMC50 / Analog Devices ADM1022@ or
            m@Via VT82C686A/B@ or
            m@Winbond W83781D, W83782D, W83783S, W83627HF, AS99127F@ or
+           m@Winbond W83627HF, W83627THF, W83697HF@ or
+           m@Winbond W83L785TS-S@ or
            m@EEprom \(DIMM\) reader@) {
       $_ = <INPUT>;
       $_ = <INPUT>;
@@ -145,12 +159,20 @@ CONFIG_I2C_ALI15X3
   built as a module which can be inserted and removed while the kernel
   is running.
 
-AMD 756/766/768/8111
+AMD 756/766/768/8111 and nVidia nForce
 CONFIG_I2C_AMD756
   If you say yes to this option, support will be included for the AMD
-  756/766/768/8111 mainboard I2C interfaces. This can also be 
-  built as a module which can be inserted and removed while the kernel
-  is running.
+  756/766/768/8111 and nVidia nForce mainboard I2C interfaces. This can
+  also be built as a module which can be inserted and removed while the
+  kernel is running.
+
+SMBus multiplexing on the Tyan S4882
+CONFIG_I2C_AMD756_S4882
+  Enabling this option will add specific SMBus support for the Tyan
+  S4882 motherboard. On this 4-CPU board, the SMBus is multiplexed
+  over 8 different channels, where the various memory module EEPROMs
+  and temperature sensors live. Saying yes here will give you access
+  to these in addition to the trunk.
 
 AMD 8111 SMBus 2.0
 CONFIG_I2C_AMD8111
@@ -191,6 +213,13 @@ CONFIG_I2C_PIIX4
   built as a module which can be inserted and removed while the kernel
   is running.
 
+Nvidia Nforce2/Nforce3 based Mainboard
+CONFIG_I2C_NFORCE2
+  If you say yes to this option, support will be included for the 
+  Nvidia Nforce2 and Nforce3 families of mainboard I2C interfaces.
+  This can also be built as a module which can be inserted and removed
+  while the kernel is running.
+
 Silicon Integrated Systems Corp. SiS5595 based Mainboard
 CONFIG_I2C_SIS5595
   If you say yes to this option, support will be included for the 
@@ -254,13 +283,12 @@ CONFIG_I2C_ISA
   http://www.lm-sensors.nu
 
 Analog Devices ADM1021 and compatibles
-CONFIG_SENSORS_ADM1021 
+CONFIG_SENSORS_ADM1021
   If you say yes here you get support for Analog Devices ADM1021 
   and ADM1023 sensor chips and clones: Maxim MAX1617 and MAX1617A,
-  Genesys Logic GL523SM, National Semi LM84, TI THMC10, Onsemi MC1066
-  and the XEON processor built-in sensor. This can also 
-  be built as a module which can be inserted and removed while the 
-  kernel is running.
+  Genesys Logic GL523SM, National Semi LM84, TI THMC10 and Onsemi
+  MC1066. This can also be built as a module which can be inserted
+  and removed while the kernel is running.
 
   You will also need the latest user-space utilties: you can find them
   in the lm_sensors package, which you can download at 
@@ -307,6 +335,15 @@ CONFIG_SENSORS_ADM9240
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+Asus ASB100
+CONFIG_SENSORS_ASB100
+  If you say yes here you get support for the Asus ASB100 (aka
+  "Bach") sensor chip.  This can also be built as a module.
+
+  You will also need the latest user-space utilities: you can find
+  them in the lm_sensors package, which you can download at
+  http://www.lm-sensors.nu/
+
 Dallas DS1621 and DS1625
 CONFIG_SENSORS_DS1621
   If you say yes here you get support for the Dallas DS1621 and DS1625x
@@ -367,6 +404,16 @@ CONFIG_SENSORS_MAXILIFE
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+Intel Xeon Thermal Sensor
+CONFIG_SENSORS_XEONTEMP
+  If you say yes here you get support for the Intel Xeon processor
+  built-in thermal sensor. This can also be built as a module which
+  can be inserted and removed while the kernel is running.
+
+  You will also need the latest user-space utilities: you can find them
+  in the lm_sensors package, which you can download at 
+  http://www.lm-sensors.nu/
+
 ITE 8705, 8712, Sis950
 CONFIG_SENSORS_IT87
   If you say yes here you get support for the ITE 8705 and 8712 and
@@ -376,6 +423,15 @@ CONFIG_SENSORS_IT87
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+Maxim MAX6650, MAX6651
+CONFIG_SENSORS_MAX6650
+  If you say yes here you get support for the Maxim MAX6650 and
+  MAX6651 sensor chips.  This can also be built as a module.
+
+  You will also need the latest user-space utilties: you can find them
+  in the lm_sensors package, which you can download at 
+  http://www.lm-sensors.nu
+
 Myson MTP008
 CONFIG_SENSORS_MTP008
   If you say yes here you get support for the Myson MTP008
@@ -388,9 +444,10 @@ CONFIG_SENSORS_MTP008
 National Semiconductor LM75 and compatibles
 CONFIG_SENSORS_LM75 
   If you say yes here you get support for National Semiconductor LM75
-  sensor chips and clones: Dallas Semi DS75 and DS1775, TelCon
-  TCN75, and National Semi LM77. This can also be built as a module which
-  can be inserted and removed while the kernel is running.
+  sensor chips and clones: Dallas Semiconductor DS75 and DS1775 (in
+  9-bit precision mode), and TelCom (now Microchip) TCN75. This can
+  also be built as a module which can be inserted and removed while
+  the kernel is running.
 
   You will also need the latest user-space utilties: you can find them
   in the lm_sensors package, which you can download at 
@@ -418,6 +475,15 @@ CONFIG_SENSORS_LM80
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+National Semiconductor LM83
+CONFIG_SENSORS_LM83
+  If you say yes here you get support for the National Semiconductor
+  LM83 sensor chip.  This can also be built as a module.
+
+  You will also need the latest user-space utilities: you can find
+  them in the lm_sensors package, which you can download at
+  http://www.lm-sensors.nu/
+
 National Semiconductor LM85
 CONFIG_SENSORS_LM85
   If you say yes here you get support for National Semiconductor LM85
@@ -440,6 +506,16 @@ CONFIG_SENSORS_LM87
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+National Semiconductor LM90
+CONFIG_SENSORS_LM90
+  If you say yes here you get support for the National Semiconductor
+  LM90, LM89 and LM99, and Analog Devices ADM1032 sensor chips.  This
+  can also be built as a module.
+
+  You will also need the latest user-space utilities: you can find
+  them in the lm_sensors package, which you can download at
+  http://www.lm-sensors.nu/
+
 National Semiconductor LM92
 CONFIG_SENSORS_LM92
   If you say yes here you get support for National Semiconductor LM92
@@ -450,6 +526,18 @@ CONFIG_SENSORS_LM92
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+National Semiconductor PC8736x Sensors
+CONFIG_SENSORS_PC87360
+  If you say yes here you get support for the integrated hardware
+  monitoring in the National Semicoductor PC87360, PC87363, PC87364,
+  PC87365 and PC87366 Super I/O chips. This can also be built as a
+  module which can be inserted and removed while the kernel is
+  running.
+
+  You will also need the latest user-space utilities: you can find them
+  in the lm_sensors package, which you can download at 
+  http://www.lm-sensors.nu/
+
 Philips PCF8574
 CONFIG_SENSORS_PCF8574
   If you say yes here you get support for the Philips PCF8574
@@ -546,6 +634,26 @@ CONFIG_SENSORS_W83781D
   in the lm_sensors package, which you can download at 
   http://www.lm-sensors.nu
 
+Winbond W83627HF, W83627THF, W83697HF
+CONFIG_SENSORS_W83627HF
+  If you say yes here you get support for the Winbond W836x7 series 
+  of sensor chips: the Winbond W83627HF, W83627THF and W83697HF. This
+  can also be built as a module which can be inserted and removed
+  while the kernel is running.
+
+  You will also need the latest user-space utilities: you can find
+  them in the lm_sensors package, which you can download at
+  http://www.lm-sensors.nu/
+
+Winbond W83L785TS-S
+CONFIG_SENSORS_W83L785TS
+  If you say yes here you get support for the Winbond W83L785TS-S
+  sensor chip.  This can also be built as a module.
+
+  You will also need the latest user-space utilities: you can find
+  them in the lm_sensors package, which you can download at
+  http://www.lm-sensors.nu/
+
 EEprom (DIMM) reader
 CONFIG_SENSORS_EEPROM
   If you say yes here you get read-only access to the EEPROM data 
@@ -776,7 +884,10 @@ sub gen_drivers_i2c_Config_in
     dep_tristate '  Acer Labs ALI 1535' CONFIG_I2C_ALI1535 $CONFIG_I2C
     dep_tristate '  Acer Labs ALI 1533 and 1543C' CONFIG_I2C_ALI15X3 $CONFIG_I2C
     dep_tristate '  Apple Hydra Mac I/O' CONFIG_I2C_HYDRA $CONFIG_I2C_ALGOBIT
-    dep_tristate '  AMD 756/766/768/8111' CONFIG_I2C_AMD756 $CONFIG_I2C
+    dep_tristate '  AMD 756/766/768/8111 and nVidia nForce' CONFIG_I2C_AMD756 $CONFIG_I2C
+    if [ "$CONFIG_I2C_AMD756" != "n" ]; then
+      dep_mbool '  SMBus multiplexing on the Tyan S4882' CONFIG_I2C_AMD756_S4882 $CONFIG_I2C_AMD756
+    fi
     dep_tristate '  AMD 8111 SMBus 2.0' CONFIG_I2C_AMD8111 $CONFIG_I2C
     if [ "$CONFIG_ALPHA" = "y" ]; then
       dep_tristate '  DEC Tsunami I2C interface' CONFIG_I2C_TSUNAMI $CONFIG_I2C_ALGOBIT
@@ -784,6 +895,7 @@ sub gen_drivers_i2c_Config_in
     dep_tristate '  Intel 82801AA, AB, BA, DB' CONFIG_I2C_I801 $CONFIG_I2C
     dep_tristate '  Intel i810AA/AB/E and i815' CONFIG_I2C_I810 $CONFIG_I2C_ALGOBIT
     dep_tristate '  Intel 82371AB PIIX4(E), 443MX, ServerWorks OSB4/CSB5, SMSC Victory66' CONFIG_I2C_PIIX4 $CONFIG_I2C
+    dep_tristate '  Nvidia Nforce2/Nforce3' CONFIG_I2C_NFORCE2 $CONFIG_I2C
     dep_tristate '  SiS 5595' CONFIG_I2C_SIS5595 $CONFIG_I2C
     dep_tristate '  SiS 630/730' CONFIG_I2C_SIS630 $CONFIG_I2C
     dep_tristate '  SiS 645/961,645DX/961,735' CONFIG_I2C_SIS645 $CONFIG_I2C $CONFIG_HOTPLUG
@@ -810,14 +922,10 @@ sub gen_drivers_sensors_Makefile
   my ($package_root,$kernel_root) = @_;
   my $kernel_file = "drivers/sensors/Makefile";
   my $package_file = $temp;
-  my $use_new_format;
-  `grep -q -s 'i2c\.o' "$kernel_root/drivers/i2c/Makefile"`;
-  $use_new_format = ! $?;
 
   open OUTPUT,">$package_root/$package_file"
         or die "Can't open $package_root/$package_file";
-  if ($use_new_format) {
-    print OUTPUT <<'EOF';
+  print OUTPUT <<'EOF';
 #
 # Makefile for the kernel hardware sensors drivers.
 #
@@ -830,6 +938,7 @@ obj-$(CONFIG_SENSORS_ADM1024)	+= adm1024.o
 obj-$(CONFIG_SENSORS_ADM1025)	+= adm1025.o
 obj-$(CONFIG_SENSORS_ADM1026)	+= adm1026.o
 obj-$(CONFIG_SENSORS_ADM9240)	+= adm9240.o
+obj-$(CONFIG_SENSORS_ASB100)	+= asb100.o
 obj-$(CONFIG_SENSORS_BT869)	+= bt869.o
 obj-$(CONFIG_SENSORS_DDCMON)	+= ddcmon.o
 obj-$(CONFIG_SENSORS_DS1621)	+= ds1621.o
@@ -842,11 +951,15 @@ obj-$(CONFIG_SENSORS_IT87)	+= it87.o
 obj-$(CONFIG_SENSORS_LM75)	+= lm75.o
 obj-$(CONFIG_SENSORS_LM78)	+= lm78.o
 obj-$(CONFIG_SENSORS_LM80)	+= lm80.o
+obj-$(CONFIG_SENSORS_LM83)	+= lm83.o
 obj-$(CONFIG_SENSORS_LM85)	+= lm85.o
 obj-$(CONFIG_SENSORS_LM87)	+= lm87.o
+obj-$(CONFIG_SENSORS_LM90)	+= lm90.o
 obj-$(CONFIG_SENSORS_LM92)	+= lm92.o
+obj-$(CONFIG_SENSORS_MAX6650)	+= max6650.o
 obj-$(CONFIG_SENSORS_MAXILIFE)	+= maxilife.o
 obj-$(CONFIG_SENSORS_MTP008)	+= mtp008.o
+obj-$(CONFIG_SENSORS_PC87360)	+= pc87360.o
 obj-$(CONFIG_SENSORS_PCF8574)	+= pcf8574.o
 obj-$(CONFIG_SENSORS_PCF8591)	+= pcf8591.o
 obj-$(CONFIG_SENSORS_SIS5595)	+= sis5595.o
@@ -856,290 +969,13 @@ obj-$(CONFIG_SENSORS_VIA686A)	+= via686a.o
 obj-$(CONFIG_SENSORS_VT1211)	+= vt1211.o
 obj-$(CONFIG_SENSORS_VT8231)	+= vt8231.o
 obj-$(CONFIG_SENSORS_W83781D)	+= w83781d.o
+obj-$(CONFIG_SENSORS_W83627HF)	+= w83627hf.o
+obj-$(CONFIG_SENSORS_W83L785TS)	+= w83l785ts.o
+obj-$(CONFIG_SENSORS_XEONTEMP)	+= xeontemp.o
 
 include $(TOPDIR)/Rules.make
 
 EOF
-  } else {
-    print OUTPUT <<'EOF';
-#
-# Makefile for the kernel hardware sensors drivers.
-#
-
-SUB_DIRS     :=
-MOD_SUB_DIRS := $(SUB_DIRS)
-ALL_SUB_DIRS := $(SUB_DIRS)
-MOD_LIST_NAME := SENSORS_MODULES
-
-L_TARGET := sensors.a
-MX_OBJS :=  
-M_OBJS  := 
-LX_OBJS :=
-L_OBJS  := 
-
-# -----
-# i2c core components
-# -----
-
-ifeq ($(CONFIG_SENSORS),y)
-  LX_OBJS += sensors.o
-else
-  ifeq ($(CONFIG_SENSORS),m)
-    MX_OBJS += sensors.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_ADM1021),y)
-  L_OBJS += adm1021.o
-else
-  ifeq ($(CONFIG_SENSORS_ADM1021),m)
-    M_OBJS += adm1021.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_ADM1024),y)
-  L_OBJS += adm1024.o
-else
-  ifeq ($(CONFIG_SENSORS_ADM1024),m)
-    M_OBJS += adm1024.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_ADM1025),y)
-  L_OBJS += adm1025.o
-else
-  ifeq ($(CONFIG_SENSORS_ADM1025),m)
-    M_OBJS += adm1025.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_ADM1026),y)
-  L_OBJS += adm1026.o
-else
-  ifeq ($(CONFIG_SENSORS_ADM1026),m)
-    M_OBJS += adm1026.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_ADM9240),y)
-  L_OBJS += adm9240.o
-else
-  ifeq ($(CONFIG_SENSORS_ADM9240),m)
-    M_OBJS += adm9240.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_DDCMON),y)
-  L_OBJS += ddcmon.o
-else
-  ifeq ($(CONFIG_SENSORS_DDCMON),m)
-    M_OBJS += ddcmon.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_DS1621),y)
-  L_OBJS += ds1621.o
-else
-  ifeq ($(CONFIG_SENSORS_DS1621),m)
-    M_OBJS += ds1621.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_EEPROM),y)
-  L_OBJS += eeprom.o
-else
-  ifeq ($(CONFIG_SENSORS_EEPROM),m)
-    M_OBJS += eeprom.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_FSCPOS),y)
-  L_OBJS += fscpos.o
-else
-  ifeq ($(CONFIG_SENSORS_FSCPOS),m)
-    M_OBJS += fscpos.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_FSCSCY),y)
-  L_OBJS += fscscy.o
-else
-  ifeq ($(CONFIG_SENSORS_FSCSCY),m)
-    M_OBJS += fscscy.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_GL518SM),y)
-  L_OBJS += gl518sm.o
-else
-  ifeq ($(CONFIG_SENSORS_GL518SM),m)
-    M_OBJS += gl518sm.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_GL520SM),y)
-  L_OBJS += gl520sm.o
-else
-  ifeq ($(CONFIG_SENSORS_GL520SM),m)
-    M_OBJS += gl520sm.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_IT87),y)
-  L_OBJS += it87.o
-else
-  ifeq ($(CONFIG_SENSORS_IT87),m)
-    M_OBJS += it87.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM75),y)
-  L_OBJS += lm75.o
-else
-  ifeq ($(CONFIG_SENSORS_LM75),m)
-    M_OBJS += lm75.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM78),y)
-  L_OBJS += lm78.o
-else
-  ifeq ($(CONFIG_SENSORS_LM78),m)
-    M_OBJS += lm78.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM80),y)
-  L_OBJS += lm80.o
-else
-  ifeq ($(CONFIG_SENSORS_LM80),m)
-    M_OBJS += lm80.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM85),y)
-  L_OBJS += lm85.o
-else
-  ifeq ($(CONFIG_SENSORS_LM85),m)
-    M_OBJS += lm85.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM87),y)
-  L_OBJS += lm87.o
-else
-  ifeq ($(CONFIG_SENSORS_LM87),m)
-    M_OBJS += lm87.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_LM92),y)
-  L_OBJS += lm92.o
-else
-  ifeq ($(CONFIG_SENSORS_LM92),m)
-    M_OBJS += lm92.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_MATORB),y)
-  L_OBJS += matorb.o
-else
-  ifeq ($(CONFIG_SENSORS_MATORB),m)
-    M_OBJS += matorb.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_MAXILIFE),y)
-  L_OBJS += maxilife.o
-else
-  ifeq ($(CONFIG_SENSORS_MAXILIFE),m)
-    M_OBJS += maxilife.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_MTP008),y)
-  L_OBJS += mtp008.o
-else
-  ifeq ($(CONFIG_SENSORS_MTP008),m)
-    M_OBJS += mtp008.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_PCF8574),y)
-  L_OBJS += pcf8574.o
-else
-  ifeq ($(CONFIG_SENSORS_PCF8574),m)
-    M_OBJS += pcf8574.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_PCF8591),y)
-  L_OBJS += pcf8591.o
-else
-  ifeq ($(CONFIG_SENSORS_PCF8591),m)
-    M_OBJS += pcf8591.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_SIS5595),y)
-  L_OBJS += sis5595.o
-else
-  ifeq ($(CONFIG_SENSORS_SIS5595),m)
-    M_OBJS += sis5595.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_SMSC47M1),y)
-  L_OBJS += smsc47m1.o
-else
-  ifeq ($(CONFIG_SENSORS_SMSC47M1),m)
-    M_OBJS += smsc47m1.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_THMC50),y)
-  L_OBJS += thmc50.o
-else
-  ifeq ($(CONFIG_SENSORS_THMC50),m)
-    M_OBJS += thmc50.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_VIA686A),y)
-  L_OBJS += via686a.o
-else
-  ifeq ($(CONFIG_SENSORS_VIA686A),m)
-    M_OBJS += via686a.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_VT1211),y)
-  L_OBJS += vt1211.o
-else
-  ifeq ($(CONFIG_SENSORS_VT1211),m)
-    M_OBJS += vt1211.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_VT8211),y)
-  L_OBJS += vt8231.o
-else
-  ifeq ($(CONFIG_SENSORS_VT8231),m)
-    M_OBJS += vt8231.o
-  endif
-endif
-
-ifeq ($(CONFIG_SENSORS_W83781D),y)
-  L_OBJS += w83781d.o
-else
-  ifeq ($(CONFIG_SENSORS_W83781D),m)
-    M_OBJS += w83781d.o
-  endif
-endif
-
-include $(TOPDIR)/Rules.make
-EOF
-  }  
   close OUTPUT;
   print_diff $package_root,$kernel_root,$kernel_file,$package_file;
 }
@@ -1156,22 +992,19 @@ sub gen_drivers_i2c_Makefile
   my $kernel_file = "drivers/i2c/Makefile";
   my $package_file = $temp;
   my $pr1 = 0;
-  my $new_format = 0;
 
   open INPUT,"$kernel_root/$kernel_file"
         or die "Can't open `$kernel_root/$kernel_file'";
   open OUTPUT,">$package_root/$package_file"
         or die "Can't open $package_root/$package_file";
   while(<INPUT>) {
-    $new_format = 1 if m@i2c\.o@;
     if (m@sensors code starts here@) {
       $pr1 ++;
       print OUTPUT;
       while (<INPUT>) {
         last if m@sensors code ends here@;
       }
-      if ($new_format) {
-        print OUTPUT << 'EOF';
+      print OUTPUT <<'EOF';
 obj-$(CONFIG_I2C_ALI1535)		+= i2c-ali1535.o
 obj-$(CONFIG_I2C_ALI15X3)		+= i2c-ali15x3.o
 obj-$(CONFIG_I2C_AMD756)		+= i2c-amd756.o
@@ -1180,7 +1013,8 @@ obj-$(CONFIG_I2C_HYDRA)			+= i2c-hydra.o
 obj-$(CONFIG_I2C_I801)			+= i2c-i801.o
 obj-$(CONFIG_I2C_I810)			+= i2c-i810.o
 obj-$(CONFIG_I2C_ISA)			+= i2c-isa.o
-obj-$(CONFIG_I2C_PIIX4)			+= i2c-piix4.o dmi_scan.o
+obj-$(CONFIG_I2C_NFORCE2)		+= i2c-nforce2.o
+obj-$(CONFIG_I2C_PIIX4)			+= i2c-piix4.o
 obj-$(CONFIG_I2C_SIS5595)		+= i2c-sis5595.o
 obj-$(CONFIG_I2C_SIS630)		+= i2c-sis630.o
 obj-$(CONFIG_I2C_SIS645)		+= i2c-sis645.o
@@ -1189,148 +1023,7 @@ obj-$(CONFIG_I2C_TSUNAMI)		+= i2c-tsunami.o
 obj-$(CONFIG_I2C_VIA)			+= i2c-via.o
 obj-$(CONFIG_I2C_VIAPRO)		+= i2c-viapro.o
 obj-$(CONFIG_I2C_VOODOO3)		+= i2c-voodoo3.o
-export-objs				+= dmi_scan.o
 EOF
-      } else {
-        print OUTPUT << 'EOF';
-ifeq ($(CONFIG_I2C_ALI1535),y)
-  L_OBJS += i2c-ali1535.o
-else 
-  ifeq ($(CONFIG_I2C_ALI1535),m)
-    M_OBJS += i2c-ali1535.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_ALI15X3),y)
-  L_OBJS += i2c-ali15x3.o
-else 
-  ifeq ($(CONFIG_I2C_ALI15X3),m)
-    M_OBJS += i2c-ali15x3.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_AMD756),y)
-  L_OBJS += i2c-amd756.o
-else 
-  ifeq ($(CONFIG_I2C_AMD756),m)
-    M_OBJS += i2c-amd756.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_AMD8111),y)
-  L_OBJS += i2c-amd8111.o
-else 
-  ifeq ($(CONFIG_I2C_AMD8111),m)
-    M_OBJS += i2c-amd8111.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_HYDRA),y)
-  L_OBJS += i2c-hydra.o
-else 
-  ifeq ($(CONFIG_I2C_HYDRA),m)
-    M_OBJS += i2c-hydra.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_I801),y)
-  L_OBJS += i2c-i801.o
-else 
-  ifeq ($(CONFIG_I2C_I801),m)
-    M_OBJS += i2c-i801.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_I810),y)
-  L_OBJS += i2c-i810.o
-else 
-  ifeq ($(CONFIG_I2C_I810),m)
-    M_OBJS += i2c-i810.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_ISA),y)
-  L_OBJS += i2c-isa.o
-else 
-  ifeq ($(CONFIG_I2C_ISA),m)
-    M_OBJS += i2c-isa.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_PIIX4),y)
-  L_OBJS += i2c-piix4.o dmi_scan.o
-else 
-  ifeq ($(CONFIG_I2C_PIIX4),m)
-    M_OBJS += i2c-piix4.o dmi_scan.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_SIS5595),y)
-  L_OBJS += i2c-sis5595.o
-else 
-  ifeq ($(CONFIG_I2C_SIS5595),m)
-    M_OBJS += i2c-sis5595.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_SIS630),y)
-  L_OBJS += i2c-sis630.o
-else 
-  ifeq ($(CONFIG_I2C_SIS630),m)
-    M_OBJS += i2c-sis630.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_SIS645),y)
-  L_OBJS += i2c-sis645.o
-else 
-  ifeq ($(CONFIG_I2C_SIS645),m)
-    M_OBJS += i2c-sis645.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_SAVAGE4),y)
-  L_OBJS += i2c-savage4.o
-else 
-  ifeq ($(CONFIG_I2C_SAVAGE4),m)
-    M_OBJS += i2c-SAVAGE4.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_TSUNAMI),y)
-  L_OBJS += i2c-tsunami.o
-else 
-  ifeq ($(CONFIG_I2C_TSUNAMI),m)
-    M_OBJS += i2c-tsunami.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_VIA),y)
-  L_OBJS += i2c-via.o
-else 
-  ifeq ($(CONFIG_I2C_VIA),m)
-    M_OBJS += i2c-via.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_VIAPRO),y)
-  L_OBJS += i2c-viapro.o
-else 
-  ifeq ($(CONFIG_I2C_VIAPRO),m)
-    M_OBJS += i2c-viapro.o
-  endif
-endif
-
-ifeq ($(CONFIG_I2C_VOODOO3),y)
-  L_OBJS += i2c-voodoo3.o
-else 
-  ifeq ($(CONFIG_I2C_VOODOO3),m)
-    M_OBJS += i2c-voodoo3.o
-  endif
-endif
-
-EOF
-      }
     }
     print OUTPUT;
   }
@@ -1364,10 +1057,6 @@ sub gen_MAINTAINERS
     if (not $done and (m@SGI VISUAL WORKSTATION 320 AND 540@)) {
       print OUTPUT <<'EOF';
 SENSORS DRIVERS
-P:      Frodo Looijaard
-M:      frodol@dds.nl
-P:      Philip Edelbrock
-M:      phil@netroedge.com
 L:      sensors@stimpy.netroedge.com
 W:      http://www.lm-sensors.nu/
 S:      Maintained
@@ -1384,6 +1073,87 @@ EOF
   print_diff $package_root,$kernel_root,$kernel_file,$package_file;
 }
 
+# Generate the diffs for dmi_scan.c and i386_ksyms.c
+# $_[0]: i2c package root (like /tmp/i2c)
+# $_[1]: Linux kernel tree (like /usr/src/linux)
+sub gen_dmi_scan
+{
+  my ($package_root,$kernel_root) = @_;
+  my $kernel_file = "arch/i386/kernel/dmi_scan.c";
+  my $package_file = $temp;
+  my $done = 0;
+
+  open INPUT,"$kernel_root/$kernel_file"
+        or die "Can't open `$kernel_root/$kernel_file'";
+  open OUTPUT,">$package_root/$package_file"
+        or die "Can't open $package_root/$package_file";
+  MAIN: while(<INPUT>) {
+    if ($done == 0 && m/^\s*int is_sony_vaio_laptop;\s*$/) {
+      print OUTPUT <<'EOF';
+int is_unsafe_smbus;
+EOF
+      $done++;
+    }
+    if ($done == 1 && m/^\s*\* Check for a Sony Vaio system\s*$/) {
+      print OUTPUT <<'EOF';
+ * Don't access SMBus on IBM systems which get corrupted eeproms 
+ */
+
+static __init int disable_smbus(struct dmi_blacklist *d)
+{   
+	if (is_unsafe_smbus == 0) {
+		is_unsafe_smbus = 1;
+		printk(KERN_INFO "%s machine detected. Disabling SMBus accesses.\n", d->ident);
+	}
+	return 0;
+}
+
+/*
+EOF
+      $done++;
+    }
+    if ($done == 2 && m/^\s*\{ sony_vaio_laptop, "Sony Vaio", \{ \/\* This is a Sony Vaio laptop \*\/\s*$/) {
+      print OUTPUT <<'EOF';
+	{ disable_smbus, "IBM", {
+			MATCH(DMI_SYS_VENDOR, "IBM"),
+			NO_MATCH, NO_MATCH, NO_MATCH
+			} },
+EOF
+      $done++;
+    }
+    print OUTPUT;
+  }
+  close INPUT;
+  close OUTPUT;
+  die "Automatic patch generation for `$kernel_file' failed.\n".
+      "See our home page http://www.lm-sensors.nu for assistance!" if $done != 3;
+  print_diff $package_root,$kernel_root,$kernel_file,$package_file;
+
+  $kernel_file = "arch/i386/kernel/i386_ksyms.c";
+  $done = 0;
+
+  open INPUT,"$kernel_root/$kernel_file"
+        or die "Can't open `$kernel_root/$kernel_file'";
+  open OUTPUT,">$package_root/$package_file"
+        or die "Can't open $package_root/$package_file";
+  MAIN: while(<INPUT>) {
+    if ($done == 0 && m/^\s*extern int is_sony_vaio_laptop;\s*$/) {
+      print OUTPUT <<'EOF';
+extern int is_unsafe_smbus;
+EXPORT_SYMBOL(is_unsafe_smbus);
+
+EOF
+      $done++;
+    }
+    print OUTPUT;
+  }
+  close INPUT;
+  close OUTPUT;
+  die "Automatic patch generation for `$kernel_file' failed.\n".
+      "See our home page http://www.lm-sensors.nu for assistance!" if $done != 1;
+  print_diff $package_root,$kernel_root,$kernel_file,$package_file;
+}
+
 
 # Main function
 sub main
@@ -1456,6 +1226,7 @@ sub main
   gen_drivers_i2c_Makefile $package_root, $kernel_root;
   gen_Documentation_Configure_help $package_root, $kernel_root;
   gen_MAINTAINERS $package_root, $kernel_root;
+  gen_dmi_scan $package_root, $kernel_root;
 }
 
 main;
